@@ -6,7 +6,7 @@
 # the MIT License: http://www.opensource.org/licenses/mit-license.php
 
 """Define result set constructs including :class:`.ResultProxy`
-and :class:`.RowProxy."""
+and :class:`.RowProxy`."""
 
 
 import collections
@@ -18,6 +18,7 @@ from ..sql import expression
 from ..sql import sqltypes
 from ..sql import util as sql_util
 
+
 # This reconstructor is necessary so that pickles with the C extension or
 # without use the same Binary format.
 try:
@@ -502,7 +503,6 @@ class ResultMetaData(object):
     def _merge_textual_cols_by_position(
         self, context, cursor_description, result_columns
     ):
-        dialect = context.dialect
         num_ctx_cols = len(result_columns) if result_columns else None
 
         if num_ctx_cols > len(cursor_description):
@@ -511,7 +511,6 @@ class ResultMetaData(object):
                 "smaller than number of columns requested (%d)"
                 % (num_ctx_cols, len(cursor_description))
             )
-
         seen = set()
         for (
             idx,
@@ -558,7 +557,6 @@ class ResultMetaData(object):
             yield idx, colname, mapped_type, coltype, obj, untranslated
 
     def _merge_cols_by_none(self, context, cursor_description):
-        dialect = context.dialect
         for (
             idx,
             colname,
