@@ -7,7 +7,9 @@
 
 """sqlalchemy.orm.interfaces.LoaderStrategy
    implementations, and related MapperOptions."""
+from __future__ import absolute_import
 
+import collections
 import itertools
 
 from . import attributes
@@ -140,7 +142,7 @@ class UninstrumentedColumnLoader(LoaderStrategy):
     def setup_query(
         self,
         context,
-        entity,
+        query_entity,
         path,
         loadopt,
         adapter,
@@ -173,7 +175,7 @@ class ColumnLoader(LoaderStrategy):
     def setup_query(
         self,
         context,
-        entity,
+        query_entity,
         path,
         loadopt,
         adapter,
@@ -235,7 +237,7 @@ class ExpressionColumnLoader(ColumnLoader):
     def setup_query(
         self,
         context,
-        entity,
+        query_entity,
         path,
         loadopt,
         adapter,
@@ -335,7 +337,7 @@ class DeferredColumnLoader(LoaderStrategy):
     def setup_query(
         self,
         context,
-        entity,
+        query_entity,
         path,
         loadopt,
         adapter,
@@ -366,7 +368,7 @@ class DeferredColumnLoader(LoaderStrategy):
                 (("deferred", False), ("instrument", True))
             ).setup_query(
                 context,
-                entity,
+                query_entity,
                 path,
                 loadopt,
                 adapter,
@@ -440,11 +442,12 @@ class LoadDeferredColumns(object):
 class AbstractRelationshipLoader(LoaderStrategy):
     """LoaderStratgies which deal with related objects."""
 
-    __slots__ = "mapper", "target", "uselist"
+    __slots__ = "mapper", "target", "uselist", "entity"
 
     def __init__(self, parent, strategy_key):
         super(AbstractRelationshipLoader, self).__init__(parent, strategy_key)
         self.mapper = self.parent_property.mapper
+        self.entity = self.parent_property.entity
         self.target = self.parent_property.target
         self.uselist = self.parent_property.uselist
 
@@ -510,6 +513,7 @@ class LazyLoader(AbstractRelationshipLoader, util.MemoizedSlots):
         "_lazywhere",
         "_rev_lazywhere",
         "use_get",
+        "is_aliased_class",
         "_bind_to_col",
         "_equated_columns",
         "_rev_bind_to_col",
@@ -525,10 +529,14 @@ class LazyLoader(AbstractRelationshipLoader, util.MemoizedSlots):
         self._raise_always = self.strategy_opts["lazy"] == "raise"
         self._raise_on_sql = self.strategy_opts["lazy"] == "raise_on_sql"
 
+        self.is_aliased_class = inspect(self.entity).is_aliased_class
+
         join_condition = self.parent_property._join_condition
-        self._lazywhere, self._bind_to_col, self._equated_columns = (
-            join_condition.create_lazy_clause()
-        )
+        (
+            self._lazywhere,
+            self._bind_to_col,
+            self._equated_columns,
+        ) = join_condition.create_lazy_clause()
 
         (
             self._rev_lazywhere,
@@ -540,10 +548,19 @@ class LazyLoader(AbstractRelationshipLoader, util.MemoizedSlots):
 
         # determine if our "lazywhere" clause is the same as the mapper's
         # get() clause.  then we can just use mapper.get()
-        self.use_get = not self.uselist and self.mapper._get_clause[0].compare(
-            self._lazywhere,
-            use_proxies=True,
-            equivalents=self.mapper._equivalent_columns,
+        #
+        # TODO: the "not self.uselist" can be taken out entirely; a m2o
+        # load that populates for a list (very unusual, but is possible with
+        # the API) can still set for "None" and the attribute system will
+        # populate as an empty list.
+        self.use_get = (
+            not self.is_aliased_class
+            and not self.uselist
+            and self.entity._get_clause[0].compare(
+                self._lazywhere,
+                use_proxies=True,
+                equivalents=self.mapper._equivalent_columns,
+            )
         )
 
         if self.use_get:
@@ -587,6 +604,10 @@ class LazyLoader(AbstractRelationshipLoader, util.MemoizedSlots):
 
         def visit_bindparam(bindparam):
             bindparam.unique = False
+
+        visitors.traverse(criterion, {}, {"bindparam": visit_bindparam})
+
+        def visit_bindparam(bindparam):
             if bindparam._identifying_key in bind_to_col:
                 params.append(
                     (
@@ -660,11 +681,29 @@ class LazyLoader(AbstractRelationshipLoader, util.MemoizedSlots):
         ):
             return attributes.PASSIVE_NO_RESULT
 
-        if self._raise_always:
+        if (
+            # we were given lazy="raise"
+            self._raise_always
+            # the no_raise history-related flag was not passed
+            and not passive & attributes.NO_RAISE
+            and (
+                # if we are use_get and related_object_ok is disabled,
+                # which means we are at most looking in the identity map
+                # for history purposes or otherwise returning
+                # PASSIVE_NO_RESULT, don't raise.  This is also a
+                # history-related flag
+                not self.use_get
+                or passive & attributes.RELATED_OBJECT_OK
+            )
+        ):
+
             self._invoke_raise_load(state, passive, "raise")
 
         session = _state_session(state)
         if not session:
+            if passive & attributes.NO_RAISE:
+                return attributes.PASSIVE_NO_RESULT
+
             raise orm_exc.DetachedInstanceError(
                 "Parent instance %s is not bound to a Session; "
                 "lazy load operation of attribute '%s' cannot proceed"
@@ -690,7 +729,7 @@ class LazyLoader(AbstractRelationshipLoader, util.MemoizedSlots):
             # does this, including how it decides what the correct
             # identity_token would be for this identity.
             instance = session.query()._identity_lookup(
-                self.mapper,
+                self.entity,
                 primary_key_identity,
                 passive=passive,
                 lazy_loaded_from=state,
@@ -754,7 +793,7 @@ class LazyLoader(AbstractRelationshipLoader, util.MemoizedSlots):
         # lazy loaders.   Currently the LRU cache is local to the LazyLoader,
         # however add ourselves to the initial cache key just to future
         # proof in case it moves
-        q = self._bakery(lambda session: session.query(self.mapper), self)
+        q = self._bakery(lambda session: session.query(self.entity), self)
 
         q.add_criteria(
             lambda q: q._adapt_all_clauses()._with_invoke_all_eagers(False),
@@ -989,7 +1028,7 @@ class SubqueryLoader(AbstractRelationshipLoader):
         if with_poly_info is not None:
             effective_entity = with_poly_info.entity
         else:
-            effective_entity = self.mapper
+            effective_entity = self.entity
 
         subq_path = context.attributes.get(
             ("subquery_path", None), orm_util.PathRegistry.root
@@ -1419,7 +1458,7 @@ class JoinedLoader(AbstractRelationshipLoader):
     def setup_query(
         self,
         context,
-        entity,
+        query_entity,
         path,
         loadopt,
         adapter,
@@ -1451,7 +1490,7 @@ class JoinedLoader(AbstractRelationshipLoader):
                 adapter,
                 add_to_collection,
             ) = self._setup_query_on_user_defined_adapter(
-                context, entity, path, adapter, user_defined_adapter
+                context, query_entity, path, adapter, user_defined_adapter
             )
         else:
             # if not via query option, check for
@@ -1470,7 +1509,7 @@ class JoinedLoader(AbstractRelationshipLoader):
                 chained_from_outerjoin,
             ) = self._generate_row_adapter(
                 context,
-                entity,
+                query_entity,
                 path,
                 loadopt,
                 adapter,
@@ -1487,12 +1526,12 @@ class JoinedLoader(AbstractRelationshipLoader):
         else:
             with_polymorphic = None
 
-        path = path[self.mapper]
+        path = path[self.entity]
 
         loading._setup_entity_query(
             context,
             self.mapper,
-            entity,
+            query_entity,
             path,
             clauses,
             add_to_collection,
@@ -1533,11 +1572,6 @@ class JoinedLoader(AbstractRelationshipLoader):
 
         root_mapper, prop = path[-2:]
 
-        # from .mapper import Mapper
-        # from .interfaces import MapperProperty
-        # assert isinstance(root_mapper, Mapper)
-        # assert isinstance(prop, MapperProperty)
-
         if alias is not None:
             if isinstance(alias, str):
                 alias = prop.target.alias(alias)
@@ -1594,6 +1628,11 @@ class JoinedLoader(AbstractRelationshipLoader):
         # we need one unique AliasedClass per query per appearance of our
         # entity in the query.
 
+        if inspect(self.entity).is_aliased_class:
+            alt_selectable = inspect(self.entity).selectable
+        else:
+            alt_selectable = None
+
         key = ("joinedloader_ac", self)
         if key not in context.attributes:
             context.attributes[key] = idx = 0
@@ -1602,8 +1641,14 @@ class JoinedLoader(AbstractRelationshipLoader):
 
         if idx >= len(self._aliased_class_pool):
             to_adapt = orm_util.AliasedClass(
-                self.mapper, flat=True, use_mapper_path=True
+                self.mapper,
+                alias=alt_selectable.alias(flat=True)
+                if alt_selectable is not None
+                else None,
+                flat=True,
+                use_mapper_path=True,
             )
+
             # load up the .columns collection on the Alias() before
             # the object becomes shared among threads.  this prevents
             # races for column identities.
@@ -1680,7 +1725,7 @@ class JoinedLoader(AbstractRelationshipLoader):
     def _create_eager_join(
         self,
         context,
-        entity,
+        query_entity,
         path,
         adapter,
         parentmapper,
@@ -1690,7 +1735,7 @@ class JoinedLoader(AbstractRelationshipLoader):
     ):
 
         if parentmapper is None:
-            localparent = entity.mapper
+            localparent = query_entity.mapper
         else:
             localparent = parentmapper
 
@@ -1702,27 +1747,43 @@ class JoinedLoader(AbstractRelationshipLoader):
             and context.query._should_nest_selectable
         )
 
-        entity_key = None
+        query_entity_key = None
 
         if (
-            entity not in context.eager_joins
+            query_entity not in context.eager_joins
             and not should_nest_selectable
             and context.from_clause
         ):
-            index, clause = sql_util.find_join_source(
-                context.from_clause, entity.selectable
+            indexes = sql_util.find_left_clause_that_matches_given(
+                context.from_clause, query_entity.selectable
             )
-            if clause is not None:
+
+            if len(indexes) > 1:
+                # for the eager load case, I can't reproduce this right
+                # now.   For query.join() I can.
+                raise sa_exc.InvalidRequestError(
+                    "Can't identify which query entity in which to joined "
+                    "eager load from.   Please use an exact match when "
+                    "specifying the join path."
+                )
+
+            if indexes:
+                clause = context.from_clause[indexes[0]]
                 # join to an existing FROM clause on the query.
                 # key it to its list index in the eager_joins dict.
                 # Query._compile_context will adapt as needed and
                 # append to the FROM clause of the select().
-                entity_key, default_towrap = index, clause
+                query_entity_key, default_towrap = indexes[0], clause
 
-        if entity_key is None:
-            entity_key, default_towrap = entity, entity.selectable
+        if query_entity_key is None:
+            query_entity_key, default_towrap = (
+                query_entity,
+                query_entity.selectable,
+            )
 
-        towrap = context.eager_joins.setdefault(entity_key, default_towrap)
+        towrap = context.eager_joins.setdefault(
+            query_entity_key, default_towrap
+        )
 
         if adapter:
             if getattr(adapter, "aliased_class", None):
@@ -1757,7 +1818,7 @@ class JoinedLoader(AbstractRelationshipLoader):
             not chained_from_outerjoin
             or not innerjoin
             or innerjoin == "unnested"
-            or entity.entity_zero.represents_outer_join
+            or query_entity.entity_zero.represents_outer_join
         )
 
         if attach_on_outside:
@@ -1767,7 +1828,7 @@ class JoinedLoader(AbstractRelationshipLoader):
                 clauses.aliased_class,
                 onclause,
                 isouter=not innerjoin
-                or entity.entity_zero.represents_outer_join
+                or query_entity.entity_zero.represents_outer_join
                 or (chained_from_outerjoin and isinstance(towrap, sql.Join)),
                 _left_memo=self.parent,
                 _right_memo=self.mapper,
@@ -1778,10 +1839,10 @@ class JoinedLoader(AbstractRelationshipLoader):
                 path, towrap, clauses, onclause
             )
 
-        context.eager_joins[entity_key] = eagerjoin
+        context.eager_joins[query_entity_key] = eagerjoin
 
         # send a hint to the Query as to where it may "splice" this join
-        eagerjoin.stop_on = entity.selectable
+        eagerjoin.stop_on = query_entity.selectable
 
         if not parentmapper:
             # for parentclause that is the non-eager end of the join,
@@ -1794,7 +1855,7 @@ class JoinedLoader(AbstractRelationshipLoader):
             for col in sql_util._find_columns(
                 self.parent_property.primaryjoin
             ):
-                if localparent.mapped_table.c.contains_column(col):
+                if localparent.persist_selectable.c.contains_column(col):
                     if adapter:
                         col = adapter.columns[col]
                     context.primary_columns.append(col)
@@ -1924,7 +1985,7 @@ class JoinedLoader(AbstractRelationshipLoader):
                 self.mapper,
                 context,
                 result,
-                our_path[self.mapper],
+                our_path[self.entity],
                 eager_adapter,
             )
 
@@ -2028,29 +2089,107 @@ class JoinedLoader(AbstractRelationshipLoader):
 class SelectInLoader(AbstractRelationshipLoader, util.MemoizedSlots):
     __slots__ = (
         "join_depth",
+        "omit_join",
         "_parent_alias",
-        "_in_expr",
-        "_parent_pk_cols",
-        "_zero_idx",
+        "_query_info",
+        "_fallback_query_info",
         "_bakery",
     )
 
+    query_info = collections.namedtuple(
+        "queryinfo",
+        [
+            "load_only_child",
+            "load_with_join",
+            "in_expr",
+            "pk_cols",
+            "zero_idx",
+            "child_lookup_cols",
+        ],
+    )
+
     _chunksize = 500
 
     def __init__(self, parent, strategy_key):
         super(SelectInLoader, self).__init__(parent, strategy_key)
         self.join_depth = self.parent_property.join_depth
+        is_m2o = self.parent_property.direction is interfaces.MANYTOONE
+
+        if self.parent_property.omit_join is not None:
+            self.omit_join = self.parent_property.omit_join
+        else:
+            lazyloader = self.parent_property._get_strategy(
+                (("lazy", "select"),)
+            )
+            if is_m2o:
+                self.omit_join = lazyloader.use_get
+            else:
+                self.omit_join = self.parent._get_clause[0].compare(
+                    lazyloader._rev_lazywhere,
+                    use_proxies=True,
+                    equivalents=self.parent._equivalent_columns,
+                )
+
+        if self.omit_join:
+            if is_m2o:
+                self._query_info = self._init_for_omit_join_m2o()
+                self._fallback_query_info = self._init_for_join()
+            else:
+                self._query_info = self._init_for_omit_join()
+        else:
+            self._query_info = self._init_for_join()
+
+    def _init_for_omit_join(self):
+        pk_to_fk = dict(
+            self.parent_property._join_condition.local_remote_pairs
+        )
+        pk_to_fk.update(
+            (equiv, pk_to_fk[k])
+            for k in list(pk_to_fk)
+            for equiv in self.parent._equivalent_columns.get(k, ())
+        )
+
+        pk_cols = fk_cols = [
+            pk_to_fk[col] for col in self.parent.primary_key if col in pk_to_fk
+        ]
+        if len(fk_cols) > 1:
+            in_expr = sql.tuple_(*fk_cols)
+            zero_idx = False
+        else:
+            in_expr = fk_cols[0]
+            zero_idx = True
+
+        return self.query_info(False, False, in_expr, pk_cols, zero_idx, None)
+
+    def _init_for_omit_join_m2o(self):
+        pk_cols = self.mapper.primary_key
+        if len(pk_cols) > 1:
+            in_expr = sql.tuple_(*pk_cols)
+            zero_idx = False
+        else:
+            in_expr = pk_cols[0]
+            zero_idx = True
+
+        lazyloader = self.parent_property._get_strategy((("lazy", "select"),))
+        lookup_cols = [lazyloader._equated_columns[pk] for pk in pk_cols]
+
+        return self.query_info(
+            True, False, in_expr, pk_cols, zero_idx, lookup_cols
+        )
+
+    def _init_for_join(self):
         self._parent_alias = aliased(self.parent.class_)
         pa_insp = inspect(self._parent_alias)
-        self._parent_pk_cols = pk_cols = [
+        pk_cols = [
             pa_insp._adapt_element(col) for col in self.parent.primary_key
         ]
         if len(pk_cols) > 1:
-            self._in_expr = sql.tuple_(*pk_cols)
-            self._zero_idx = False
+            in_expr = sql.tuple_(*pk_cols)
+            zero_idx = False
         else:
-            self._in_expr = pk_cols[0]
-            self._zero_idx = True
+            in_expr = pk_cols[0]
+            zero_idx = True
+        return self.query_info(False, True, in_expr, pk_cols, zero_idx, None)
 
     def init_class_attribute(self, mapper):
         self.parent_property._get_strategy(
@@ -2092,7 +2231,7 @@ class SelectInLoader(AbstractRelationshipLoader, util.MemoizedSlots):
         if with_poly_info is not None:
             effective_entity = with_poly_info.entity
         else:
-            effective_entity = self.mapper
+            effective_entity = self.entity
 
         if not path_w_prop.contains(context.attributes, "loader"):
             if self.join_depth:
@@ -2118,12 +2257,65 @@ class SelectInLoader(AbstractRelationshipLoader, util.MemoizedSlots):
         if load_only and self.key not in load_only:
             return
 
-        our_states = [
-            (state.key[1], state, overwrite) for state, overwrite in states
-        ]
+        query_info = self._query_info
 
-        pk_cols = self._parent_pk_cols
-        pa = self._parent_alias
+        if query_info.load_only_child:
+            our_states = collections.defaultdict(list)
+            none_states = []
+
+            mapper = self.parent
+
+            for state, overwrite in states:
+                state_dict = state.dict
+                related_ident = tuple(
+                    mapper._get_state_attr_by_column(
+                        state,
+                        state_dict,
+                        lk,
+                        passive=attributes.PASSIVE_NO_FETCH,
+                    )
+                    for lk in query_info.child_lookup_cols
+                )
+                # if the loaded parent objects do not have the foreign key
+                # to the related item loaded, then degrade into the joined
+                # version of selectinload
+                if attributes.PASSIVE_NO_RESULT in related_ident:
+                    query_info = self._fallback_query_info
+                    break
+
+                # organize states into lists keyed to particular foreign
+                # key values.
+                if None not in related_ident:
+                    our_states[related_ident].append(
+                        (state, state_dict, overwrite)
+                    )
+                else:
+                    # For FK values that have None, add them to a
+                    # separate collection that will be populated separately
+                    none_states.append((state, state_dict, overwrite))
+
+        # note the above conditional may have changed query_info
+        if not query_info.load_only_child:
+            our_states = [
+                (state.key[1], state, state.dict, overwrite)
+                for state, overwrite in states
+            ]
+
+        pk_cols = query_info.pk_cols
+        in_expr = query_info.in_expr
+
+        if not query_info.load_with_join:
+            # in "omit join" mode, the primary key column and the
+            # "in" expression are in terms of the related entity.  So
+            # if the related entity is polymorphic or otherwise aliased,
+            # we need to adapt our "pk_cols" and "in_expr" to that
+            # entity.   in non-"omit join" mode, these are against the
+            # parent entity and do not need adaption.
+            insp = inspect(effective_entity)
+            if insp.is_aliased_class:
+                pk_cols = [insp._adapt_element(col) for col in pk_cols]
+                in_expr = insp._adapt_element(in_expr)
+                pk_cols = [insp._adapt_element(col) for col in pk_cols]
 
         q = self._bakery(
             lambda session: session.query(
@@ -2132,18 +2324,38 @@ class SelectInLoader(AbstractRelationshipLoader, util.MemoizedSlots):
             self,
         )
 
-        q.add_criteria(
-            lambda q: q.select_from(pa)
-            .join(
-                getattr(pa, self.parent_property.key).of_type(effective_entity)
-            )
-            .filter(
-                self._in_expr.in_(
-                    sql.bindparam("primary_keys", expanding=True)
+        if not query_info.load_with_join:
+            # the Bundle we have in the "omit_join" case is against raw, non
+            # annotated columns, so to ensure the Query knows its primary
+            # entity, we add it explicitly.  If we made the Bundle against
+            # annotated columns, we hit a performance issue in this specific
+            # case, which is detailed in issue #4347.
+            q.add_criteria(lambda q: q.select_from(effective_entity))
+        else:
+            # in the non-omit_join case, the Bundle is against the annotated/
+            # mapped column of the parent entity, but the #4347 issue does not
+            # occur in this case.
+            pa = self._parent_alias
+            q.add_criteria(
+                lambda q: q.select_from(pa).join(
+                    getattr(pa, self.parent_property.key).of_type(
+                        effective_entity
+                    )
                 )
             )
-            .order_by(*pk_cols)
-        )
+
+        if query_info.load_only_child:
+            q.add_criteria(
+                lambda q: q.filter(
+                    in_expr.in_(sql.bindparam("primary_keys", expanding=True))
+                )
+            )
+        else:
+            q.add_criteria(
+                lambda q: q.filter(
+                    in_expr.in_(sql.bindparam("primary_keys", expanding=True))
+                ).order_by(*pk_cols)
+            )
 
         orig_query = context.query
 
@@ -2155,19 +2367,78 @@ class SelectInLoader(AbstractRelationshipLoader, util.MemoizedSlots):
             q.add_criteria(lambda q: q.populate_existing())
 
         if self.parent_property.order_by:
+            if not query_info.load_with_join:
+                eager_order_by = self.parent_property.order_by
+                if insp.is_aliased_class:
+                    eager_order_by = [
+                        insp._adapt_element(elem) for elem in eager_order_by
+                    ]
+                q.add_criteria(lambda q: q.order_by(*eager_order_by))
+            else:
 
-            def _setup_outermost_orderby(q):
-                # imitate the same method that
-                # subquery eager loading does it, looking for the
-                # adapted "secondary" table
-                eagerjoin = q._from_obj[0]
-                eager_order_by = eagerjoin._target_adapter.copy_and_process(
-                    util.to_list(self.parent_property.order_by)
+                def _setup_outermost_orderby(q):
+                    # imitate the same method that subquery eager loading uses,
+                    # looking for the adapted "secondary" table
+                    eagerjoin = q._from_obj[0]
+
+                    return q.order_by(
+                        *eagerjoin._target_adapter.copy_and_process(
+                            util.to_list(self.parent_property.order_by)
+                        )
+                    )
+
+                q.add_criteria(_setup_outermost_orderby)
+
+        if query_info.load_only_child:
+            self._load_via_child(
+                our_states, none_states, query_info, q, context
+            )
+        else:
+            self._load_via_parent(our_states, query_info, q, context)
+
+    def _load_via_child(self, our_states, none_states, query_info, q, context):
+        uselist = self.uselist
+
+        # this sort is really for the benefit of the unit tests
+        our_keys = sorted(our_states)
+        while our_keys:
+            chunk = our_keys[0 : self._chunksize]
+            our_keys = our_keys[self._chunksize :]
+            data = {
+                k: v
+                for k, v in q(context.session).params(
+                    primary_keys=[
+                        key[0] if query_info.zero_idx else key for key in chunk
+                    ]
                 )
-                return q.order_by(*eager_order_by)
+            }
 
-            q.add_criteria(_setup_outermost_orderby)
+            for key in chunk:
+                # for a real foreign key and no concurrent changes to the
+                # DB while running this method, "key" is always present in
+                # data.  However, for primaryjoins without real foreign keys
+                # a non-None primaryjoin condition may still refer to no
+                # related object.
+                related_obj = data.get(key, None)
+                for state, dict_, overwrite in our_states[key]:
+                    if not overwrite and self.key in dict_:
+                        continue
 
+                    state.get_impl(self.key).set_committed_value(
+                        state,
+                        dict_,
+                        related_obj if not uselist else [related_obj],
+                    )
+        # populate none states with empty value / collection
+        for state, dict_, overwrite in none_states:
+            if not overwrite and self.key in dict_:
+                continue
+
+            # note it's OK if this is a uselist=True attribute, the empty
+            # collection will be populated
+            state.get_impl(self.key).set_committed_value(state, dict_, None)
+
+    def _load_via_parent(self, our_states, query_info, q, context):
         uselist = self.uselist
         _empty_result = () if uselist else None
 
@@ -2175,22 +2446,22 @@ class SelectInLoader(AbstractRelationshipLoader, util.MemoizedSlots):
             chunk = our_states[0 : self._chunksize]
             our_states = our_states[self._chunksize :]
 
+            primary_keys = [
+                key[0] if query_info.zero_idx else key
+                for key, state, state_dict, overwrite in chunk
+            ]
+
             data = {
                 k: [vv[1] for vv in v]
                 for k, v in itertools.groupby(
-                    q(context.session).params(
-                        primary_keys=[
-                            key[0] if self._zero_idx else key
-                            for key, state, overwrite in chunk
-                        ]
-                    ),
+                    q(context.session).params(primary_keys=primary_keys),
                     lambda x: x[0],
                 )
             }
 
-            for key, state, overwrite in chunk:
+            for key, state, state_dict, overwrite in chunk:
 
-                if not overwrite and self.key in state.dict:
+                if not overwrite and self.key in state_dict:
                     continue
 
                 collection = data.get(key, _empty_result)
@@ -2203,11 +2474,13 @@ class SelectInLoader(AbstractRelationshipLoader, util.MemoizedSlots):
                             "attribute '%s' " % self
                         )
                     state.get_impl(self.key).set_committed_value(
-                        state, state.dict, collection[0]
+                        state, state_dict, collection[0]
                     )
                 else:
+                    # note that empty tuple set on uselist=False sets the
+                    # value to None
                     state.get_impl(self.key).set_committed_value(
-                        state, state.dict, collection
+                        state, state_dict, collection
                     )
 
 
