@@ -264,7 +264,7 @@ class MemUsageWBackendTest(EnsureZeroed):
 
     __tags__ = ("memory_intensive",)
     __requires__ = "cpython", "memory_process_intensive"
-    __backend__ = True
+    __sparse_backend__ = True
 
     # ensure a pure growing test trips the assertion
     @testing.fails_if(lambda: True)
@@ -321,8 +321,6 @@ class MemUsageWBackendTest(EnsureZeroed):
         )
         m2 = mapper(B, table2)
 
-        m3 = mapper(A, table1, non_primary=True)
-
         @profile_memory()
         def go():
             sess = create_session()
@@ -354,7 +352,7 @@ class MemUsageWBackendTest(EnsureZeroed):
         go()
 
         metadata.drop_all()
-        del m1, m2, m3
+        del m1, m2
         assert_no_mappers()
 
     def test_sessionmaker(self):
@@ -415,8 +413,6 @@ class MemUsageWBackendTest(EnsureZeroed):
         )
         m2 = mapper(B, table2, _compiled_cache_size=50)
 
-        m3 = mapper(A, table1, non_primary=True)
-
         @profile_memory()
         def go():
             engine = engines.testing_engine(
@@ -458,7 +454,7 @@ class MemUsageWBackendTest(EnsureZeroed):
         go()
 
         metadata.drop_all()
-        del m1, m2, m3
+        del m1, m2
         assert_no_mappers()
 
     @testing.emits_warning("Compiled statement cache for.*")
@@ -642,8 +638,6 @@ class MemUsageWBackendTest(EnsureZeroed):
             )
             mapper(B, table2)
 
-            mapper(A, table1, non_primary=True)
-
             sess = create_session()
             a1 = A(col2="a1")
             a2 = A(col2="a2")
@@ -744,12 +738,19 @@ class MemUsageWBackendTest(EnsureZeroed):
             Column("foo", Integer),
             Column("bar", Integer),
         )
-        m1 = mapper(A, a)
+        b = Table(
+            "b",
+            metadata,
+            Column("id", Integer, primary_key=True),
+            Column("a_id", ForeignKey("a.id")),
+        )
+        m1 = mapper(A, a, properties={"bs": relationship(B)})
+        mapper(B, b)
 
         @profile_memory()
         def go():
             ma = sa.inspect(aliased(A))
-            m1._path_registry[m1.attrs.foo][ma][m1.attrs.bar]
+            m1._path_registry[m1.attrs.bs][ma][m1.attrs.bar]
 
         go()
         clear_mappers()
@@ -957,7 +958,7 @@ class MemUsageWBackendTest(EnsureZeroed):
 
         go()
 
-    # fails on newer versions of pysqlite due to unusual memory behvior
+    # fails on newer versions of pysqlite due to unusual memory behavior
     # in pysqlite itself. background at:
     # http://thread.gmane.org/gmane.comp.python.db.pysqlite.user/2290
 
