@@ -1,6 +1,7 @@
 #! coding: utf-8
 
 import copy
+import datetime
 import inspect
 import sys
 
@@ -15,6 +16,7 @@ from sqlalchemy.testing import eq_
 from sqlalchemy.testing import expect_warnings
 from sqlalchemy.testing import fails_if
 from sqlalchemy.testing import fixtures
+from sqlalchemy.testing import in_
 from sqlalchemy.testing import is_
 from sqlalchemy.testing import mock
 from sqlalchemy.testing import ne_
@@ -24,6 +26,7 @@ from sqlalchemy.util import classproperty
 from sqlalchemy.util import compat
 from sqlalchemy.util import get_callable_argspec
 from sqlalchemy.util import langhelpers
+from sqlalchemy.util import timezone
 from sqlalchemy.util import WeakSequence
 
 
@@ -762,7 +765,7 @@ class LRUTest(fixtures.TestBase):
         for id_ in (25, 24, 23, 14, 12, 19, 18, 17, 16, 15):
             assert id_ in lru
 
-        i1 = lru[25]
+        lru[25]
         i2 = item(25)
         lru[25] = i2
         assert 25 in lru
@@ -1673,7 +1676,10 @@ class ArgInspectionTest(fixtures.TestBase):
         def foo(x, y, **kw):
             pass
 
-        eq_(get_callable_argspec(foo), (["x", "y"], None, "kw", None))
+        eq_(
+            get_callable_argspec(foo),
+            compat.FullArgSpec(["x", "y"], None, "kw", None, [], None, {}),
+        )
 
     def test_callable_argspec_fn_no_self(self):
         def foo(x, y, **kw):
@@ -1681,7 +1687,7 @@ class ArgInspectionTest(fixtures.TestBase):
 
         eq_(
             get_callable_argspec(foo, no_self=True),
-            (["x", "y"], None, "kw", None),
+            compat.FullArgSpec(["x", "y"], None, "kw", None, [], None, {}),
         )
 
     def test_callable_argspec_fn_no_self_but_self(self):
@@ -1690,7 +1696,9 @@ class ArgInspectionTest(fixtures.TestBase):
 
         eq_(
             get_callable_argspec(foo, no_self=True),
-            (["self", "x", "y"], None, "kw", None),
+            compat.FullArgSpec(
+                ["self", "x", "y"], None, "kw", None, [], None, {}
+            ),
         )
 
     @fails_if(lambda: util.pypy, "pypy returns plain *arg, **kw")
@@ -1710,7 +1718,9 @@ class ArgInspectionTest(fixtures.TestBase):
 
         eq_(
             get_callable_argspec(Foo.foo),
-            (["self", "x", "y"], None, "kw", None),
+            compat.FullArgSpec(
+                ["self", "x", "y"], None, "kw", None, [], None, {}
+            ),
         )
 
     def test_callable_argspec_instance_method_no_self(self):
@@ -1720,7 +1730,7 @@ class ArgInspectionTest(fixtures.TestBase):
 
         eq_(
             get_callable_argspec(Foo().foo, no_self=True),
-            (["x", "y"], None, "kw", None),
+            compat.FullArgSpec(["x", "y"], None, "kw", None, [], None, {}),
         )
 
     def test_callable_argspec_unbound_method_no_self(self):
@@ -1730,7 +1740,9 @@ class ArgInspectionTest(fixtures.TestBase):
 
         eq_(
             get_callable_argspec(Foo.foo, no_self=True),
-            (["self", "x", "y"], None, "kw", None),
+            compat.FullArgSpec(
+                ["self", "x", "y"], None, "kw", None, [], None, {}
+            ),
         )
 
     def test_callable_argspec_init(self):
@@ -1738,7 +1750,12 @@ class ArgInspectionTest(fixtures.TestBase):
             def __init__(self, x, y):
                 pass
 
-        eq_(get_callable_argspec(Foo), (["self", "x", "y"], None, None, None))
+        eq_(
+            get_callable_argspec(Foo),
+            compat.FullArgSpec(
+                ["self", "x", "y"], None, None, None, [], None, {}
+            ),
+        )
 
     def test_callable_argspec_init_no_self(self):
         class Foo(object):
@@ -1747,7 +1764,7 @@ class ArgInspectionTest(fixtures.TestBase):
 
         eq_(
             get_callable_argspec(Foo, no_self=True),
-            (["x", "y"], None, None, None),
+            compat.FullArgSpec(["x", "y"], None, None, None, [], None, {}),
         )
 
     def test_callable_argspec_call(self):
@@ -1756,7 +1773,10 @@ class ArgInspectionTest(fixtures.TestBase):
                 pass
 
         eq_(
-            get_callable_argspec(Foo()), (["self", "x", "y"], None, None, None)
+            get_callable_argspec(Foo()),
+            compat.FullArgSpec(
+                ["self", "x", "y"], None, None, None, [], None, {}
+            ),
         )
 
     def test_callable_argspec_call_no_self(self):
@@ -1766,7 +1786,7 @@ class ArgInspectionTest(fixtures.TestBase):
 
         eq_(
             get_callable_argspec(Foo(), no_self=True),
-            (["x", "y"], None, None, None),
+            compat.FullArgSpec(["x", "y"], None, None, None, [], None, {}),
         )
 
     @fails_if(lambda: util.pypy, "pypy returns plain *arg, **kw")
@@ -1780,6 +1800,25 @@ class ArgInspectionTest(fixtures.TestBase):
 
         assert_raises(TypeError, get_callable_argspec, bar)
 
+    def test_getargspec_6_tuple(self):
+        def foo(x, y, z, **kw):
+            pass
+
+        spec = compat.inspect_getfullargspec(foo)
+
+        eq_(
+            spec,
+            compat.FullArgSpec(
+                args=["x", "y", "z"],
+                varargs=None,
+                varkw="kw",
+                defaults=None,
+                kwonlyargs=[],
+                kwonlydefaults=None,
+                annotations={},
+            ),
+        )
+
 
 class SymbolTest(fixtures.TestBase):
     def test_basic(self):
@@ -1802,7 +1841,7 @@ class SymbolTest(fixtures.TestBase):
 
         # default
         s = util.pickle.dumps(sym1)
-        sym3 = util.pickle.loads(s)
+        util.pickle.loads(s)
 
         for protocol in 0, 1, 2:
             print(protocol)
@@ -1831,17 +1870,113 @@ class SymbolTest(fixtures.TestBase):
         assert not (sym1 | sym2) & (sym3 | sym4)
         assert (sym1 | sym2) & (sym2 | sym4)
 
+    def test_parser(self):
+        sym1 = util.symbol("sym1", canonical=1)
+        sym2 = util.symbol("sym2", canonical=2)
+        sym3 = util.symbol("sym3", canonical=4)
+        sym4 = util.symbol("sym4", canonical=8)
+
+        lookup_one = {sym1: [], sym2: [True], sym3: [False], sym4: [None]}
+        lookup_two = {sym1: [], sym2: [True], sym3: [False]}
+        lookup_three = {sym1: [], sym2: ["symbol2"], sym3: []}
+
+        is_(
+            util.symbol.parse_user_argument(
+                "sym2", lookup_one, "some_name", resolve_symbol_names=True
+            ),
+            sym2,
+        )
+
+        assert_raises_message(
+            exc.ArgumentError,
+            "Invalid value for 'some_name': 'sym2'",
+            util.symbol.parse_user_argument,
+            "sym2",
+            lookup_one,
+            "some_name",
+        )
+        is_(
+            util.symbol.parse_user_argument(
+                True, lookup_one, "some_name", resolve_symbol_names=False
+            ),
+            sym2,
+        )
+
+        is_(
+            util.symbol.parse_user_argument(sym2, lookup_one, "some_name"),
+            sym2,
+        )
+
+        is_(
+            util.symbol.parse_user_argument(None, lookup_one, "some_name"),
+            sym4,
+        )
+
+        is_(
+            util.symbol.parse_user_argument(None, lookup_two, "some_name"),
+            None,
+        )
+
+        is_(
+            util.symbol.parse_user_argument(
+                "symbol2", lookup_three, "some_name"
+            ),
+            sym2,
+        )
+
+        assert_raises_message(
+            exc.ArgumentError,
+            "Invalid value for 'some_name': 'foo'",
+            util.symbol.parse_user_argument,
+            "foo",
+            lookup_three,
+            "some_name",
+        )
+
+
+class _Py3KFixtures(object):
+    pass
+
+
+if util.py3k:
+    _locals = {}
+    exec(
+        """
+def _kw_only_fixture(self, a, *, b, c):
+    pass
+
+def _kw_plus_posn_fixture(self, a, *args, b, c):
+    pass
+
+def _kw_opt_fixture(self, a, *, b, c="c"):
+    pass
+""",
+        _locals,
+    )
+    for k in _locals:
+        setattr(_Py3KFixtures, k, _locals[k])
+
+
+class TestFormatArgspec(_Py3KFixtures, fixtures.TestBase):
+    def _test_format_argspec_plus(self, fn, wanted, grouped=None):
+
+        # test direct function
+        if grouped is None:
+            parsed = util.format_argspec_plus(fn)
+        else:
+            parsed = util.format_argspec_plus(fn, grouped=grouped)
+        eq_(parsed, wanted)
+
+        # test sending fullargspec
+        spec = compat.inspect_getfullargspec(fn)
+        if grouped is None:
+            parsed = util.format_argspec_plus(spec)
+        else:
+            parsed = util.format_argspec_plus(spec, grouped=grouped)
+        eq_(parsed, wanted)
 
-class TestFormatArgspec(fixtures.TestBase):
     def test_specs(self):
-        def test(fn, wanted, grouped=None):
-            if grouped is None:
-                parsed = util.format_argspec_plus(fn)
-            else:
-                parsed = util.format_argspec_plus(fn, grouped=grouped)
-            eq_(parsed, wanted)
-
-        test(
+        self._test_format_argspec_plus(
             lambda: None,
             {
                 "args": "()",
@@ -1851,13 +1986,13 @@ class TestFormatArgspec(fixtures.TestBase):
             },
         )
 
-        test(
+        self._test_format_argspec_plus(
             lambda: None,
             {"args": "", "self_arg": None, "apply_kw": "", "apply_pos": ""},
             grouped=False,
         )
 
-        test(
+        self._test_format_argspec_plus(
             lambda self: None,
             {
                 "args": "(self)",
@@ -1867,7 +2002,7 @@ class TestFormatArgspec(fixtures.TestBase):
             },
         )
 
-        test(
+        self._test_format_argspec_plus(
             lambda self: None,
             {
                 "args": "self",
@@ -1878,7 +2013,7 @@ class TestFormatArgspec(fixtures.TestBase):
             grouped=False,
         )
 
-        test(
+        self._test_format_argspec_plus(
             lambda *a: None,
             {
                 "args": "(*a)",
@@ -1888,7 +2023,7 @@ class TestFormatArgspec(fixtures.TestBase):
             },
         )
 
-        test(
+        self._test_format_argspec_plus(
             lambda **kw: None,
             {
                 "args": "(**kw)",
@@ -1898,7 +2033,7 @@ class TestFormatArgspec(fixtures.TestBase):
             },
         )
 
-        test(
+        self._test_format_argspec_plus(
             lambda *a, **kw: None,
             {
                 "args": "(*a, **kw)",
@@ -1908,7 +2043,7 @@ class TestFormatArgspec(fixtures.TestBase):
             },
         )
 
-        test(
+        self._test_format_argspec_plus(
             lambda a, *b: None,
             {
                 "args": "(a, *b)",
@@ -1918,7 +2053,7 @@ class TestFormatArgspec(fixtures.TestBase):
             },
         )
 
-        test(
+        self._test_format_argspec_plus(
             lambda a, **b: None,
             {
                 "args": "(a, **b)",
@@ -1928,7 +2063,7 @@ class TestFormatArgspec(fixtures.TestBase):
             },
         )
 
-        test(
+        self._test_format_argspec_plus(
             lambda a, *b, **c: None,
             {
                 "args": "(a, *b, **c)",
@@ -1938,7 +2073,7 @@ class TestFormatArgspec(fixtures.TestBase):
             },
         )
 
-        test(
+        self._test_format_argspec_plus(
             lambda a, b=1, **c: None,
             {
                 "args": "(a, b=1, **c)",
@@ -1948,7 +2083,7 @@ class TestFormatArgspec(fixtures.TestBase):
             },
         )
 
-        test(
+        self._test_format_argspec_plus(
             lambda a=1, b=2: None,
             {
                 "args": "(a=1, b=2)",
@@ -1958,7 +2093,7 @@ class TestFormatArgspec(fixtures.TestBase):
             },
         )
 
-        test(
+        self._test_format_argspec_plus(
             lambda a=1, b=2: None,
             {
                 "args": "a=1, b=2",
@@ -1969,6 +2104,38 @@ class TestFormatArgspec(fixtures.TestBase):
             grouped=False,
         )
 
+        if util.py3k:
+            self._test_format_argspec_plus(
+                self._kw_only_fixture,
+                {
+                    "args": "self, a, *, b, c",
+                    "self_arg": "self",
+                    "apply_pos": "self, a, *, b, c",
+                    "apply_kw": "self, a, b=b, c=c",
+                },
+                grouped=False,
+            )
+            self._test_format_argspec_plus(
+                self._kw_plus_posn_fixture,
+                {
+                    "args": "self, a, *args, b, c",
+                    "self_arg": "self",
+                    "apply_pos": "self, a, *args, b, c",
+                    "apply_kw": "self, a, b=b, c=c, *args",
+                },
+                grouped=False,
+            )
+            self._test_format_argspec_plus(
+                self._kw_opt_fixture,
+                {
+                    "args": "self, a, *, b, c='c'",
+                    "self_arg": "self",
+                    "apply_pos": "self, a, *, b, c",
+                    "apply_kw": "self, a, b=b, c=c",
+                },
+                grouped=False,
+            )
+
     @testing.fails_if(
         lambda: util.pypy,
         "pypy doesn't report Obj.__init__ as object.__init__",
@@ -2203,6 +2370,10 @@ class AsInterfaceTest(fixtures.TestBase):
     class Object(object):
         pass
 
+    def test_no_cls_no_methods(self):
+        obj = object()
+        assert_raises(TypeError, util.as_interface, obj)
+
     def test_instance(self):
         obj = object()
         assert_raises(TypeError, util.as_interface, obj, cls=self.Something)
@@ -2455,6 +2626,11 @@ class TestProperties(fixtures.TestBase):
             eq_(props._data, p._data)
             eq_(props.keys(), p.keys())
 
+    def test_keys_in_dir(self):
+        data = {"hello": "bla"}
+        props = util.Properties(data)
+        in_("hello", dir(props))
+
     def test_pickle_immuatbleprops(self):
         data = {"hello": "bla"}
         props = util.Properties(data).as_immutable()
@@ -2592,3 +2768,68 @@ class BackslashReplaceTest(fixtures.TestBase):
             compat.decode_backslashreplace(message, "cp1251"),
             util.u("some message П"),
         )
+
+
+class TimezoneTest(fixtures.TestBase):
+    """test the python 2 backport of the "timezone" class.
+
+    Note under python 3, these tests work against the builtin timezone,
+    thereby providing confirmation that the tests are correct.
+
+    """
+
+    @testing.combinations(
+        (datetime.timedelta(0), "UTC"),
+        (datetime.timedelta(hours=5), "UTC+05:00"),
+        (datetime.timedelta(hours=5, minutes=10), "UTC+05:10"),
+        (
+            datetime.timedelta(hours=5, minutes=10, seconds=27),
+            "UTC+05:10:27",
+            testing.requires.granular_timezone,
+        ),
+        (datetime.timedelta(hours=-3, minutes=10), "UTC-02:50"),
+        (
+            datetime.timedelta(
+                hours=5, minutes=10, seconds=27, microseconds=550
+            ),
+            "UTC+05:10:27.000550",
+            testing.requires.granular_timezone,
+        ),
+    )
+    def test_tzname(self, td, expected):
+        eq_(timezone(td).tzname(None), expected)
+
+    def test_utcoffset(self):
+        eq_(
+            timezone(datetime.timedelta(hours=5)).utcoffset(None),
+            datetime.timedelta(hours=5),
+        )
+
+    def test_fromutc(self):
+        tzinfo = timezone(datetime.timedelta(hours=5))
+        dt = datetime.datetime(2017, 10, 5, 12, 55, 38, tzinfo=tzinfo)
+        eq_(
+            dt.astimezone(timezone.utc),
+            datetime.datetime(2017, 10, 5, 7, 55, 38, tzinfo=timezone.utc),
+        )
+
+        # this is the same as hours=-3
+        del_ = datetime.timedelta(days=-1, seconds=75600)
+        eq_(
+            dt.astimezone(timezone(datetime.timedelta(hours=-3))),
+            datetime.datetime(2017, 10, 5, 4, 55, 38, tzinfo=timezone(del_)),
+        )
+
+    @testing.requires.python3
+    def test_repr_py3k(self):
+        eq_(
+            repr(timezone(datetime.timedelta(hours=5))),
+            "datetime.timezone(%r)" % (datetime.timedelta(hours=5)),
+        )
+
+    @testing.requires.python2
+    def test_repr_py2k(self):
+        eq_(
+            repr(timezone(datetime.timedelta(hours=5))),
+            "sqlalchemy.util.timezone(%r)" % (datetime.timedelta(hours=5)),
+        )
