@@ -1,5 +1,6 @@
 # -*- encoding: utf-8
 from sqlalchemy import Column
+from sqlalchemy import Computed
 from sqlalchemy import delete
 from sqlalchemy import extract
 from sqlalchemy import func
@@ -7,6 +8,7 @@ from sqlalchemy import Index
 from sqlalchemy import insert
 from sqlalchemy import Integer
 from sqlalchemy import literal
+from sqlalchemy import literal_column
 from sqlalchemy import MetaData
 from sqlalchemy import PrimaryKeyConstraint
 from sqlalchemy import schema
@@ -20,8 +22,8 @@ from sqlalchemy import union
 from sqlalchemy import UniqueConstraint
 from sqlalchemy import update
 from sqlalchemy.dialects import mssql
-from sqlalchemy.dialects.mssql import base
 from sqlalchemy.dialects.mssql import mxodbc
+from sqlalchemy.dialects.mssql.base import try_cast
 from sqlalchemy.sql import column
 from sqlalchemy.sql import quoted_name
 from sqlalchemy.sql import table
@@ -377,6 +379,36 @@ class CompileTest(fixtures.TestBase, AssertsCompiledSQL):
             "foo.myid = mytable.myid",
         )
 
+    def test_noorderby_parameters_insubquery(self):
+        """test that the ms-sql dialect does not include ORDER BY
+        positional parameters in subqueries"""
+
+        table1 = table(
+            "mytable",
+            column("myid", Integer),
+            column("name", String),
+            column("description", String),
+        )
+
+        q = select(
+            [table1.c.myid, sql.literal("bar").label("c1")],
+            order_by=[table1.c.name + "-"],
+        ).alias("foo")
+        crit = q.c.myid == table1.c.myid
+        dialect = mssql.dialect()
+        dialect.paramstyle = "qmark"
+        dialect.positional = True
+        self.assert_compile(
+            select(["*"], crit),
+            "SELECT * FROM (SELECT mytable.myid AS "
+            "myid, ? AS c1 FROM mytable) AS foo, mytable WHERE "
+            "foo.myid = mytable.myid",
+            dialect=dialect,
+            checkparams={"param_1": "bar"},
+            # if name_1 is included, too many parameters are passed to dbapi
+            checkpositional=("bar",),
+        )
+
     def test_force_schema_quoted_name_w_dot_case_insensitive(self):
         metadata = MetaData()
         tbl = Table(
@@ -449,21 +481,6 @@ class CompileTest(fixtures.TestBase, AssertsCompiledSQL):
             select([tbl]), "SELECT [Foo].dbo.test.id FROM [Foo].dbo.test"
         )
 
-    def test_owner_database_pairs(self):
-        dialect = mssql.dialect()
-
-        for identifier, expected_schema, expected_owner in [
-            ("foo", None, "foo"),
-            ("foo.bar", "foo", "bar"),
-            ("Foo.Bar", "Foo", "Bar"),
-            ("[Foo.Bar]", None, "Foo.Bar"),
-            ("[Foo.Bar].[bat]", "Foo.Bar", "bat"),
-        ]:
-            schema, owner = base._owner_plus_db(dialect, identifier)
-
-            eq_(owner, expected_owner)
-            eq_(schema, expected_schema)
-
     def test_delete_schema(self):
         metadata = MetaData()
         tbl = Table(
@@ -766,6 +783,28 @@ class CompileTest(fixtures.TestBase, AssertsCompiledSQL):
             eq_(len(c._result_columns), 2)
             assert t.c.x in set(c._create_result_map()["x"][1])
 
+    def test_simple_limit_expression_offset_using_window(self):
+        t = table("t", column("x", Integer), column("y", Integer))
+
+        s = (
+            select([t])
+            .where(t.c.x == 5)
+            .order_by(t.c.y)
+            .limit(10)
+            .offset(literal_column("20"))
+        )
+
+        self.assert_compile(
+            s,
+            "SELECT anon_1.x, anon_1.y "
+            "FROM (SELECT t.x AS x, t.y AS y, "
+            "ROW_NUMBER() OVER (ORDER BY t.y) AS mssql_rn "
+            "FROM t "
+            "WHERE t.x = :x_1) AS anon_1 "
+            "WHERE mssql_rn > 20 AND mssql_rn <= :param_1 + 20",
+            checkparams={"param_1": 10, "x_1": 5},
+        )
+
     def test_limit_offset_using_window(self):
         t = table("t", column("x", Integer), column("y", Integer))
 
@@ -879,12 +918,74 @@ class CompileTest(fixtures.TestBase, AssertsCompiledSQL):
             checkparams={"x_1": 5},
         )
 
-    def test_sequence_start_0(self):
+    def test_primary_key_no_identity(self):
         metadata = MetaData()
         tbl = Table(
             "test",
             metadata,
-            Column("id", Integer, Sequence("", 0), primary_key=True),
+            Column("id", Integer, autoincrement=False, primary_key=True),
+        )
+        self.assert_compile(
+            schema.CreateTable(tbl),
+            "CREATE TABLE test (id INTEGER NOT NULL, " "PRIMARY KEY (id))",
+        )
+
+    def test_primary_key_defaults_to_identity(self):
+        metadata = MetaData()
+        tbl = Table("test", metadata, Column("id", Integer, primary_key=True))
+        self.assert_compile(
+            schema.CreateTable(tbl),
+            "CREATE TABLE test (id INTEGER NOT NULL IDENTITY(1,1), "
+            "PRIMARY KEY (id))",
+        )
+
+    def test_identity_no_primary_key(self):
+        metadata = MetaData()
+        tbl = Table(
+            "test", metadata, Column("id", Integer, autoincrement=True)
+        )
+        self.assert_compile(
+            schema.CreateTable(tbl),
+            "CREATE TABLE test (id INTEGER NOT NULL IDENTITY(1,1)" ")",
+        )
+
+    def test_identity_separate_from_primary_key(self):
+        metadata = MetaData()
+        tbl = Table(
+            "test",
+            metadata,
+            Column("id", Integer, autoincrement=False, primary_key=True),
+            Column("x", Integer, autoincrement=True),
+        )
+        self.assert_compile(
+            schema.CreateTable(tbl),
+            "CREATE TABLE test (id INTEGER NOT NULL, "
+            "x INTEGER NOT NULL IDENTITY(1,1), "
+            "PRIMARY KEY (id))",
+        )
+
+    def test_identity_illegal_two_autoincrements(self):
+        metadata = MetaData()
+        tbl = Table(
+            "test",
+            metadata,
+            Column("id", Integer, autoincrement=True),
+            Column("id2", Integer, autoincrement=True),
+        )
+        # this will be rejected by the database, just asserting this is what
+        # the two autoincrements will do right now
+        self.assert_compile(
+            schema.CreateTable(tbl),
+            "CREATE TABLE test (id INTEGER NOT NULL IDENTITY(1,1), "
+            "id2 INTEGER NOT NULL IDENTITY(1,1))",
+        )
+
+    def test_identity_start_0(self):
+        metadata = MetaData()
+        tbl = Table(
+            "test",
+            metadata,
+            Column("id", Integer, mssql_identity_start=0, primary_key=True),
         )
         self.assert_compile(
             schema.CreateTable(tbl),
@@ -892,29 +993,66 @@ class CompileTest(fixtures.TestBase, AssertsCompiledSQL):
             "PRIMARY KEY (id))",
         )
 
+    def test_identity_increment_5(self):
+        metadata = MetaData()
+        tbl = Table(
+            "test",
+            metadata,
+            Column(
+                "id", Integer, mssql_identity_increment=5, primary_key=True
+            ),
+        )
+        self.assert_compile(
+            schema.CreateTable(tbl),
+            "CREATE TABLE test (id INTEGER NOT NULL IDENTITY(1,5), "
+            "PRIMARY KEY (id))",
+        )
+
+    def test_sequence_start_0(self):
+        metadata = MetaData()
+        tbl = Table(
+            "test",
+            metadata,
+            Column("id", Integer, Sequence("", 0), primary_key=True),
+        )
+        with testing.expect_deprecated(
+            "Use of Sequence with SQL Server in order to affect "
+        ):
+            self.assert_compile(
+                schema.CreateTable(tbl),
+                "CREATE TABLE test (id INTEGER NOT NULL IDENTITY(0,1), "
+                "PRIMARY KEY (id))",
+            )
+
     def test_sequence_non_primary_key(self):
         metadata = MetaData()
         tbl = Table(
             "test",
             metadata,
-            Column("id", Integer, Sequence(""), primary_key=False),
-        )
-        self.assert_compile(
-            schema.CreateTable(tbl),
-            "CREATE TABLE test (id INTEGER NOT NULL IDENTITY(1,1))",
+            Column("id", Integer, Sequence("", start=5), primary_key=False),
         )
+        with testing.expect_deprecated(
+            "Use of Sequence with SQL Server in order to affect "
+        ):
+            self.assert_compile(
+                schema.CreateTable(tbl),
+                "CREATE TABLE test (id INTEGER NOT NULL IDENTITY(5,1))",
+            )
 
     def test_sequence_ignore_nullability(self):
         metadata = MetaData()
         tbl = Table(
             "test",
             metadata,
-            Column("id", Integer, Sequence(""), nullable=True),
-        )
-        self.assert_compile(
-            schema.CreateTable(tbl),
-            "CREATE TABLE test (id INTEGER NOT NULL IDENTITY(1,1))",
+            Column("id", Integer, Sequence("", start=5), nullable=True),
         )
+        with testing.expect_deprecated(
+            "Use of Sequence with SQL Server in order to affect "
+        ):
+            self.assert_compile(
+                schema.CreateTable(tbl),
+                "CREATE TABLE test (id INTEGER NOT NULL IDENTITY(5,1))",
+            )
 
     def test_table_pkc_clustering(self):
         metadata = MetaData()
@@ -1000,6 +1138,15 @@ class CompileTest(fixtures.TestBase, AssertsCompiledSQL):
             schema.CreateIndex(idx), "CREATE CLUSTERED INDEX foo ON test (id)"
         )
 
+    def test_index_where(self):
+        metadata = MetaData()
+        tbl = Table("test", metadata, Column("data", Integer))
+        idx = Index("test_idx_data_1", tbl.c.data, mssql_where=tbl.c.data > 1)
+        self.assert_compile(
+            schema.CreateIndex(idx),
+            "CREATE INDEX test_idx_data_1 ON test (data) WHERE data > 1",
+        )
+
     def test_index_ordering(self):
         metadata = MetaData()
         tbl = Table(
@@ -1058,6 +1205,36 @@ class CompileTest(fixtures.TestBase, AssertsCompiledSQL):
             schema.CreateIndex(idx), "CREATE INDEX foo ON test (x) INCLUDE (y)"
         )
 
+    def test_try_cast(self):
+        metadata = MetaData()
+        t1 = Table("t1", metadata, Column("id", Integer, primary_key=True))
+
+        self.assert_compile(
+            select([try_cast(t1.c.id, Integer)]),
+            "SELECT TRY_CAST (t1.id AS INTEGER) AS anon_1 FROM t1",
+        )
+
+    @testing.combinations(
+        ("no_persisted", "", "ignore"),
+        ("persisted_none", "", None),
+        ("persisted_true", " PERSISTED", True),
+        ("persisted_false", "", False),
+        id_="iaa",
+    )
+    def test_column_computed(self, text, persisted):
+        m = MetaData()
+        kwargs = {"persisted": persisted} if persisted != "ignore" else {}
+        t = Table(
+            "t",
+            m,
+            Column("x", Integer),
+            Column("y", Integer, Computed("x + 2", **kwargs)),
+        )
+        self.assert_compile(
+            schema.CreateTable(t),
+            "CREATE TABLE t (x INTEGER NULL, y AS (x + 2)%s)" % text,
+        )
+
 
 class SchemaTest(fixtures.TestBase):
     def setup(self):
