@@ -3,12 +3,14 @@
 
 from sqlalchemy import and_
 from sqlalchemy import bindparam
+from sqlalchemy import Computed
 from sqlalchemy import exc
 from sqlalchemy import except_
 from sqlalchemy import ForeignKey
 from sqlalchemy import func
 from sqlalchemy import Index
 from sqlalchemy import Integer
+from sqlalchemy import literal_column
 from sqlalchemy import MetaData
 from sqlalchemy import or_
 from sqlalchemy import outerjoin
@@ -17,6 +19,7 @@ from sqlalchemy import select
 from sqlalchemy import Sequence
 from sqlalchemy import sql
 from sqlalchemy import String
+from sqlalchemy import testing
 from sqlalchemy import text
 from sqlalchemy import type_coerce
 from sqlalchemy import TypeDecorator
@@ -25,6 +28,7 @@ from sqlalchemy.dialects.oracle import base as oracle
 from sqlalchemy.dialects.oracle import cx_oracle
 from sqlalchemy.engine import default
 from sqlalchemy.sql import column
+from sqlalchemy.sql import quoted_name
 from sqlalchemy.sql import table
 from sqlalchemy.testing import assert_raises_message
 from sqlalchemy.testing import AssertsCompiledSQL
@@ -211,7 +215,7 @@ class CompileTest(fixtures.TestBase, AssertsCompiledSQL):
         eq_(len(c._result_columns), 2)
         assert t.c.col1 in set(c._create_result_map()["col1"][1])
 
-        s = select([t], for_update=True).limit(10).order_by(t.c.col2)
+        s = select([t]).with_for_update().limit(10).order_by(t.c.col2)
         self.assert_compile(
             s,
             "SELECT col1, col2 FROM (SELECT "
@@ -222,7 +226,8 @@ class CompileTest(fixtures.TestBase, AssertsCompiledSQL):
         )
 
         s = (
-            select([t], for_update=True)
+            select([t])
+            .with_for_update()
             .limit(10)
             .offset(20)
             .order_by(t.c.col2)
@@ -238,6 +243,62 @@ class CompileTest(fixtures.TestBase, AssertsCompiledSQL):
             "UPDATE",
         )
 
+    def test_limit_special_quoting(self):
+        """Oracle-specific test for #4730.
+
+        Even though this issue is generic, test the originally reported Oracle
+        use case.
+
+        """
+
+        col = literal_column("SUM(ABC)").label("SUM(ABC)")
+        tbl = table("my_table")
+        query = select([col]).select_from(tbl).order_by(col).limit(100)
+
+        self.assert_compile(
+            query,
+            'SELECT "SUM(ABC)" FROM '
+            '(SELECT SUM(ABC) AS "SUM(ABC)" '
+            "FROM my_table ORDER BY SUM(ABC)) "
+            "WHERE ROWNUM <= :param_1",
+        )
+
+        col = literal_column("SUM(ABC)").label(quoted_name("SUM(ABC)", True))
+        tbl = table("my_table")
+        query = select([col]).select_from(tbl).order_by(col).limit(100)
+
+        self.assert_compile(
+            query,
+            'SELECT "SUM(ABC)" FROM '
+            '(SELECT SUM(ABC) AS "SUM(ABC)" '
+            "FROM my_table ORDER BY SUM(ABC)) "
+            "WHERE ROWNUM <= :param_1",
+        )
+
+        col = literal_column("SUM(ABC)").label("SUM(ABC)_")
+        tbl = table("my_table")
+        query = select([col]).select_from(tbl).order_by(col).limit(100)
+
+        self.assert_compile(
+            query,
+            'SELECT "SUM(ABC)_" FROM '
+            '(SELECT SUM(ABC) AS "SUM(ABC)_" '
+            "FROM my_table ORDER BY SUM(ABC)) "
+            "WHERE ROWNUM <= :param_1",
+        )
+
+        col = literal_column("SUM(ABC)").label(quoted_name("SUM(ABC)_", True))
+        tbl = table("my_table")
+        query = select([col]).select_from(tbl).order_by(col).limit(100)
+
+        self.assert_compile(
+            query,
+            'SELECT "SUM(ABC)_" FROM '
+            '(SELECT SUM(ABC) AS "SUM(ABC)_" '
+            "FROM my_table ORDER BY SUM(ABC)) "
+            "WHERE ROWNUM <= :param_1",
+        )
+
     def test_for_update(self):
         table1 = table(
             "mytable", column("myid"), column("name"), column("description")
@@ -849,6 +910,40 @@ class CompileTest(fixtures.TestBase, AssertsCompiledSQL):
             "t1.c2, t1.c3 INTO :ret_0, :ret_1",
         )
 
+    def test_returning_insert_computed(self):
+        m = MetaData()
+        t1 = Table(
+            "t1",
+            m,
+            Column("id", Integer, primary_key=True),
+            Column("foo", Integer),
+            Column("bar", Integer, Computed("foo + 42")),
+        )
+
+        self.assert_compile(
+            t1.insert().values(id=1, foo=5).returning(t1.c.bar),
+            "INSERT INTO t1 (id, foo) VALUES (:id, :foo) "
+            "RETURNING t1.bar INTO :ret_0",
+        )
+
+    def test_returning_update_computed_warning(self):
+        m = MetaData()
+        t1 = Table(
+            "t1",
+            m,
+            Column("id", Integer, primary_key=True),
+            Column("foo", Integer),
+            Column("bar", Integer, Computed("foo + 42")),
+        )
+
+        with testing.expect_warnings(
+            "Computed columns don't work with Oracle UPDATE"
+        ):
+            self.assert_compile(
+                t1.update().values(id=1, foo=5).returning(t1.c.bar),
+                "UPDATE t1 SET id=:id, foo=:foo RETURNING t1.bar INTO :ret_0",
+            )
+
     def test_compound(self):
         t1 = table("t1", column("c1"), column("c2"), column("c3"))
         t2 = table("t2", column("c1"), column("c2"), column("c3"))
@@ -947,6 +1042,42 @@ class CompileTest(fixtures.TestBase, AssertsCompiledSQL):
             "CREATE BITMAP INDEX idx3 ON testtbl (data)",
         )
 
+    @testing.combinations(
+        ("no_persisted", "", "ignore"),
+        ("persisted_none", "", None),
+        ("persisted_false", " VIRTUAL", False),
+        id_="iaa",
+    )
+    def test_column_computed(self, text, persisted):
+        m = MetaData()
+        kwargs = {"persisted": persisted} if persisted != "ignore" else {}
+        t = Table(
+            "t",
+            m,
+            Column("x", Integer),
+            Column("y", Integer, Computed("x + 2", **kwargs)),
+        )
+        self.assert_compile(
+            schema.CreateTable(t),
+            "CREATE TABLE t (x INTEGER, y INTEGER GENERATED "
+            "ALWAYS AS (x + 2)%s)" % text,
+        )
+
+    def test_column_computed_persisted_true(self):
+        m = MetaData()
+        t = Table(
+            "t",
+            m,
+            Column("x", Integer),
+            Column("y", Integer, Computed("x + 2", persisted=True)),
+        )
+        assert_raises_message(
+            exc.CompileError,
+            r".*Oracle computed columns do not support 'stored' ",
+            schema.CreateTable(t).compile,
+            dialect=oracle.dialect(),
+        )
+
 
 class SequenceTest(fixtures.TestBase, AssertsCompiledSQL):
     def test_basic(self):
