@@ -49,7 +49,7 @@ class ForeignTableReflectionTest(fixtures.TablesTest, AssertsExecutionResults):
             "sqla_testing", "postgres_test_db_link"
         )
 
-        testtable = Table(
+        Table(
             "testtable",
             metadata,
             Column("id", Integer, primary_key=True),
@@ -86,17 +86,76 @@ class ForeignTableReflectionTest(fixtures.TablesTest, AssertsExecutionResults):
 
     def test_get_foreign_table_names(self):
         inspector = inspect(testing.db)
-        with testing.db.connect() as conn:
+        with testing.db.connect():
             ft_names = inspector.get_foreign_table_names()
             eq_(ft_names, ["test_foreigntable"])
 
     def test_get_table_names_no_foreign(self):
         inspector = inspect(testing.db)
-        with testing.db.connect() as conn:
+        with testing.db.connect():
             names = inspector.get_table_names()
             eq_(names, ["testtable"])
 
 
+class PartitionedReflectionTest(fixtures.TablesTest, AssertsExecutionResults):
+    # partitioned table reflection, issue #4237
+
+    __only_on__ = "postgresql >= 10"
+    __backend__ = True
+
+    @classmethod
+    def define_tables(cls, metadata):
+        # the actual function isn't reflected yet
+        dv = Table(
+            "data_values",
+            metadata,
+            Column("modulus", Integer, nullable=False),
+            Column("data", String(30)),
+            Column("q", Integer),
+            postgresql_partition_by="range(modulus)",
+        )
+
+        # looks like this is reflected prior to #4237
+        sa.event.listen(
+            dv,
+            "after_create",
+            sa.DDL(
+                "CREATE TABLE data_values_4_10 PARTITION OF data_values "
+                "FOR VALUES FROM (4) TO (10)"
+            ),
+        )
+
+        if testing.against("postgresql >= 11"):
+            Index("my_index", dv.c.q)
+
+    def test_get_tablenames(self):
+        assert {"data_values", "data_values_4_10"}.issubset(
+            inspect(testing.db).get_table_names()
+        )
+
+    def test_reflect_cols(self):
+        cols = inspect(testing.db).get_columns("data_values")
+        eq_([c["name"] for c in cols], ["modulus", "data", "q"])
+
+    def test_reflect_cols_from_partition(self):
+        cols = inspect(testing.db).get_columns("data_values_4_10")
+        eq_([c["name"] for c in cols], ["modulus", "data", "q"])
+
+    @testing.only_on("postgresql >= 11")
+    def test_reflect_index(self):
+        idx = inspect(testing.db).get_indexes("data_values")
+        eq_(
+            idx, [{"column_names": ["q"], "name": "my_index", "unique": False}]
+        )
+
+    @testing.only_on("postgresql >= 11")
+    def test_reflect_index_from_partition(self):
+        idx = inspect(testing.db).get_indexes("data_values_4_10")
+        # note the name appears to be generated by PG, currently
+        # 'data_values_4_10_q_idx'
+        eq_(idx, [{"column_names": ["q"], "name": mock.ANY, "unique": False}])
+
+
 class MaterializedViewReflectionTest(
     fixtures.TablesTest, AssertsExecutionResults
 ):
@@ -173,6 +232,17 @@ class MaterializedViewReflectionTest(
             set(["test_mview"]),
         )
 
+    def test_get_view_names_reflection_cache_ok(self):
+        insp = inspect(testing.db)
+        eq_(
+            set(insp.get_view_names(include=("plain",))), set(["test_regview"])
+        )
+        eq_(
+            set(insp.get_view_names(include=("materialized",))),
+            set(["test_mview"]),
+        )
+        eq_(set(insp.get_view_names()), set(["test_regview", "test_mview"]))
+
     def test_get_view_names_empty(self):
         insp = inspect(testing.db)
         assert_raises(ValueError, insp.get_view_names, include=())
@@ -409,7 +479,7 @@ class ReflectionTest(fixtures.TestBase):
     @testing.provide_metadata
     def test_renamed_sequence_reflection(self):
         metadata = self.metadata
-        t = Table("t", metadata, Column("id", Integer, primary_key=True))
+        Table("t", metadata, Column("id", Integer, primary_key=True))
         metadata.create_all()
         m2 = MetaData(testing.db)
         t2 = Table("t", m2, autoload=True, implicit_returning=False)
@@ -431,7 +501,7 @@ class ReflectionTest(fixtures.TestBase):
     @testing.provide_metadata
     def test_altered_type_autoincrement_pk_reflection(self):
         metadata = self.metadata
-        t = Table(
+        Table(
             "t",
             metadata,
             Column("id", Integer, primary_key=True),
@@ -449,7 +519,7 @@ class ReflectionTest(fixtures.TestBase):
     @testing.provide_metadata
     def test_renamed_pk_reflection(self):
         metadata = self.metadata
-        t = Table("t", metadata, Column("id", Integer, primary_key=True))
+        Table("t", metadata, Column("id", Integer, primary_key=True))
         metadata.create_all()
         testing.db.connect().execution_options(autocommit=True).execute(
             "alter table t rename id to t_id"
@@ -664,9 +734,7 @@ class ReflectionTest(fixtures.TestBase):
 
             m1 = MetaData(conn)
 
-            t1_schema = Table(
-                "some_table", m1, schema="test_schema", autoload=True
-            )
+            Table("some_table", m1, schema="test_schema", autoload=True)
             t2_schema = Table(
                 "some_other_table", m1, schema="test_schema_2", autoload=True
             )
@@ -814,7 +882,7 @@ class ReflectionTest(fixtures.TestBase):
 
         metadata = self.metadata
 
-        t1 = Table(
+        Table(
             "party",
             metadata,
             Column("id", String(10), nullable=False),
@@ -866,6 +934,83 @@ class ReflectionTest(fixtures.TestBase):
             ],
         )
 
+    @testing.fails_if("postgresql < 8.3", "index ordering not supported")
+    @testing.provide_metadata
+    def test_index_reflection_with_sorting(self):
+        """reflect indexes with sorting options set"""
+
+        t1 = Table(
+            "party",
+            self.metadata,
+            Column("id", String(10), nullable=False),
+            Column("name", String(20)),
+            Column("aname", String(20)),
+        )
+
+        with testing.db.connect() as conn:
+
+            t1.create(conn)
+
+            # check ASC, DESC options alone
+            conn.execute(
+                """
+                create index idx1 on party
+                    (id, name ASC, aname DESC)
+            """
+            )
+
+            # check DESC w/ NULLS options
+            conn.execute(
+                """
+              create index idx2 on party
+                    (name DESC NULLS FIRST, aname DESC NULLS LAST)
+            """
+            )
+
+            # check ASC w/ NULLS options
+            conn.execute(
+                """
+              create index idx3 on party
+                    (name ASC NULLS FIRST, aname ASC NULLS LAST)
+            """
+            )
+
+        # reflect data
+        with testing.db.connect() as conn:
+            m2 = MetaData(conn)
+            t2 = Table("party", m2, autoload=True)
+
+        eq_(len(t2.indexes), 3)
+
+        # Make sure indexes are in the order we expect them in
+        r1, r2, r3 = sorted(t2.indexes, key=lambda idx: idx.name)
+
+        eq_(r1.name, "idx1")
+        eq_(r2.name, "idx2")
+        eq_(r3.name, "idx3")
+
+        # "ASC NULLS LAST" is implicit default for indexes,
+        # and "NULLS FIRST" is implicit default for "DESC".
+        # (https://www.postgresql.org/docs/11/indexes-ordering.html)
+
+        def compile_exprs(exprs):
+            return list(map(str, exprs))
+
+        eq_(
+            compile_exprs([t2.c.id, t2.c.name, t2.c.aname.desc()]),
+            compile_exprs(r1.expressions),
+        )
+
+        eq_(
+            compile_exprs([t2.c.name.desc(), t2.c.aname.desc().nullslast()]),
+            compile_exprs(r2.expressions),
+        )
+
+        eq_(
+            compile_exprs([t2.c.name.nullsfirst(), t2.c.aname]),
+            compile_exprs(r3.expressions),
+        )
+
     @testing.provide_metadata
     def test_index_reflection_modified(self):
         """reflect indexes when a column name has changed - PG 9
@@ -876,7 +1021,7 @@ class ReflectionTest(fixtures.TestBase):
 
         metadata = self.metadata
 
-        t1 = Table(
+        Table(
             "t",
             metadata,
             Column("id", Integer, primary_key=True),
@@ -1249,6 +1394,33 @@ class ReflectionTest(fixtures.TestBase):
             ],
         )
 
+    @testing.provide_metadata
+    def test_inspect_enum_empty(self):
+        enum_type = postgresql.ENUM(name="empty", metadata=self.metadata)
+        enum_type.create(testing.db)
+        inspector = reflection.Inspector.from_engine(testing.db)
+
+        eq_(
+            inspector.get_enums(),
+            [
+                {
+                    "visible": True,
+                    "labels": [],
+                    "name": "empty",
+                    "schema": "public",
+                }
+            ],
+        )
+
+    @testing.provide_metadata
+    def test_inspect_enum_empty_from_table(self):
+        Table(
+            "t", self.metadata, Column("x", postgresql.ENUM(name="empty"))
+        ).create(testing.db)
+
+        t = Table("t", MetaData(testing.db), autoload_with=testing.db)
+        eq_(t.c.x.type.enums, [])
+
     @testing.provide_metadata
     @testing.only_on("postgresql >= 8.5")
     def test_reflection_with_unique_constraint(self):
@@ -1385,9 +1557,10 @@ class ReflectionTest(fixtures.TestBase):
         )
 
     def test_reflect_check_warning(self):
+        rows = [("some name", "NOTCHECK foobar")]
         conn = mock.Mock(
-            execute=lambda *arg, **kw: mock.Mock(
-                fetchall=lambda: [("some name", "NOTCHECK foobar")]
+            execute=lambda *arg, **kw: mock.MagicMock(
+                fetchall=lambda: rows, __iter__=lambda self: iter(rows)
             )
         )
         with mock.patch.object(
@@ -1398,6 +1571,30 @@ class ReflectionTest(fixtures.TestBase):
             ):
                 testing.db.dialect.get_check_constraints(conn, "foo")
 
+    def test_reflect_with_not_valid_check_constraint(self):
+        rows = [("some name", "CHECK ((a IS NOT NULL)) NOT VALID")]
+        conn = mock.Mock(
+            execute=lambda *arg, **kw: mock.MagicMock(
+                fetchall=lambda: rows, __iter__=lambda self: iter(rows)
+            )
+        )
+        with mock.patch.object(
+            testing.db.dialect, "get_table_oid", lambda *arg, **kw: 1
+        ):
+            check_constraints = testing.db.dialect.get_check_constraints(
+                conn, "foo"
+            )
+            eq_(
+                check_constraints,
+                [
+                    {
+                        "name": "some name",
+                        "sqltext": "a IS NOT NULL",
+                        "dialect_options": {"not_valid": True},
+                    }
+                ],
+            )
+
 
 class CustomTypeReflectionTest(fixtures.TestBase):
     class CustomType(object):
