@@ -1,9 +1,17 @@
 import logging.handlers
 
 import sqlalchemy as tsa
+from sqlalchemy import bindparam
+from sqlalchemy import Column
+from sqlalchemy import MetaData
+from sqlalchemy import or_
 from sqlalchemy import select
+from sqlalchemy import String
+from sqlalchemy import Table
 from sqlalchemy import util
+from sqlalchemy.sql import util as sql_util
 from sqlalchemy.testing import assert_raises_message
+from sqlalchemy.testing import assert_raises_return
 from sqlalchemy.testing import engines
 from sqlalchemy.testing import eq_
 from sqlalchemy.testing import eq_regex
@@ -18,7 +26,11 @@ class LogParamsTest(fixtures.TestBase):
 
     def setup(self):
         self.eng = engines.testing_engine(options={"echo": True})
+        self.no_param_engine = engines.testing_engine(
+            options={"echo": True, "hide_parameters": True}
+        )
         self.eng.execute("create table foo (data string)")
+        self.no_param_engine.execute("create table foo (data string)")
         self.buf = logging.handlers.BufferingHandler(100)
         for log in [logging.getLogger("sqlalchemy.engine")]:
             log.addHandler(self.buf)
@@ -28,7 +40,7 @@ class LogParamsTest(fixtures.TestBase):
         for log in [logging.getLogger("sqlalchemy.engine")]:
             log.removeHandler(self.buf)
 
-    def test_log_large_dict(self):
+    def test_log_large_list_of_dict(self):
         self.eng.execute(
             "INSERT INTO foo (data) values (:data)",
             [{"data": str(i)} for i in range(100)],
@@ -41,7 +53,32 @@ class LogParamsTest(fixtures.TestBase):
             "parameter sets ...  {'data': '98'}, {'data': '99'}]",
         )
 
-    def test_log_large_list(self):
+    def test_repr_params_large_list_of_dict(self):
+        eq_(
+            repr(
+                sql_util._repr_params(
+                    [{"data": str(i)} for i in range(100)],
+                    batches=10,
+                    ismulti=True,
+                )
+            ),
+            "[{'data': '0'}, {'data': '1'}, {'data': '2'}, {'data': '3'}, "
+            "{'data': '4'}, {'data': '5'}, {'data': '6'}, {'data': '7'}"
+            "  ... displaying 10 of 100 total bound "
+            "parameter sets ...  {'data': '98'}, {'data': '99'}]",
+        )
+
+    def test_log_no_parameters(self):
+        self.no_param_engine.execute(
+            "INSERT INTO foo (data) values (:data)",
+            [{"data": str(i)} for i in range(100)],
+        )
+        eq_(
+            self.buf.buffer[1].message,
+            "[SQL parameters hidden due to hide_parameters=True]",
+        )
+
+    def test_log_large_list_of_tuple(self):
         self.eng.execute(
             "INSERT INTO foo (data) values (?)",
             [(str(i),) for i in range(100)],
@@ -53,6 +90,119 @@ class LogParamsTest(fixtures.TestBase):
             "bound parameter sets ...  ('98',), ('99',)]",
         )
 
+    def test_log_positional_array(self):
+        with self.eng.connect() as conn:
+            exc_info = assert_raises_return(
+                tsa.exc.DBAPIError,
+                conn.execute,
+                tsa.text("SELECT * FROM foo WHERE id IN :foo AND bar=:bar"),
+                {"foo": [1, 2, 3], "bar": "hi"},
+            )
+
+            assert (
+                "[SQL: SELECT * FROM foo WHERE id IN ? AND bar=?]\n"
+                "[parameters: ([1, 2, 3], 'hi')]\n" in str(exc_info)
+            )
+
+            eq_(self.buf.buffer[1].message, "([1, 2, 3], 'hi')")
+
+    def test_repr_params_positional_array(self):
+        eq_(
+            repr(
+                sql_util._repr_params(
+                    [[1, 2, 3], 5], batches=10, ismulti=False
+                )
+            ),
+            "[[1, 2, 3], 5]",
+        )
+
+    def test_repr_params_unknown_list(self):
+        # not known if given multiparams or not.   repr params with
+        # straight truncation
+        eq_(
+            repr(
+                sql_util._repr_params(
+                    [[i for i in range(300)], 5], batches=10, max_chars=80
+                )
+            ),
+            "[[0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11,  ... "
+            "(1315 characters truncated) ... , 293, 294, 295, 296, "
+            "297, 298, 299], 5]",
+        )
+
+    def test_repr_params_positional_list(self):
+        # given non-multi-params in a list.   repr params with
+        # per-element truncation, mostly does the exact same thing
+        eq_(
+            repr(
+                sql_util._repr_params(
+                    [[i for i in range(300)], 5],
+                    batches=10,
+                    max_chars=80,
+                    ismulti=False,
+                )
+            ),
+            "[[0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 1 ... "
+            "(1310 characters truncated) ...  "
+            "292, 293, 294, 295, 296, 297, 298, 299], 5]",
+        )
+
+    def test_repr_params_named_dict(self):
+        # given non-multi-params in a list.   repr params with
+        # per-element truncation, mostly does the exact same thing
+        params = {"key_%s" % i: i for i in range(10)}
+        eq_(
+            repr(
+                sql_util._repr_params(
+                    params, batches=10, max_chars=80, ismulti=False
+                )
+            ),
+            repr(params),
+        )
+
+    def test_repr_params_ismulti_named_dict(self):
+        # given non-multi-params in a list.   repr params with
+        # per-element truncation, mostly does the exact same thing
+        param = {"key_%s" % i: i for i in range(10)}
+        eq_(
+            repr(
+                sql_util._repr_params(
+                    [param for j in range(50)],
+                    batches=5,
+                    max_chars=80,
+                    ismulti=True,
+                )
+            ),
+            "[%(param)r, %(param)r, %(param)r  ... "
+            "displaying 5 of 50 total bound parameter sets ...  "
+            "%(param)r, %(param)r]" % {"param": param},
+        )
+
+    def test_repr_params_ismulti_list(self):
+        # given multi-params in a list.   repr params with
+        # per-element truncation, mostly does the exact same thing
+        eq_(
+            repr(
+                sql_util._repr_params(
+                    [
+                        [[i for i in range(300)], 5],
+                        [[i for i in range(300)], 5],
+                        [[i for i in range(300)], 5],
+                    ],
+                    batches=10,
+                    max_chars=80,
+                    ismulti=True,
+                )
+            ),
+            "[[[0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 1 ... "
+            "(1310 characters truncated) ...  292, 293, 294, 295, 296, 297, "
+            "298, 299], 5], [[0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 1 ... "
+            "(1310 characters truncated) ...  292, 293, 294, 295, 296, 297, "
+            "298, 299], 5], [[0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 1 ... "
+            "(1310 characters truncated) ...  292, 293, 294, 295, 296, 297, "
+            "298, 299], 5]]",
+        )
+
     def test_log_large_parameter_single(self):
         import random
 
@@ -103,16 +253,58 @@ class LogParamsTest(fixtures.TestBase):
         exception = tsa.exc.IntegrityError("foo", {"x": "y"}, None)
         eq_regex(
             str(exception),
-            r"\(.*.NoneType\) None \[SQL: 'foo'\] \[parameters: {'x': 'y'}\]",
+            r"\(.*.NoneType\) None\n\[SQL: foo\]\n\[parameters: {'x': 'y'}\]",
         )
 
+    def test_exception_format_hide_parameters(self):
+        exception = tsa.exc.IntegrityError(
+            "foo", {"x": "y"}, None, hide_parameters=True
+        )
+        eq_regex(
+            str(exception),
+            r"\(.*.NoneType\) None\n\[SQL: foo\]\n"
+            r"\[SQL parameters hidden due to hide_parameters=True\]",
+        )
+
+    def test_exception_format_hide_parameters_dbapi_round_trip(self):
+        assert_raises_message(
+            tsa.exc.DBAPIError,
+            r".*INSERT INTO nonexistent \(data\) values \(:data\)\]\n"
+            r"\[SQL parameters hidden due to hide_parameters=True\]",
+            lambda: self.no_param_engine.execute(
+                "INSERT INTO nonexistent (data) values (:data)",
+                [{"data": str(i)} for i in range(10)],
+            ),
+        )
+
+    def test_exception_format_hide_parameters_nondbapi_round_trip(self):
+        foo = Table("foo", MetaData(), Column("data", String))
+
+        with self.no_param_engine.connect() as conn:
+            assert_raises_message(
+                tsa.exc.StatementError,
+                r"\(sqlalchemy.exc.InvalidRequestError\) A value is required "
+                r"for bind parameter 'the_data_2'\n"
+                r"\[SQL: SELECT foo.data \nFROM foo \nWHERE "
+                r"foo.data = \? OR foo.data = \?\]\n"
+                r"\[SQL parameters hidden due to hide_parameters=True\]",
+                conn.execute,
+                select([foo]).where(
+                    or_(
+                        foo.c.data == bindparam("the_data_1"),
+                        foo.c.data == bindparam("the_data_2"),
+                    )
+                ),
+                {"the_data_1": "some data"},
+            )
+
     def test_exception_format_unexpected_parameter(self):
         # test that if the parameters aren't any known type, we just
         # run through repr()
         exception = tsa.exc.IntegrityError("foo", "bar", "bat")
         eq_regex(
             str(exception),
-            r"\(.*.str\) bat \[SQL: 'foo'\] \[parameters: 'bar'\]",
+            r"\(.*.str\) bat\n\[SQL: foo\]\n\[parameters: 'bar'\]",
         )
 
     def test_exception_format_unexpected_member_parameter(self):
@@ -121,7 +313,7 @@ class LogParamsTest(fixtures.TestBase):
         exception = tsa.exc.IntegrityError("foo", ["bar", "bat"], "hoho")
         eq_regex(
             str(exception),
-            r"\(.*.str\) hoho \[SQL: 'foo'\] \[parameters: \['bar', 'bat'\]\]",
+            r"\(.*.str\) hoho\n\[SQL: foo\]\n\[parameters: \['bar', 'bat'\]\]",
         )
 
     def test_result_large_param(self):
@@ -169,7 +361,7 @@ class LogParamsTest(fixtures.TestBase):
     def test_error_large_dict(self):
         assert_raises_message(
             tsa.exc.DBAPIError,
-            r".*'INSERT INTO nonexistent \(data\) values \(:data\)'\] "
+            r".*INSERT INTO nonexistent \(data\) values \(:data\)\]\n"
             r"\[parameters: "
             r"\[{'data': '0'}, {'data': '1'}, {'data': '2'}, "
             r"{'data': '3'}, {'data': '4'}, {'data': '5'}, "
@@ -186,7 +378,7 @@ class LogParamsTest(fixtures.TestBase):
         assert_raises_message(
             tsa.exc.DBAPIError,
             r".*INSERT INTO nonexistent \(data\) values "
-            r"\(\?\)'\] \[parameters: \[\('0',\), \('1',\), \('2',\), "
+            r"\(\?\)\]\n\[parameters: \[\('0',\), \('1',\), \('2',\), "
             r"\('3',\), \('4',\), \('5',\), \('6',\), \('7',\)  "
             r"... displaying "
             r"10 of 100 total bound parameter sets ...  "
@@ -284,7 +476,7 @@ class LoggingNameTest(fixtures.TestBase):
         for name in [b.name for b in self.buf.buffer]:
             assert name in (
                 "sqlalchemy.engine.base.Engine.%s" % eng_name,
-                "sqlalchemy.pool.%s.%s"
+                "sqlalchemy.pool.impl.%s.%s"
                 % (eng.pool.__class__.__name__, pool_name),
             )
 
@@ -294,7 +486,7 @@ class LoggingNameTest(fixtures.TestBase):
         for name in [b.name for b in self.buf.buffer]:
             assert name in (
                 "sqlalchemy.engine.base.Engine",
-                "sqlalchemy.pool.%s" % eng.pool.__class__.__name__,
+                "sqlalchemy.pool.impl.%s" % eng.pool.__class__.__name__,
             )
 
     def _named_engine(self, **kw):
