@@ -18,6 +18,8 @@ from sqlalchemy.testing import engines
 from sqlalchemy.testing import eq_
 from sqlalchemy.testing import expect_warnings
 from sqlalchemy.testing import fixtures
+from sqlalchemy.testing import is_false
+from sqlalchemy.testing import is_true
 from sqlalchemy.testing import mock
 from sqlalchemy.testing import ne_
 from sqlalchemy.testing.engines import testing_engine
@@ -286,8 +288,6 @@ class MockReconnectTest(fixtures.TestBase):
         connection, and additionally dispose the previous connection
         pool and recreate."""
 
-        db_pool = self.db.pool
-
         # make a connection
 
         conn = self.db.connect()
@@ -540,8 +540,6 @@ class MockReconnectTest(fixtures.TestBase):
 
         dbapi = self.dbapi
 
-        mock_dialect = Mock()
-
         class MyURL(URL):
             def _get_entrypoint(self):
                 return Dialect
@@ -553,11 +551,79 @@ class MockReconnectTest(fixtures.TestBase):
             initialize = Mock()
 
         engine = create_engine(MyURL("foo://"), module=dbapi)
-        c1 = engine.connect()
+        engine.connect()
+
+        # note that the dispose() call replaces the old pool with a new one;
+        # this is to test that even though a single pool is using
+        # dispatch.exec_once(), by replacing the pool with a new one, the event
+        # would normally fire again onless once=True is set on the original
+        # listen as well.
+
         engine.dispose()
-        c2 = engine.connect()
+        engine.connect()
         eq_(Dialect.initialize.call_count, 1)
 
+    def test_dialect_initialize_retry_if_exception(self):
+        from sqlalchemy.engine.url import URL
+        from sqlalchemy.engine.default import DefaultDialect
+
+        dbapi = self.dbapi
+
+        class MyURL(URL):
+            def _get_entrypoint(self):
+                return Dialect
+
+            def get_dialect(self):
+                return Dialect
+
+        class Dialect(DefaultDialect):
+            initialize = Mock()
+
+        # note that the first_connect hook is only invoked when the pool
+        # makes a new DBAPI connection, and not when it checks out an existing
+        # connection.  So there is a dependency here that if the initializer
+        # raises an exception, the pool-level connection attempt is also
+        # failed, meaning no DBAPI connection is pooled.  If the first_connect
+        # exception raise did not prevent the connection from being pooled,
+        # there could be the case where the pool could return that connection
+        # on a subsequent attempt without initialization having proceeded.
+
+        Dialect.initialize.side_effect = TypeError
+        engine = create_engine(MyURL("foo://"), module=dbapi)
+
+        assert_raises(TypeError, engine.connect)
+        eq_(Dialect.initialize.call_count, 1)
+        is_true(engine.pool._pool.empty())
+
+        assert_raises(TypeError, engine.connect)
+        eq_(Dialect.initialize.call_count, 2)
+        is_true(engine.pool._pool.empty())
+
+        engine.dispose()
+
+        assert_raises(TypeError, engine.connect)
+        eq_(Dialect.initialize.call_count, 3)
+        is_true(engine.pool._pool.empty())
+
+        Dialect.initialize.side_effect = None
+
+        conn = engine.connect()
+        eq_(Dialect.initialize.call_count, 4)
+        conn.close()
+        is_false(engine.pool._pool.empty())
+
+        conn = engine.connect()
+        eq_(Dialect.initialize.call_count, 4)
+        conn.close()
+        is_false(engine.pool._pool.empty())
+
+        engine.dispose()
+        conn = engine.connect()
+
+        eq_(Dialect.initialize.call_count, 4)
+        conn.close()
+        is_false(engine.pool._pool.empty())
+
     def test_invalidate_conn_w_contextmanager_interrupt(self):
         # test [ticket:3803]
         pool = self.db.pool
@@ -752,7 +818,6 @@ class RealReconnectTest(fixtures.TestBase):
 
         eq_(c1.execute(select([1])).scalar(), 1)
 
-        p1 = self.engine.pool
         self.engine.test_shutdown()
 
         _assert_invalidated(c1.execute, select([1]))
@@ -844,7 +909,7 @@ class RealReconnectTest(fixtures.TestBase):
     @testing.requires.savepoints
     def test_rollback_on_invalid_savepoint(self):
         conn = self.engine.connect()
-        trans = conn.begin()
+        conn.begin()
         trans2 = conn.begin_nested()
         conn.invalidate()
         trans2.rollback()
@@ -879,8 +944,6 @@ class RealReconnectTest(fixtures.TestBase):
 
         engine.dialect.initialize = broken_initialize
 
-        p1 = engine.pool
-
         def is_disconnect(e, conn, cursor):
             return True
 
@@ -950,33 +1013,30 @@ class RecycleTest(fixtures.TestBase):
     __backend__ = True
 
     def test_basic(self):
-        for threadlocal in False, True:
-            engine = engines.reconnecting_engine(
-                options={"pool_threadlocal": threadlocal}
-            )
+        engine = engines.reconnecting_engine()
 
-            conn = engine.contextual_connect()
-            eq_(conn.execute(select([1])).scalar(), 1)
-            conn.close()
+        conn = engine.connect()
+        eq_(conn.execute(select([1])).scalar(), 1)
+        conn.close()
 
-            # set the pool recycle down to 1.
-            # we aren't doing this inline with the
-            # engine create since cx_oracle takes way
-            # too long to create the 1st connection and don't
-            # want to build a huge delay into this test.
+        # set the pool recycle down to 1.
+        # we aren't doing this inline with the
+        # engine create since cx_oracle takes way
+        # too long to create the 1st connection and don't
+        # want to build a huge delay into this test.
 
-            engine.pool._recycle = 1
+        engine.pool._recycle = 1
 
-            # kill the DB connection
-            engine.test_shutdown()
+        # kill the DB connection
+        engine.test_shutdown()
 
-            # wait until past the recycle period
-            time.sleep(2)
+        # wait until past the recycle period
+        time.sleep(2)
 
-            # can connect, no exception
-            conn = engine.contextual_connect()
-            eq_(conn.execute(select([1])).scalar(), 1)
-            conn.close()
+        # can connect, no exception
+        conn = engine.connect()
+        eq_(conn.execute(select([1])).scalar(), 1)
+        conn.close()
 
 
 class PrePingRealTest(fixtures.TestBase):
