@@ -1,4 +1,5 @@
 import sqlalchemy as sa
+from sqlalchemy import CheckConstraint
 from sqlalchemy import event
 from sqlalchemy import exc
 from sqlalchemy import ForeignKey
@@ -9,17 +10,23 @@ from sqlalchemy import Integer
 from sqlalchemy import MetaData
 from sqlalchemy import String
 from sqlalchemy import testing
+from sqlalchemy import UniqueConstraint
 from sqlalchemy import util
 from sqlalchemy.ext import declarative as decl
 from sqlalchemy.ext.declarative import declared_attr
 from sqlalchemy.ext.declarative import synonym_for
+from sqlalchemy.ext.declarative.base import _DeferredMapperConfig
+from sqlalchemy.ext.hybrid import hybrid_property
 from sqlalchemy.orm import backref
 from sqlalchemy.orm import class_mapper
 from sqlalchemy.orm import clear_mappers
+from sqlalchemy.orm import close_all_sessions
 from sqlalchemy.orm import column_property
 from sqlalchemy.orm import composite
 from sqlalchemy.orm import configure_mappers
 from sqlalchemy.orm import create_session
+from sqlalchemy.orm import deferred
+from sqlalchemy.orm import exc as orm_exc
 from sqlalchemy.orm import joinedload
 from sqlalchemy.orm import mapper
 from sqlalchemy.orm import properties
@@ -38,6 +45,7 @@ from sqlalchemy.testing.schema import Column
 from sqlalchemy.testing.schema import Table
 from sqlalchemy.util import with_metaclass
 
+
 Base = None
 
 User = Address = None
@@ -55,7 +63,7 @@ class DeclarativeTestBase(
         Base = decl.declarative_base(testing.db)
 
     def teardown(self):
-        Session.close_all()
+        close_all_sessions()
         clear_mappers()
         Base.metadata.drop_all()
 
@@ -110,6 +118,37 @@ class DeclarativeTest(DeclarativeTestBase):
         eq_(a1, Address(email="two"))
         eq_(a1.user, User(name="u1"))
 
+    def test_deferred_reflection_default_error(self):
+        class MyExt(object):
+            @classmethod
+            def prepare(cls):
+                "sample prepare method"
+                to_map = _DeferredMapperConfig.classes_for_base(cls)
+                for thingy in to_map:
+                    thingy.map()
+
+            @classmethod
+            def _sa_decl_prepare(cls):
+                pass
+
+        class User(MyExt, Base):
+            __tablename__ = "user"
+            id = Column(Integer, primary_key=True)
+
+        assert_raises_message(
+            orm_exc.UnmappedClassError,
+            "Class test.ext.declarative.test_basic.User has a deferred "
+            "mapping on it.  It is not yet usable as a mapped class.",
+            Session().query,
+            User,
+        )
+
+        User.prepare()
+
+        self.assert_compile(
+            Session().query(User), 'SELECT "user".id AS user_id FROM "user"'
+        )
+
     def test_unicode_string_resolve(self):
         class User(Base, fixtures.ComparableEntity):
             __tablename__ = "users"
@@ -304,6 +343,54 @@ class DeclarativeTest(DeclarativeTestBase):
             go,
         )
 
+    def test_using_explicit_prop_in_schema_objects(self):
+        class Foo(Base):
+            __tablename__ = "foo"
+
+            id = Column(Integer, primary_key=True)
+            cprop = column_property(Column(Integer))
+
+            __table_args__ = (UniqueConstraint(cprop),)
+
+        uq = [
+            c
+            for c in Foo.__table__.constraints
+            if isinstance(c, UniqueConstraint)
+        ][0]
+        is_(uq.columns.cprop, Foo.__table__.c.cprop)
+
+        class Bar(Base):
+            __tablename__ = "bar"
+
+            id = Column(Integer, primary_key=True)
+            cprop = deferred(Column(Integer))
+
+            __table_args__ = (CheckConstraint(cprop > sa.func.foo()),)
+
+        ck = [
+            c
+            for c in Bar.__table__.constraints
+            if isinstance(c, CheckConstraint)
+        ][0]
+        is_(ck.columns.cprop, Bar.__table__.c.cprop)
+
+        if testing.requires.python3.enabled:
+            # test the existing failure case in case something changes
+            def go():
+                class Bat(Base):
+                    __tablename__ = "bat"
+
+                    id = Column(Integer, primary_key=True)
+                    cprop = deferred(Column(Integer))
+
+                    # we still can't do an expression like
+                    # "cprop > 5" because the column property isn't
+                    # a full blown column
+
+                    __table_args__ = (CheckConstraint(cprop > 5),)
+
+            assert_raises(TypeError, go)
+
     def test_relationship_level_msg_for_invalid_callable(self):
         class A(Base):
             __tablename__ = "a"
@@ -928,7 +1015,7 @@ class DeclarativeTest(DeclarativeTestBase):
                 sa.exc.InvalidRequestError,
                 "^One or more mappers failed to initialize"
                 " - can't proceed with initialization of other mappers. "
-                r"Triggering mapper: 'Mapper\|User\|users'. "
+                r"Triggering mapper: 'mapped class User->users'. "
                 "Original exception was: When initializing.*",
                 configure_mappers,
             )
@@ -1250,6 +1337,15 @@ class DeclarativeTest(DeclarativeTestBase):
 
         eq_(Foo.__mapper__.CHECK, True)
 
+    def test_no_change_to_all_descriptors(self):
+        base = decl.declarative_base()
+
+        class Foo(base):
+            __tablename__ = "foo"
+            id = Column(Integer, primary_key=True)
+
+        eq_(Foo.__mapper__.all_orm_descriptors.keys(), ["id"])
+
     def test_oops(self):
 
         with testing.expect_warnings(
@@ -1919,44 +2015,6 @@ class DeclarativeTest(DeclarativeTestBase):
         rt = sess.query(User).filter(User.namesyn == "someuser").one()
         eq_(rt, u1)
 
-    def test_comparable_using(self):
-        class NameComparator(sa.orm.PropComparator):
-            @property
-            def upperself(self):
-                cls = self.prop.parent.class_
-                col = getattr(cls, "name")
-                return sa.func.upper(col)
-
-            def operate(self, op, other, **kw):
-                return op(self.upperself, other, **kw)
-
-        class User(Base, fixtures.ComparableEntity):
-
-            __tablename__ = "users"
-            id = Column(
-                "id", Integer, primary_key=True, test_needs_autoincrement=True
-            )
-            name = Column("name", String(50))
-
-            @decl.comparable_using(NameComparator)
-            @property
-            def uc_name(self):
-                return self.name is not None and self.name.upper() or None
-
-        Base.metadata.create_all()
-        sess = create_session()
-        u1 = User(name="someuser")
-        eq_(u1.name, "someuser", u1.name)
-        eq_(u1.uc_name, "SOMEUSER", u1.uc_name)
-        sess.add(u1)
-        sess.flush()
-        sess.expunge_all()
-        rt = sess.query(User).filter(User.uc_name == "SOMEUSER").one()
-        eq_(rt, u1)
-        sess.expunge_all()
-        rt = sess.query(User).filter(User.uc_name.startswith("SOMEUSE")).one()
-        eq_(rt, u1)
-
     def test_duplicate_classes_in_base(self):
         class Test(Base):
             __tablename__ = "a"
@@ -1990,7 +2048,6 @@ class DeclarativeTest(DeclarativeTestBase):
             __tablename__ = "test"
             id = Column(Integer, primary_key=True)
 
-        # MARKMARK
         eq_(
             canary.mock_calls,
             [
@@ -2007,6 +2064,69 @@ class DeclarativeTest(DeclarativeTestBase):
 
         eq_(Base.__doc__, MyBase.__doc__)
 
+    def test_delattr_mapped_raises(self):
+        Base = decl.declarative_base()
+
+        class Foo(Base):
+            __tablename__ = "foo"
+
+            id = Column(Integer, primary_key=True)
+            data = Column(String)
+
+        def go():
+            del Foo.data
+
+        assert_raises_message(
+            NotImplementedError,
+            "Can't un-map individual mapped attributes on a mapped class.",
+            go,
+        )
+
+    def test_delattr_hybrid_fine(self):
+        Base = decl.declarative_base()
+
+        class Foo(Base):
+            __tablename__ = "foo"
+
+            id = Column(Integer, primary_key=True)
+            data = Column(String)
+
+            @hybrid_property
+            def data_hybrid(self):
+                return self.data
+
+        assert "data_hybrid" in Foo.__mapper__.all_orm_descriptors.keys()
+
+        del Foo.data_hybrid
+
+        assert "data_hybrid" not in Foo.__mapper__.all_orm_descriptors.keys()
+
+        assert not hasattr(Foo, "data_hybrid")
+
+    def test_setattr_hybrid_updates_descriptors(self):
+        Base = decl.declarative_base()
+
+        class Foo(Base):
+            __tablename__ = "foo"
+
+            id = Column(Integer, primary_key=True)
+            data = Column(String)
+
+        assert "data_hybrid" not in Foo.__mapper__.all_orm_descriptors.keys()
+
+        @hybrid_property
+        def data_hybrid(self):
+            return self.data
+
+        Foo.data_hybrid = data_hybrid
+        assert "data_hybrid" in Foo.__mapper__.all_orm_descriptors.keys()
+
+        del Foo.data_hybrid
+
+        assert "data_hybrid" not in Foo.__mapper__.all_orm_descriptors.keys()
+
+        assert not hasattr(Foo, "data_hybrid")
+
 
 def _produce_test(inline, stringbased):
     class ExplicitJoinTest(fixtures.MappedTest):
