@@ -2,7 +2,10 @@ import contextlib
 import itertools
 
 from sqlalchemy import bindparam
+from sqlalchemy import event
+from sqlalchemy import exc as sa_exc
 from sqlalchemy import func
+from sqlalchemy import literal_column
 from sqlalchemy import testing
 from sqlalchemy.ext import baked
 from sqlalchemy.orm import aliased
@@ -15,6 +18,7 @@ from sqlalchemy.orm import mapper
 from sqlalchemy.orm import relationship
 from sqlalchemy.orm import Session
 from sqlalchemy.orm import subqueryload
+from sqlalchemy.orm.query import Query
 from sqlalchemy.testing import assert_raises_message
 from sqlalchemy.testing import eq_
 from sqlalchemy.testing import is_
@@ -734,6 +738,63 @@ class ResultTest(BakedTest):
 
                 sess.close()
 
+    def test_to_query_query(self):
+        User = self.classes.User
+        Address = self.classes.Address
+
+        sub_bq = self.bakery(lambda s: s.query(User.name))
+        sub_bq += (
+            lambda q: q.filter(User.id == Address.user_id)
+            .filter(User.name == "ed")
+            .correlate(Address)
+        )
+
+        main_bq = self.bakery(lambda s: s.query(Address.id))
+        main_bq += lambda q: q.filter(sub_bq.to_query(q).exists())
+        main_bq += lambda q: q.order_by(Address.id)
+
+        sess = Session()
+        result = main_bq(sess).all()
+        eq_(result, [(2,), (3,), (4,)])
+
+    def test_to_query_session(self):
+        User = self.classes.User
+        Address = self.classes.Address
+
+        sub_bq = self.bakery(lambda s: s.query(User.name))
+        sub_bq += lambda q: q.filter(User.id == Address.user_id).correlate(
+            Address
+        )
+
+        main_bq = self.bakery(
+            lambda s: s.query(Address.id, sub_bq.to_query(s).as_scalar())
+        )
+        main_bq += lambda q: q.filter(sub_bq.to_query(q).as_scalar() == "ed")
+        main_bq += lambda q: q.order_by(Address.id)
+
+        sess = Session()
+        result = main_bq(sess).all()
+        eq_(result, [(2, "ed"), (3, "ed"), (4, "ed")])
+
+    def test_to_query_args(self):
+        User = self.classes.User
+        sub_bq = self.bakery(lambda s: s.query(User.name))
+
+        q = Query([], None)
+        assert_raises_message(
+            sa_exc.ArgumentError,
+            "Given Query needs to be associated with a Session",
+            sub_bq.to_query,
+            q,
+        )
+
+        assert_raises_message(
+            TypeError,
+            "Query or Session object expected, got .*'int'.*",
+            sub_bq.to_query,
+            5,
+        )
+
     def test_subquery_eagerloading(self):
         User = self.classes.User
         Address = self.classes.Address
@@ -858,10 +919,77 @@ class ResultTest(BakedTest):
 
         self.assert_sql_count(testing.db, go, 2)
 
+    @testing.fixture()
+    def before_compile_nobake_fixture(self):
+        @event.listens_for(Query, "before_compile", retval=True)
+        def _modify_query(query):
+            query = query.enable_assertions(False)
+            return query
+
+        yield
+        event.remove(Query, "before_compile", _modify_query)
+
+    def test_subqueryload_post_context_w_cancelling_event(
+        self, before_compile_nobake_fixture
+    ):
+        User = self.classes.User
+        Address = self.classes.Address
+
+        assert_result = [
+            User(
+                id=7, addresses=[Address(id=1, email_address="jack@bean.com")]
+            )
+        ]
+
+        self.bakery = baked.bakery(size=3)
+
+        bq = self.bakery(lambda s: s.query(User))
+
+        bq += lambda q: q.options(subqueryload(User.addresses))
+        bq += lambda q: q.order_by(User.id)
+        bq += lambda q: q.filter(User.name == bindparam("name"))
+        sess = Session()
+
+        def set_params(q):
+            return q.params(name="jack")
+
+        # test that the changes we make using with_post_criteria()
+        # are also applied to the subqueryload query.
+        def go():
+            result = bq(sess).with_post_criteria(set_params).all()
+            eq_(assert_result, result)
+
+        self.assert_sql_count(testing.db, go, 2)
+
 
 class LazyLoaderTest(testing.AssertsCompiledSQL, BakedTest):
     run_setup_mappers = "each"
 
+    @testing.fixture
+    def modify_query_fixture(self):
+        def set_event(bake_ok):
+
+            event.listen(
+                Query,
+                "before_compile",
+                _modify_query,
+                retval=True,
+                bake_ok=bake_ok,
+            )
+            return m1
+
+        m1 = mock.Mock()
+
+        def _modify_query(query):
+            m1(query.column_descriptions[0]["entity"])
+            query = query.enable_assertions(False).filter(
+                literal_column("1") == 1
+            )
+            return query
+
+        yield set_event
+        event.remove(Query, "before_compile", _modify_query)
+
     def _o2m_fixture(self, lazy="select", **kw):
         User = self.classes.User
         Address = self.classes.Address
@@ -918,6 +1046,45 @@ class LazyLoaderTest(testing.AssertsCompiledSQL, BakedTest):
         )
         return User, Address
 
+    def test_no_cache_for_event(self, modify_query_fixture):
+
+        m1 = modify_query_fixture(False)
+
+        User, Address = self._o2m_fixture()
+
+        sess = Session()
+        u1 = sess.query(User).filter(User.id == 7).first()
+
+        u1.addresses
+
+        eq_(m1.mock_calls, [mock.call(User), mock.call(Address)])
+
+        sess.expire(u1, ["addresses"])
+
+        u1.addresses
+        eq_(
+            m1.mock_calls,
+            [mock.call(User), mock.call(Address), mock.call(Address)],
+        )
+
+    def test_cache_ok_for_event(self, modify_query_fixture):
+
+        m1 = modify_query_fixture(True)
+
+        User, Address = self._o2m_fixture()
+
+        sess = Session()
+        u1 = sess.query(User).filter(User.id == 7).first()
+
+        u1.addresses
+
+        eq_(m1.mock_calls, [mock.call(User), mock.call(Address)])
+
+        sess.expire(u1, ["addresses"])
+
+        u1.addresses
+        eq_(m1.mock_calls, [mock.call(User), mock.call(Address)])
+
     def test_unsafe_unbound_option_cancels_bake(self):
         User, Address, Dingaling = self._o2m_twolevel_fixture(lazy="joined")
 
@@ -1266,6 +1433,21 @@ class LazyLoaderTest(testing.AssertsCompiledSQL, BakedTest):
             ),
         )
 
+    def test_simple_lazy_clause_no_race_on_generate(self):
+        User, Address = self._o2m_fixture()
+
+        expr1, paramdict1 = (
+            User.addresses.property._lazy_strategy._simple_lazy_clause
+        )
+
+        # delete the attr, as though a concurrent thread is also generating it
+        del User.addresses.property._lazy_strategy._simple_lazy_clause
+        expr2, paramdict2 = (
+            User.addresses.property._lazy_strategy._simple_lazy_clause
+        )
+
+        eq_(paramdict1, paramdict2)
+
     # additional tests:
     # 1. m2m w lazyload
     # 2. o2m lazyload where m2o backrefs have an eager load, test
