@@ -23,12 +23,14 @@ from sqlalchemy.orm import relationship
 from sqlalchemy.orm import selectinload
 from sqlalchemy.orm import Session
 from sqlalchemy.orm import sessionmaker
+from sqlalchemy.pool import SingletonThreadPool
 from sqlalchemy.sql import operators
 from sqlalchemy.testing import eq_
 from sqlalchemy.testing import fixtures
 from sqlalchemy.testing import provision
 from sqlalchemy.testing.engines import testing_engine
 
+
 # TODO: ShardTest can be turned into a base for further subclasses
 
 
@@ -49,8 +51,8 @@ class ShardTest(object):
         def id_generator(ctx):
             # in reality, might want to use a separate transaction for this.
 
-            c = db1.contextual_connect()
-            nextid = c.execute(ids.select(for_update=True)).scalar()
+            c = db1.connect()
+            nextid = c.execute(ids.select().with_for_update()).scalar()
             c.execute(ids.update(values={ids.c.nextid: ids.c.nextid + 1}))
             return nextid
 
@@ -317,7 +319,7 @@ class ShardTest(object):
         event.listen(WeatherLocation, "load", load)
         sess = self._fixture_data()
 
-        tokyo = (
+        tokyo = (  # noqa
             sess.query(WeatherLocation)
             .filter_by(city="Tokyo")
             .set_shard("asia")
@@ -365,12 +367,52 @@ class ShardTest(object):
         t = get_tokyo(sess2)
         eq_(t.city, tokyo.city)
 
+    def test_bulk_update(self):
+        sess = self._fixture_data()
+
+        eq_(
+            set(row.temperature for row in sess.query(Report.temperature)),
+            {80.0, 75.0, 85.0},
+        )
+
+        temps = sess.query(Report).all()
+        eq_(set(t.temperature for t in temps), {80.0, 75.0, 85.0})
+
+        sess.query(Report).filter(Report.temperature >= 80).update(
+            {"temperature": Report.temperature + 6}
+        )
+
+        eq_(
+            set(row.temperature for row in sess.query(Report.temperature)),
+            {86.0, 75.0, 91.0},
+        )
+
+        # test synchronize session as well
+        eq_(set(t.temperature for t in temps), {86.0, 75.0, 91.0})
+
+    def test_bulk_delete(self):
+        sess = self._fixture_data()
+
+        temps = sess.query(Report).all()
+        eq_(set(t.temperature for t in temps), {80.0, 75.0, 85.0})
+
+        sess.query(Report).filter(Report.temperature >= 80).delete()
+
+        eq_(
+            set(row.temperature for row in sess.query(Report.temperature)),
+            {75.0},
+        )
+
+        # test synchronize session as well
+        for t in temps:
+            assert inspect(t).deleted is (t.temperature >= 80)
+
 
 class DistinctEngineShardTest(ShardTest, fixtures.TestBase):
     def _init_dbs(self):
         db1 = testing_engine(
             "sqlite:///shard1_%s.db" % provision.FOLLOWER_IDENT,
-            options=dict(pool_threadlocal=True),
+            options=dict(poolclass=SingletonThreadPool),
         )
         db2 = testing_engine(
             "sqlite:///shard2_%s.db" % provision.FOLLOWER_IDENT
@@ -421,6 +463,9 @@ class AttachedFileShardTest(ShardTest, fixtures.TestBase):
 
 
 class SelectinloadRegressionTest(fixtures.DeclarativeMappedTest):
+    """test #4175
+    """
+
     @classmethod
     def setup_classes(cls):
         Base = cls.DeclarativeBasic
@@ -507,8 +552,7 @@ class RefreshDeferExpireTest(fixtures.DeclarativeMappedTest):
 class LazyLoadIdentityKeyTest(fixtures.DeclarativeMappedTest):
     def _init_dbs(self):
         self.db1 = db1 = testing_engine(
-            "sqlite:///shard1_%s.db" % provision.FOLLOWER_IDENT,
-            options=dict(pool_threadlocal=True),
+            "sqlite:///shard1_%s.db" % provision.FOLLOWER_IDENT
         )
         self.db2 = db2 = testing_engine(
             "sqlite:///shard2_%s.db" % provision.FOLLOWER_IDENT
