@@ -7,7 +7,6 @@ from sqlalchemy import exists
 from sqlalchemy import ForeignKey
 from sqlalchemy import func
 from sqlalchemy import Integer
-from sqlalchemy import MetaData
 from sqlalchemy import select
 from sqlalchemy import Sequence
 from sqlalchemy import String
@@ -15,31 +14,22 @@ from sqlalchemy import testing
 from sqlalchemy import Unicode
 from sqlalchemy import util
 from sqlalchemy.orm import class_mapper
-from sqlalchemy.orm import clear_mappers
+from sqlalchemy.orm import column_property
+from sqlalchemy.orm import contains_eager
 from sqlalchemy.orm import create_session
 from sqlalchemy.orm import join
 from sqlalchemy.orm import joinedload
-from sqlalchemy.orm import joinedload_all
 from sqlalchemy.orm import mapper
 from sqlalchemy.orm import polymorphic_union
-from sqlalchemy.orm import Query
 from sqlalchemy.orm import relationship
 from sqlalchemy.orm import Session
+from sqlalchemy.orm import with_polymorphic
 from sqlalchemy.orm.interfaces import MANYTOONE
 from sqlalchemy.testing import AssertsExecutionResults
 from sqlalchemy.testing import eq_
 from sqlalchemy.testing import fixtures
 from sqlalchemy.testing.schema import Column
 from sqlalchemy.testing.schema import Table
-from sqlalchemy.testing.util import function_named
-
-
-class AttrSettable(object):
-    def __init__(self, **kwargs):
-        [setattr(self, k, v) for k, v in kwargs.items()]
-
-    def __repr__(self):
-        return self.__class__.__name__ + "(%s)" % (hex(id(self)))
 
 
 class RelationshipTest1(fixtures.MappedTest):
@@ -82,17 +72,17 @@ class RelationshipTest1(fixtures.MappedTest):
             Column("manager_name", String(50)),
         )
 
-    def teardown(self):
-        people.update(values={people.c.manager_id: None}).execute()
-        super(RelationshipTest1, self).teardown()
-
-    def test_parent_refs_descendant(self):
-        class Person(AttrSettable):
+    @classmethod
+    def setup_classes(cls):
+        class Person(cls.Comparable):
             pass
 
         class Manager(Person):
             pass
 
+    def test_parent_refs_descendant(self):
+        Person, Manager = self.classes("Person", "Manager")
+
         mapper(
             Person,
             people,
@@ -130,11 +120,7 @@ class RelationshipTest1(fixtures.MappedTest):
         assert p.manager is m
 
     def test_descendant_refs_parent(self):
-        class Person(AttrSettable):
-            pass
-
-        class Manager(Person):
-            pass
+        Person, Manager = self.classes("Person", "Manager")
 
         mapper(Person, people)
         mapper(
@@ -210,31 +196,22 @@ class RelationshipTest2(fixtures.MappedTest):
             Column("data", String(30)),
         )
 
-    def test_relationshiponsubclass_j1_nodata(self):
-        self._do_test("join1", False)
-
-    def test_relationshiponsubclass_j2_nodata(self):
-        self._do_test("join2", False)
-
-    def test_relationshiponsubclass_j1_data(self):
-        self._do_test("join1", True)
-
-    def test_relationshiponsubclass_j2_data(self):
-        self._do_test("join2", True)
-
-    def test_relationshiponsubclass_j3_nodata(self):
-        self._do_test("join3", False)
-
-    def test_relationshiponsubclass_j3_data(self):
-        self._do_test("join3", True)
-
-    def _do_test(self, jointype="join1", usedata=False):
-        class Person(AttrSettable):
+    @classmethod
+    def setup_classes(cls):
+        class Person(cls.Comparable):
             pass
 
         class Manager(Person):
             pass
 
+    @testing.combinations(
+        ("join1",), ("join2",), ("join3",), argnames="jointype"
+    )
+    @testing.combinations(
+        ("usedata", True), ("nodata", False), id_="ia", argnames="usedata"
+    )
+    def test_relationshiponsubclass(self, jointype, usedata):
+        Person, Manager = self.classes("Person", "Manager")
         if jointype == "join1":
             poly_union = polymorphic_union(
                 {
@@ -376,21 +353,20 @@ class RelationshipTest3(fixtures.MappedTest):
             Column("data", String(30)),
         )
 
-
-def _generate_test(jointype="join1", usedata=False):
-    def _do_test(self):
-        class Person(AttrSettable):
+    @classmethod
+    def setup_classes(cls):
+        class Person(cls.Comparable):
             pass
 
         class Manager(Person):
             pass
 
-        if usedata:
-
-            class Data(object):
-                def __init__(self, data):
-                    self.data = data
+        class Data(cls.Comparable):
+            def __init__(self, data):
+                self.data = data
 
+    def _setup_mappings(self, jointype, usedata):
+        Person, Manager, Data = self.classes("Person", "Manager", "Data")
         if jointype == "join1":
             poly_union = polymorphic_union(
                 {
@@ -417,6 +393,8 @@ def _generate_test(jointype="join1", usedata=False):
             poly_union = people.outerjoin(managers)
         elif jointype == "join4":
             poly_union = None
+        else:
+            assert False
 
         if usedata:
             mapper(Data, data)
@@ -465,6 +443,16 @@ def _generate_test(jointype="join1", usedata=False):
             polymorphic_identity="manager",
         )
 
+    @testing.combinations(
+        ("join1",), ("join2",), ("join3",), ("join4",), argnames="jointype"
+    )
+    @testing.combinations(
+        ("usedata", True), ("nodata", False), id_="ia", argnames="usedata"
+    )
+    def test_relationship_on_base_class(self, jointype, usedata):
+        self._setup_mappings(jointype, usedata)
+        Person, Manager, Data = self.classes("Person", "Manager", "Data")
+
         sess = create_session()
         p = Person(name="person1")
         p2 = Person(name="person2")
@@ -492,20 +480,6 @@ def _generate_test(jointype="join1", usedata=False):
             assert p.data.data == "ps data"
             assert m.data.data == "ms data"
 
-    do_test = function_named(
-        _do_test,
-        "test_relationship_on_base_class_%s_%s"
-        % (jointype, data and "nodata" or "data"),
-    )
-    return do_test
-
-
-for jointype in ["join1", "join2", "join3", "join4"]:
-    for data in (True, False):
-        _fn = _generate_test(jointype, data)
-        setattr(RelationshipTest3, _fn.__name__, _fn)
-del func
-
 
 class RelationshipTest4(fixtures.MappedTest):
     @classmethod
@@ -612,21 +586,19 @@ class RelationshipTest4(fixtures.MappedTest):
             polymorphic_on=employee_join.c.type,
             polymorphic_identity="person",
         )
-        engineer_mapper = mapper(
+        mapper(
             Engineer,
             engineers,
             inherits=person_mapper,
             polymorphic_identity="engineer",
         )
-        manager_mapper = mapper(
+        mapper(
             Manager,
             managers,
             inherits=person_mapper,
             polymorphic_identity="manager",
         )
-        car_mapper = mapper(
-            Car, cars, properties={"employee": relationship(person_mapper)}
-        )
+        mapper(Car, cars, properties={"employee": relationship(person_mapper)})
 
         session = create_session()
 
@@ -779,7 +751,7 @@ class RelationshipTest5(fixtures.MappedTest):
             polymorphic_on=people.c.type,
             polymorphic_identity="person",
         )
-        engineer_mapper = mapper(
+        mapper(
             Engineer,
             engineers,
             inherits=person_mapper,
@@ -791,7 +763,7 @@ class RelationshipTest5(fixtures.MappedTest):
             inherits=person_mapper,
             polymorphic_identity="manager",
         )
-        car_mapper = mapper(
+        mapper(
             Car,
             cars,
             properties={
@@ -845,13 +817,17 @@ class RelationshipTest6(fixtures.MappedTest):
             Column("status", String(30)),
         )
 
-    def test_basic(self):
-        class Person(AttrSettable):
+    @classmethod
+    def setup_classes(cls):
+        class Person(cls.Comparable):
             pass
 
         class Manager(Person):
             pass
 
+    def test_basic(self):
+        Person, Manager = self.classes("Person", "Manager")
+
         mapper(Person, people)
 
         mapper(
@@ -1013,7 +989,7 @@ class RelationshipTest7(fixtures.MappedTest):
             polymorphic_on=car_join.c.type,
             polymorphic_identity="car",
         )
-        offroad_car_mapper = mapper(
+        mapper(
             Offraod_Car,
             offroad_cars,
             inherits=car_mapper,
@@ -1027,13 +1003,13 @@ class RelationshipTest7(fixtures.MappedTest):
             polymorphic_identity="person",
             properties={"car": relationship(car_mapper)},
         )
-        engineer_mapper = mapper(
+        mapper(
             Engineer,
             engineers,
             inherits=person_mapper,
             polymorphic_identity="engineer",
         )
-        manager_mapper = mapper(
+        mapper(
             Manager,
             managers,
             inherits=person_mapper,
@@ -1041,8 +1017,6 @@ class RelationshipTest7(fixtures.MappedTest):
         )
 
         session = create_session()
-        basic_car = Car(name="basic")
-        offroad_car = Offraod_Car(name="offroad")
 
         for i in range(1, 4):
             if i % 2:
@@ -1121,9 +1095,9 @@ class RelationshipTest8(fixtures.MappedTest):
         )
 
 
-class GenerativeTest(fixtures.TestBase, AssertsExecutionResults):
+class GenerativeTest(fixtures.MappedTest, AssertsExecutionResults):
     @classmethod
-    def setup_class(cls):
+    def define_tables(cls, metadata):
         #  cars---owned by---  people (abstract) --- has a --- status
         #   |                  ^    ^                            |
         #   |                  |    |                            |
@@ -1131,10 +1105,8 @@ class GenerativeTest(fixtures.TestBase, AssertsExecutionResults):
         #   |                                                    |
         #   +--------------------------------------- has a ------+
 
-        global metadata, status, people, engineers, managers, cars
-        metadata = MetaData(testing.db)
         # table definitions
-        status = Table(
+        Table(
             "status",
             metadata,
             Column(
@@ -1146,7 +1118,7 @@ class GenerativeTest(fixtures.TestBase, AssertsExecutionResults):
             Column("name", String(20)),
         )
 
-        people = Table(
+        Table(
             "people",
             metadata,
             Column(
@@ -1164,7 +1136,7 @@ class GenerativeTest(fixtures.TestBase, AssertsExecutionResults):
             Column("name", String(50)),
         )
 
-        engineers = Table(
+        Table(
             "engineers",
             metadata,
             Column(
@@ -1176,7 +1148,7 @@ class GenerativeTest(fixtures.TestBase, AssertsExecutionResults):
             Column("field", String(30)),
         )
 
-        managers = Table(
+        Table(
             "managers",
             metadata,
             Column(
@@ -1188,7 +1160,7 @@ class GenerativeTest(fixtures.TestBase, AssertsExecutionResults):
             Column("category", String(70)),
         )
 
-        cars = Table(
+        Table(
             "cars",
             metadata,
             Column(
@@ -1211,52 +1183,31 @@ class GenerativeTest(fixtures.TestBase, AssertsExecutionResults):
             ),
         )
 
-        metadata.create_all()
-
     @classmethod
-    def teardown_class(cls):
-        metadata.drop_all()
+    def setup_classes(cls):
+        class Status(cls.Comparable):
+            pass
 
-    def teardown(self):
-        clear_mappers()
-        for t in reversed(metadata.sorted_tables):
-            t.delete().execute()
-
-    def test_join_to(self):
-        # class definitions
-        class PersistentObject(object):
-            def __init__(self, **kwargs):
-                for key, value in kwargs.items():
-                    setattr(self, key, value)
-
-        class Status(PersistentObject):
-            def __repr__(self):
-                return "Status %s" % self.name
-
-        class Person(PersistentObject):
-            def __repr__(self):
-                return "Ordinary person %s" % self.name
+        class Person(cls.Comparable):
+            pass
 
         class Engineer(Person):
-            def __repr__(self):
-                return "Engineer %s, field %s, status %s" % (
-                    self.name,
-                    self.field,
-                    self.status,
-                )
+            pass
 
         class Manager(Person):
-            def __repr__(self):
-                return "Manager %s, category %s, status %s" % (
-                    self.name,
-                    self.category,
-                    self.status,
-                )
+            pass
 
-        class Car(PersistentObject):
-            def __repr__(self):
-                return "Car number %d" % self.car_id
+        class Car(cls.Comparable):
+            pass
 
+    @classmethod
+    def setup_mappers(cls):
+        status, people, engineers, managers, cars = cls.tables(
+            "status", "people", "engineers", "managers", "cars"
+        )
+        Status, Person, Engineer, Manager, Car = cls.classes(
+            "Status", "Person", "Engineer", "Manager", "Car"
+        )
         # create a union that represents both types of joins.
         employee_join = polymorphic_union(
             {
@@ -1276,19 +1227,19 @@ class GenerativeTest(fixtures.TestBase, AssertsExecutionResults):
             polymorphic_identity="person",
             properties={"status": relationship(status_mapper)},
         )
-        engineer_mapper = mapper(
+        mapper(
             Engineer,
             engineers,
             inherits=person_mapper,
             polymorphic_identity="engineer",
         )
-        manager_mapper = mapper(
+        mapper(
             Manager,
             managers,
             inherits=person_mapper,
             polymorphic_identity="manager",
         )
-        car_mapper = mapper(
+        mapper(
             Car,
             cars,
             properties={
@@ -1297,6 +1248,11 @@ class GenerativeTest(fixtures.TestBase, AssertsExecutionResults):
             },
         )
 
+    @classmethod
+    def insert_data(cls):
+        Status, Person, Engineer, Manager, Car = cls.classes(
+            "Status", "Person", "Engineer", "Manager", "Car"
+        )
         session = create_session()
 
         active = Status(name="active")
@@ -1325,7 +1281,7 @@ class GenerativeTest(fixtures.TestBase, AssertsExecutionResults):
         session.flush()
 
         # get E4
-        engineer4 = session.query(engineer_mapper).filter_by(name="E4").one()
+        engineer4 = session.query(Engineer).filter_by(name="E4").one()
 
         # create 2 cars for E4, one active and one dead
         car1 = Car(employee=engineer4, status=active)
@@ -1334,10 +1290,11 @@ class GenerativeTest(fixtures.TestBase, AssertsExecutionResults):
         session.add(car2)
         session.flush()
 
-        # this particular adapt used to cause a recursion overflow;
-        # added here for testing
-        e = exists([Car.owner], Car.owner == employee_join.c.person_id)
-        Query(Person)._adapt_clause(employee_join, False, False)
+    def test_join_to_q_person(self):
+        Status, Person, Engineer, Manager, Car = self.classes(
+            "Status", "Person", "Engineer", "Manager", "Car"
+        )
+        session = create_session()
 
         r = (
             session.query(Person)
@@ -1347,31 +1304,52 @@ class GenerativeTest(fixtures.TestBase, AssertsExecutionResults):
             .order_by(Person.person_id)
         )
         eq_(
-            str(list(r)),
-            "[Manager M2, category YYYYYYYYY, status "
-            "Status active, Engineer E2, field X, "
-            "status Status active]",
+            list(r),
+            [
+                Manager(
+                    name="M2",
+                    category="YYYYYYYYY",
+                    status=Status(name="active"),
+                ),
+                Engineer(name="E2", field="X", status=Status(name="active")),
+            ],
         )
+
+    def test_join_to_q_engineer(self):
+        Status, Person, Engineer, Manager, Car = self.classes(
+            "Status", "Person", "Engineer", "Manager", "Car"
+        )
+        session = create_session()
         r = (
             session.query(Engineer)
             .join("status")
             .filter(
                 Person.name.in_(["E2", "E3", "E4", "M4", "M2", "M1"])
-                & (status.c.name == "active")
+                & (Status.name == "active")
             )
             .order_by(Person.name)
         )
         eq_(
-            str(list(r)),
-            "[Engineer E2, field X, status Status "
-            "active, Engineer E3, field X, status "
-            "Status active]",
+            list(r),
+            [
+                Engineer(name="E2", field="X", status=Status(name="active")),
+                Engineer(name="E3", field="X", status=Status(name="active")),
+            ],
         )
 
+    def test_join_to_q_person_car(self):
+        Status, Person, Engineer, Manager, Car = self.classes(
+            "Status", "Person", "Engineer", "Manager", "Car"
+        )
+        session = create_session()
         r = session.query(Person).filter(
             exists([1], Car.owner == Person.person_id)
         )
-        eq_(str(list(r)), "[Engineer E4, field X, status Status dead]")
+
+        eq_(
+            list(r),
+            [Engineer(name="E4", field="X", status=Status(name="dead"))],
+        )
 
 
 class MultiLevelTest(fixtures.MappedTest):
@@ -1473,7 +1451,7 @@ class MultiLevelTest(fixtures.MappedTest):
             with_polymorphic=("*", pu_Engineer),
         )
 
-        mapper_Manager = mapper(
+        mapper(
             Manager,
             table_Manager,
             inherit_condition=table_Manager.c.id == table_Engineer.c.id,
@@ -1790,7 +1768,7 @@ class InheritingEagerTest(fixtures.MappedTest):
 class MissingPolymorphicOnTest(fixtures.MappedTest):
     @classmethod
     def define_tables(cls, metadata):
-        tablea = Table(
+        Table(
             "tablea",
             metadata,
             Column(
@@ -1798,7 +1776,7 @@ class MissingPolymorphicOnTest(fixtures.MappedTest):
             ),
             Column("adata", String(50)),
         )
-        tableb = Table(
+        Table(
             "tableb",
             metadata,
             Column(
@@ -1807,13 +1785,13 @@ class MissingPolymorphicOnTest(fixtures.MappedTest):
             Column("aid", Integer, ForeignKey("tablea.id")),
             Column("data", String(50)),
         )
-        tablec = Table(
+        Table(
             "tablec",
             metadata,
             Column("id", Integer, ForeignKey("tablea.id"), primary_key=True),
             Column("cdata", String(50)),
         )
-        tabled = Table(
+        Table(
             "tabled",
             metadata,
             Column("id", Integer, ForeignKey("tablec.id"), primary_key=True),
@@ -1911,7 +1889,7 @@ class JoinedInhAdjacencyTest(fixtures.MappedTest):
             pass
 
     def _roundtrip(self):
-        Person, User = self.classes.Person, self.classes.User
+        User = self.classes.User
         sess = Session()
         u1 = User()
         u2 = User()
@@ -2082,13 +2060,7 @@ class Ticket2419Test(fixtures.DeclarativeMappedTest):
         "end though",
     )
     def test_join_w_eager_w_any(self):
-        A, B, C, D, E = (
-            self.classes.A,
-            self.classes.B,
-            self.classes.C,
-            self.classes.D,
-            self.classes.E,
-        )
+        B, C, D = (self.classes.B, self.classes.C, self.classes.D)
         s = Session(testing.db)
 
         b = B(ds=[D()])
@@ -2096,7 +2068,7 @@ class Ticket2419Test(fixtures.DeclarativeMappedTest):
 
         s.commit()
 
-        q = s.query(B, B.ds.any(D.id == 1)).options(joinedload_all("es"))
+        q = s.query(B, B.ds.any(D.id == 1)).options(joinedload("es"))
         q = q.join(C, C.b_id == B.id)
         q = q.limit(5)
         eq_(q.all(), [(b, True)])
@@ -2140,3 +2112,111 @@ class ColSubclassTest(
             "(a AS a_1 JOIN b AS b_1 ON a_1.id = b_1.id) "
             "ON a.id = b_1.id WHERE b_1.x = :x_1",
         )
+
+
+class CorrelateExceptWPolyAdaptTest(
+    fixtures.DeclarativeMappedTest, testing.AssertsCompiledSQL
+):
+    # test [ticket:4537]'s test case.
+
+    run_create_tables = run_deletes = None
+    run_setup_classes = run_setup_mappers = run_define_tables = "each"
+    __dialect__ = "default"
+
+    def _fixture(self, use_correlate_except):
+
+        Base = self.DeclarativeBasic
+
+        class Superclass(Base):
+            __tablename__ = "s1"
+            id = Column(Integer, primary_key=True)
+            common_id = Column(ForeignKey("c.id"))
+            common_relationship = relationship(
+                "Common", uselist=False, innerjoin=True, lazy="noload"
+            )
+            discriminator_field = Column(String)
+            __mapper_args__ = {
+                "polymorphic_identity": "superclass",
+                "polymorphic_on": discriminator_field,
+            }
+
+        class Subclass(Superclass):
+            __tablename__ = "s2"
+            id = Column(ForeignKey("s1.id"), primary_key=True)
+            __mapper_args__ = {"polymorphic_identity": "subclass"}
+
+        class Common(Base):
+            __tablename__ = "c"
+            id = Column(Integer, primary_key=True)
+
+            if use_correlate_except:
+                num_superclass = column_property(
+                    select([func.count(Superclass.id)])
+                    .where(Superclass.common_id == id)
+                    .correlate_except(Superclass)
+                )
+
+        if not use_correlate_except:
+            Common.num_superclass = column_property(
+                select([func.count(Superclass.id)])
+                .where(Superclass.common_id == Common.id)
+                .correlate(Common)
+            )
+
+        return Common, Superclass
+
+    def test_poly_query_on_correlate(self):
+        Common, Superclass = self._fixture(False)
+
+        poly = with_polymorphic(Superclass, "*")
+
+        s = Session()
+        q = (
+            s.query(poly)
+            .options(contains_eager(poly.common_relationship))
+            .join(poly.common_relationship)
+            .filter(Common.id == 1)
+        )
+
+        # note the order of c.id, subquery changes based on if we
+        # used correlate or correlate_except; this is only with the
+        # patch in place.   Not sure why this happens.
+        self.assert_compile(
+            q,
+            "SELECT c.id AS c_id, (SELECT count(s1.id) AS count_1 "
+            "FROM s1 LEFT OUTER JOIN s2 ON s1.id = s2.id "
+            "WHERE s1.common_id = c.id) AS anon_1, "
+            "s1.id AS s1_id, "
+            "s1.common_id AS s1_common_id, "
+            "s1.discriminator_field AS s1_discriminator_field, "
+            "s2.id AS s2_id FROM s1 "
+            "LEFT OUTER JOIN s2 ON s1.id = s2.id "
+            "JOIN c ON c.id = s1.common_id WHERE c.id = :id_1",
+        )
+
+    def test_poly_query_on_correlate_except(self):
+        Common, Superclass = self._fixture(True)
+
+        poly = with_polymorphic(Superclass, "*")
+
+        s = Session()
+        q = (
+            s.query(poly)
+            .options(contains_eager(poly.common_relationship))
+            .join(poly.common_relationship)
+            .filter(Common.id == 1)
+        )
+
+        # c.id, subquery are reversed.
+        self.assert_compile(
+            q,
+            "SELECT (SELECT count(s1.id) AS count_1 "
+            "FROM s1 LEFT OUTER JOIN s2 ON s1.id = s2.id "
+            "WHERE s1.common_id = c.id) AS anon_1, "
+            "c.id AS c_id, s1.id AS s1_id, "
+            "s1.common_id AS s1_common_id, "
+            "s1.discriminator_field AS s1_discriminator_field, "
+            "s2.id AS s2_id FROM s1 "
+            "LEFT OUTER JOIN s2 ON s1.id = s2.id "
+            "JOIN c ON c.id = s1.common_id WHERE c.id = :id_1",
+        )
