@@ -9,12 +9,10 @@ from sqlalchemy.orm import backref
 from sqlalchemy.orm import contains_eager
 from sqlalchemy.orm import create_session
 from sqlalchemy.orm import joinedload
-from sqlalchemy.orm import joinedload_all
 from sqlalchemy.orm import mapper
 from sqlalchemy.orm import relationship
 from sqlalchemy.orm import Session
 from sqlalchemy.orm import subqueryload
-from sqlalchemy.orm import subqueryload_all
 from sqlalchemy.orm import with_polymorphic
 from sqlalchemy.testing import AssertsCompiledSQL
 from sqlalchemy.testing import eq_
@@ -166,7 +164,7 @@ class SelfReferentialJ2JTest(fixtures.MappedTest):
 
     @classmethod
     def define_tables(cls, metadata):
-        people = Table(
+        Table(
             "people",
             metadata,
             Column(
@@ -179,7 +177,7 @@ class SelfReferentialJ2JTest(fixtures.MappedTest):
             Column("type", String(30)),
         )
 
-        engineers = Table(
+        Table(
             "engineers",
             metadata,
             Column(
@@ -192,7 +190,7 @@ class SelfReferentialJ2JTest(fixtures.MappedTest):
             Column("reports_to_id", Integer, ForeignKey("managers.person_id")),
         )
 
-        managers = Table(
+        Table(
             "managers",
             metadata,
             Column(
@@ -345,7 +343,7 @@ class SelfReferentialJ2JSelfTest(fixtures.MappedTest):
 
     @classmethod
     def define_tables(cls, metadata):
-        people = Table(
+        Table(
             "people",
             metadata,
             Column(
@@ -358,7 +356,7 @@ class SelfReferentialJ2JSelfTest(fixtures.MappedTest):
             Column("type", String(30)),
         )
 
-        engineers = Table(
+        Table(
             "engineers",
             metadata,
             Column(
@@ -496,7 +494,7 @@ class M2MFilterTest(fixtures.MappedTest):
 
     @classmethod
     def define_tables(cls, metadata):
-        organizations = Table(
+        Table(
             "organizations",
             metadata,
             Column(
@@ -505,14 +503,14 @@ class M2MFilterTest(fixtures.MappedTest):
             Column("name", String(50)),
         )
 
-        engineers_to_org = Table(
+        Table(
             "engineers_to_org",
             metadata,
             Column("org_id", Integer, ForeignKey("organizations.id")),
             Column("engineer_id", Integer, ForeignKey("engineers.person_id")),
         )
 
-        people = Table(
+        Table(
             "people",
             metadata,
             Column(
@@ -525,7 +523,7 @@ class M2MFilterTest(fixtures.MappedTest):
             Column("type", String(30)),
         )
 
-        engineers = Table(
+        Table(
             "engineers",
             metadata,
             Column(
@@ -956,7 +954,9 @@ class EagerToSubclassTest(fixtures.MappedTest):
         def go():
             eq_(
                 sess.query(Parent)
-                .options(subqueryload_all(Parent.children, Base.related))
+                .options(
+                    subqueryload(Parent.children).subqueryload(Base.related)
+                )
                 .order_by(Parent.data)
                 .all(),
                 [p1, p2],
@@ -973,7 +973,7 @@ class EagerToSubclassTest(fixtures.MappedTest):
         def go():
             eq_(
                 sess.query(pa)
-                .options(subqueryload_all(pa.children, Base.related))
+                .options(subqueryload(pa.children).subqueryload(Base.related))
                 .order_by(pa.data)
                 .all(),
                 [p1, p2],
@@ -1909,7 +1909,7 @@ class JoinedloadOverWPolyAliased(
 
         session = Session()
         q = session.query(cls).options(
-            joinedload_all(cls.links, Link.child, cls.links)
+            joinedload(cls.links).joinedload(Link.child).joinedload(cls.links)
         )
         if cls is self.classes.Sub1:
             extra = " WHERE parent.type IN (:type_1)"
@@ -1938,7 +1938,7 @@ class JoinedloadOverWPolyAliased(
 
         session = Session()
         q = session.query(Link).options(
-            joinedload_all(Link.child, parent_cls.owner)
+            joinedload(Link.child).joinedload(parent_cls.owner)
         )
 
         if Link.child.property.mapper.class_ is self.classes.Sub1:
@@ -1971,7 +1971,7 @@ class JoinedloadOverWPolyAliased(
         q = session.query(poly).options(
             joinedload(poly.Sub1.links)
             .joinedload(Link.child.of_type(Sub1))
-            .joinedload(poly.Sub1.links)
+            .joinedload(Sub1.links)
         )
         self.assert_compile(
             q,
@@ -1999,7 +1999,7 @@ class JoinedloadOverWPolyAliased(
         q = session.query(poly).options(
             joinedload(poly.Sub1.links, innerjoin=True)
             .joinedload(Link.child.of_type(Sub1), innerjoin=True)
-            .joinedload(poly.Sub1.links, innerjoin=True)
+            .joinedload(Sub1.links, innerjoin=True)
         )
         self.assert_compile(
             q,
@@ -2159,11 +2159,10 @@ class JoinAcrossJoinedInhMultiPath(
         )
 
     def test_joinedload(self):
-        Root, Intermediate, Sub1, Target = (
+        Root, Intermediate, Sub1 = (
             self.classes.Root,
             self.classes.Intermediate,
             self.classes.Sub1,
-            self.classes.Target,
         )
 
         sess = Session()
@@ -2257,12 +2256,7 @@ class MultipleAdaptUsesEntityOverTableTest(
         mapper(D, d, inherits=A)
 
     def _two_join_fixture(self):
-        A, B, C, D = (
-            self.classes.A,
-            self.classes.B,
-            self.classes.C,
-            self.classes.D,
-        )
+        B, C, D = (self.classes.B, self.classes.C, self.classes.D)
         s = Session()
         return (
             s.query(B.name, C.name, D.name)
@@ -2272,7 +2266,7 @@ class MultipleAdaptUsesEntityOverTableTest(
         )
 
     def test_two_joins_adaption(self):
-        a, b, c, d = self.tables.a, self.tables.b, self.tables.c, self.tables.d
+        a, c, d = self.tables.a, self.tables.c, self.tables.d
         q = self._two_join_fixture()
 
         btoc = q._from_obj[0].left
