@@ -10,6 +10,7 @@ from sqlalchemy.orm import mapper
 from sqlalchemy.orm import relationship
 from sqlalchemy.orm import Session
 from sqlalchemy.testing import assert_raises_message
+from sqlalchemy.testing import eq_
 from sqlalchemy.testing import fixtures
 from sqlalchemy.testing.schema import Column
 from sqlalchemy.testing.schema import Table
@@ -150,9 +151,9 @@ class ParentRemovalTest(fixtures.MappedTest):
             a1,
         )
 
-        # unfortunately, u1.addresses was impacted
-        # here
-        assert u1.addresses == []
+        # u1.addresses wasn't actually impacted, because the event was
+        # caught before collection mutation
+        eq_(u1.addresses, [a1])
 
         # expire all and we can continue
         s.expire_all()
@@ -171,7 +172,7 @@ class ParentRemovalTest(fixtures.MappedTest):
         User = self.classes.User
         s, u1, a1 = self._fixture()
 
-        u2 = User(addresses=[a1])
+        u2 = User(addresses=[a1])  # noqa
 
         s.expire(a1)
         u1.addresses.remove(a1)
@@ -208,13 +209,11 @@ class ParentRemovalTest(fixtures.MappedTest):
         self._assert_hasparent(a1)
 
     def test_fresh_state_positive(self):
-        User = self.classes.User
         s, u1, a1 = self._fixture()
 
         self._assert_hasparent(a1)
 
     def test_fresh_state_negative(self):
-        User = self.classes.User
         s, u1, a1 = self._fixture()
 
         u1.addresses.remove(a1)
