@@ -10,9 +10,11 @@ from sqlalchemy import ForeignKey
 from sqlalchemy import ForeignKeyConstraint
 from sqlalchemy import Integer
 from sqlalchemy import orm
+from sqlalchemy import select
 from sqlalchemy import SmallInteger
 from sqlalchemy import String
 from sqlalchemy import testing
+from sqlalchemy.orm import aliased
 from sqlalchemy.orm import attributes
 from sqlalchemy.orm import configure_mappers
 from sqlalchemy.orm import create_session
@@ -530,28 +532,107 @@ class LazyTest(_fixtures.FixtureTest):
             list(q),
         )
 
-    def test_double(self):
-        """tests lazy loading with two relationships simultaneously,
-        from the same table, using aliases.  """
+    def test_double_w_ac_against_subquery(self):
 
-        users, orders, User, Address, Order, addresses = (
+        (
+            users,
+            orders,
+            User,
+            Address,
+            Order,
+            addresses,
+            Item,
+            items,
+            order_items,
+        ) = (
             self.tables.users,
             self.tables.orders,
             self.classes.User,
             self.classes.Address,
             self.classes.Order,
             self.tables.addresses,
+            self.classes.Item,
+            self.tables.items,
+            self.tables.order_items,
         )
 
-        openorders = sa.alias(orders, "openorders")
-        closedorders = sa.alias(orders, "closedorders")
-
         mapper(Address, addresses)
 
-        mapper(Order, orders)
+        mapper(
+            Order,
+            orders,
+            properties={
+                "items": relationship(
+                    Item,
+                    secondary=order_items,
+                    lazy="select",
+                    order_by=items.c.id,
+                )
+            },
+        )
+        mapper(Item, items)
 
-        open_mapper = mapper(Order, openorders, non_primary=True)
-        closed_mapper = mapper(Order, closedorders, non_primary=True)
+        open_mapper = aliased(
+            Order, select([orders]).where(orders.c.isopen == 1).alias()
+        )
+        closed_mapper = aliased(
+            Order, select([orders]).where(orders.c.isopen == 0).alias()
+        )
+
+        mapper(
+            User,
+            users,
+            properties=dict(
+                addresses=relationship(Address, lazy=True),
+                open_orders=relationship(open_mapper, lazy="select"),
+                closed_orders=relationship(closed_mapper, lazy="select"),
+            ),
+        )
+
+        self._run_double_test()
+
+    def test_double_w_ac(self):
+
+        (
+            users,
+            orders,
+            User,
+            Address,
+            Order,
+            addresses,
+            Item,
+            items,
+            order_items,
+        ) = (
+            self.tables.users,
+            self.tables.orders,
+            self.classes.User,
+            self.classes.Address,
+            self.classes.Order,
+            self.tables.addresses,
+            self.classes.Item,
+            self.tables.items,
+            self.tables.order_items,
+        )
+
+        mapper(Address, addresses)
+
+        mapper(
+            Order,
+            orders,
+            properties={
+                "items": relationship(
+                    Item,
+                    secondary=order_items,
+                    lazy="select",
+                    order_by=items.c.id,
+                )
+            },
+        )
+        mapper(Item, items)
+
+        open_mapper = aliased(Order, orders)
+        closed_mapper = aliased(Order, orders)
         mapper(
             User,
             users,
@@ -560,47 +641,79 @@ class LazyTest(_fixtures.FixtureTest):
                 open_orders=relationship(
                     open_mapper,
                     primaryjoin=sa.and_(
-                        openorders.c.isopen == 1,
-                        users.c.id == openorders.c.user_id,
+                        open_mapper.isopen == 1,
+                        users.c.id == open_mapper.user_id,
                     ),
                     lazy="select",
                 ),
                 closed_orders=relationship(
                     closed_mapper,
                     primaryjoin=sa.and_(
-                        closedorders.c.isopen == 0,
-                        users.c.id == closedorders.c.user_id,
+                        closed_mapper.isopen == 0,
+                        users.c.id == closed_mapper.user_id,
                     ),
                     lazy="select",
                 ),
             ),
         )
-        q = create_session().query(User)
 
-        assert [
-            User(
-                id=7,
-                addresses=[Address(id=1)],
-                open_orders=[Order(id=3)],
-                closed_orders=[Order(id=1), Order(id=5)],
-            ),
-            User(
-                id=8,
-                addresses=[Address(id=2), Address(id=3), Address(id=4)],
-                open_orders=[],
-                closed_orders=[],
-            ),
-            User(
-                id=9,
-                addresses=[Address(id=5)],
-                open_orders=[Order(id=4)],
-                closed_orders=[Order(id=2)],
-            ),
-            User(id=10),
-        ] == q.all()
+        self._run_double_test()
+
+    def _run_double_test(self, no_items=False):
+        User, Address, Order, Item = self.classes(
+            "User", "Address", "Order", "Item"
+        )
+        q = create_session().query(User).order_by(User.id)
+
+        def items(*ids):
+            if no_items:
+                return {}
+            else:
+                return {"items": [Item(id=id_) for id_ in ids]}
+
+        def go():
+            eq_(
+                [
+                    User(
+                        id=7,
+                        addresses=[Address(id=1)],
+                        open_orders=[Order(id=3, **items(3, 4, 5))],
+                        closed_orders=[
+                            Order(id=1, **items(1, 2, 3)),
+                            Order(id=5, **items(5)),
+                        ],
+                    ),
+                    User(
+                        id=8,
+                        addresses=[
+                            Address(id=2),
+                            Address(id=3),
+                            Address(id=4),
+                        ],
+                        open_orders=[],
+                        closed_orders=[],
+                    ),
+                    User(
+                        id=9,
+                        addresses=[Address(id=5)],
+                        open_orders=[Order(id=4, **items(1, 5))],
+                        closed_orders=[Order(id=2, **items(1, 2, 3))],
+                    ),
+                    User(id=10),
+                ],
+                q.all(),
+            )
+
+        if no_items:
+            self.assert_sql_count(testing.db, go, 10)
+        else:
+            self.assert_sql_count(testing.db, go, 15)
 
         sess = create_session()
         user = sess.query(User).get(7)
+
+        closed_mapper = User.closed_orders.entity
+        open_mapper = User.open_orders.entity
         eq_(
             [Order(id=1), Order(id=5)],
             create_session()
