@@ -12,14 +12,12 @@ from sqlalchemy import testing
 from sqlalchemy import Text
 from sqlalchemy.orm import attributes
 from sqlalchemy.orm import backref
-from sqlalchemy.orm import comparable_property
 from sqlalchemy.orm import configure_mappers
 from sqlalchemy.orm import create_session
 from sqlalchemy.orm import defer
 from sqlalchemy.orm import deferred
 from sqlalchemy.orm import foreign
 from sqlalchemy.orm import mapper
-from sqlalchemy.orm import PropComparator
 from sqlalchemy.orm import relationship
 from sqlalchemy.orm import Session
 from sqlalchemy.orm import sessionmaker
@@ -28,6 +26,7 @@ from sqlalchemy.orm.collections import attribute_mapped_collection
 from sqlalchemy.orm.interfaces import MapperOption
 from sqlalchemy.testing import assert_raises_message
 from sqlalchemy.testing import eq_
+from sqlalchemy.testing import expect_warnings
 from sqlalchemy.testing import fixtures
 from sqlalchemy.testing import in_
 from sqlalchemy.testing import not_in_
@@ -86,6 +85,36 @@ class MergeTest(_fixtures.FixtureTest):
 
         self.assert_sql_count(testing.db, go, 0)
 
+    def test_warn_transient_already_pending_nopk(self):
+        User, users = self.classes.User, self.tables.users
+
+        mapper(User, users)
+        sess = create_session()
+        u = User(name="fred")
+
+        sess.add(u)
+
+        with expect_warnings(
+            "Instance <User.*> is already pending in this Session yet is "
+            "being merged again; this is probably not what you want to do"
+        ):
+            sess.merge(u)
+
+    def test_warn_transient_already_pending_pk(self):
+        User, users = self.classes.User, self.tables.users
+
+        mapper(User, users)
+        sess = create_session()
+        u = User(id=1, name="fred")
+
+        sess.add(u)
+
+        with expect_warnings(
+            "Instance <User.*> is already pending in this Session yet is "
+            "being merged again; this is probably not what you want to do"
+        ):
+            sess.merge(u)
+
     def test_transient_to_pending_collection(self):
         User, Address, addresses, users = (
             self.classes.User,
@@ -481,7 +510,7 @@ class MergeTest(_fixtures.FixtureTest):
         # attribute maintains modified state.
         # (usually autoflush would have happened
         # here anyway).
-        u4 = sess.merge(User(id=2))
+        u4 = sess.merge(User(id=2))  # noqa
         eq_(u3.__dict__["data"], "bar")
 
         sess.flush()
@@ -509,7 +538,7 @@ class MergeTest(_fixtures.FixtureTest):
         # set it to None.  this is actually
         # a change so gets preserved.
         u6.data = None
-        u7 = sess.merge(User(id=3))
+        u7 = sess.merge(User(id=3))  # noqa
         assert u6.__dict__["data"] is None
 
     def test_merge_irregular_collection(self):
@@ -1103,7 +1132,7 @@ class MergeTest(_fixtures.FixtureTest):
         u2 = sess2.query(User).options(sa.orm.joinedload("addresses")).get(7)
 
         sess3 = create_session()
-        u3 = sess3.merge(u2, load=False)
+        u3 = sess3.merge(u2, load=False)  # noqa
 
         def go():
             sess3.flush()
@@ -1142,7 +1171,7 @@ class MergeTest(_fixtures.FixtureTest):
         u2 = sess2.query(User).get(7)
 
         sess3 = create_session()
-        u3 = sess3.merge(u2, load=False)
+        u3 = sess3.merge(u2, load=False)  # noqa
         assert not sess3.dirty
 
         def go():
@@ -1283,13 +1312,10 @@ class MergeTest(_fixtures.FixtureTest):
         except sa.exc.InvalidRequestError as e:
             assert "load=False option does not support" in str(e)
 
-    def test_synonym_comparable(self):
+    def test_synonym(self):
         users = self.tables.users
 
         class User(object):
-            class Comparator(PropComparator):
-                pass
-
             def _getValue(self):
                 return self._value
 
@@ -1298,14 +1324,7 @@ class MergeTest(_fixtures.FixtureTest):
 
             value = property(_getValue, _setValue)
 
-        mapper(
-            User,
-            users,
-            properties={
-                "uid": synonym("id"),
-                "foobar": comparable_property(User.Comparator, User.value),
-            },
-        )
+        mapper(User, users, properties={"uid": synonym("id")})
 
         sess = create_session()
         u = User()
@@ -1355,7 +1374,7 @@ class MergeTest(_fixtures.FixtureTest):
         )
 
         sess = create_session(autoflush=True, autocommit=False)
-        m = mapper(
+        mapper(
             User,
             users,
             properties={
@@ -1670,7 +1689,7 @@ class M2ONoUseGetLoadingTest(fixtures.MappedTest):
 
         def go():
             u1 = User(id=1, addresses=[Address(id=1), Address(id=2)])
-            u2 = s.merge(u1)
+            s.merge(u1)
 
         self.assert_sql_count(testing.db, go, 2)
 
@@ -1935,13 +1954,13 @@ class LoadOnPendingTest(fixtures.MappedTest):
 
     @classmethod
     def define_tables(cls, metadata):
-        rocks_table = Table(
+        Table(
             "rocks",
             metadata,
             Column("id", Integer, primary_key=True),
             Column("description", String(10)),
         )
-        bugs_table = Table(
+        Table(
             "bugs",
             metadata,
             Column("id", Integer, primary_key=True),
