@@ -3,18 +3,24 @@ Primary key changing capabilities and passive/non-passive cascading updates.
 
 """
 
+import itertools
+
 import sqlalchemy as sa
+from sqlalchemy import bindparam
 from sqlalchemy import ForeignKey
 from sqlalchemy import Integer
 from sqlalchemy import String
 from sqlalchemy import testing
+from sqlalchemy import TypeDecorator
 from sqlalchemy.orm import create_session
 from sqlalchemy.orm import mapper
 from sqlalchemy.orm import relationship
 from sqlalchemy.orm import Session
 from sqlalchemy.orm.session import make_transient
 from sqlalchemy.testing import assert_raises
+from sqlalchemy.testing import assert_raises_message
 from sqlalchemy.testing import eq_
+from sqlalchemy.testing import expect_warnings
 from sqlalchemy.testing import fixtures
 from sqlalchemy.testing import ne_
 from sqlalchemy.testing.schema import Column
@@ -35,7 +41,7 @@ def _backend_specific_fk_args():
 class NaturalPKTest(fixtures.MappedTest):
     # MySQL 5.5 on Windows crashes (the entire server, not the client)
     # if you screw around with ON UPDATE CASCADE type of stuff.
-    __requires__ = "skip_mysql_on_windows", "on_update_or_deferrable_fks"
+    __requires__ = ("skip_mysql_on_windows",)
     __backend__ = True
 
     @classmethod
@@ -281,6 +287,13 @@ class NaturalPKTest(fixtures.MappedTest):
     def test_manytoone_nonpassive(self):
         self._test_manytoone(False)
 
+    @testing.requires.on_update_cascade
+    def test_manytoone_passive_uselist(self):
+        self._test_manytoone(True, True)
+
+    def test_manytoone_nonpassive_uselist(self):
+        self._test_manytoone(False, True)
+
     def test_manytoone_nonpassive_cold_mapping(self):
         """test that the mapper-level m2o dependency processor
         is set up even if the opposite side relationship
@@ -316,7 +329,7 @@ class NaturalPKTest(fixtures.MappedTest):
 
         self.assert_sql_count(testing.db, go, 2)
 
-    def _test_manytoone(self, passive_updates):
+    def _test_manytoone(self, passive_updates, uselist=False, dynamic=False):
         users, Address, addresses, User = (
             self.tables.users,
             self.classes.Address,
@@ -329,19 +342,27 @@ class NaturalPKTest(fixtures.MappedTest):
             Address,
             addresses,
             properties={
-                "user": relationship(User, passive_updates=passive_updates)
+                "user": relationship(
+                    User, uselist=uselist, passive_updates=passive_updates
+                )
             },
         )
 
         sess = create_session()
         a1 = Address(email="jack1")
         a2 = Address(email="jack2")
+        a3 = Address(email="fred")
 
         u1 = User(username="jack", fullname="jack")
-        a1.user = u1
-        a2.user = u1
+        if uselist:
+            a1.user = [u1]
+            a2.user = [u1]
+        else:
+            a1.user = u1
+            a2.user = u1
         sess.add(a1)
         sess.add(a2)
+        sess.add(a3)
         sess.flush()
 
         u1.username = "ed"
@@ -361,10 +382,24 @@ class NaturalPKTest(fixtures.MappedTest):
 
         assert a1.username == a2.username == "ed"
         sess.expunge_all()
-        eq_(
-            [Address(username="ed"), Address(username="ed")],
-            sess.query(Address).all(),
-        )
+        if uselist:
+            eq_(
+                [
+                    Address(email="fred", user=[]),
+                    Address(username="ed"),
+                    Address(username="ed"),
+                ],
+                sess.query(Address).order_by(Address.email).all(),
+            )
+        else:
+            eq_(
+                [
+                    Address(email="fred", user=None),
+                    Address(username="ed"),
+                    Address(username="ed"),
+                ],
+                sess.query(Address).order_by(Address.email).all(),
+            )
 
     @testing.requires.on_update_cascade
     def test_onetoone_passive(self):
@@ -501,7 +536,6 @@ class NaturalPKTest(fixtures.MappedTest):
         self._test_manytomany(True)
 
     @testing.requires.non_updating_cascade
-    @testing.requires.sane_multi_rowcount.not_()
     def test_manytomany_nonpassive(self):
         self._test_manytomany(False)
 
@@ -575,6 +609,124 @@ class NaturalPKTest(fixtures.MappedTest):
         r = sess.query(Item).with_parent(u2).all()
         eq_(Item(itemname="item2"), r[0])
 
+    def test_manytoone_deferred_relationship_expr(self):
+        """for [ticket:4359], test that updates to the columns embedded
+        in an object expression are also updated."""
+        users, Address, addresses, User = (
+            self.tables.users,
+            self.classes.Address,
+            self.tables.addresses,
+            self.classes.User,
+        )
+
+        mapper(User, users)
+        mapper(
+            Address,
+            addresses,
+            properties={
+                "user": relationship(
+                    User,
+                    passive_updates=testing.requires.on_update_cascade.enabled,
+                )
+            },
+        )
+
+        s = Session()
+        a1 = Address(email="jack1")
+        u1 = User(username="jack", fullname="jack")
+
+        a1.user = u1
+
+        # scenario 1.  object is still transient, we get a value.
+        expr = Address.user == u1
+
+        eq_(expr.left.callable(), "jack")
+
+        # scenario 2.  value has been changed while we are transient.
+        # we get the updated value.
+        u1.username = "ed"
+        eq_(expr.left.callable(), "ed")
+
+        s.add_all([u1, a1])
+        s.commit()
+
+        eq_(a1.username, "ed")
+
+        # scenario 3.  the value is changed and flushed, we get the new value.
+        u1.username = "fred"
+        s.flush()
+
+        eq_(expr.left.callable(), "fred")
+
+        # scenario 4.  the value is changed, flushed, and expired.
+        # the callable goes out to get that value.
+        u1.username = "wendy"
+        s.commit()
+        assert "username" not in u1.__dict__
+
+        eq_(expr.left.callable(), "wendy")
+
+        # scenario 5.  the value is changed flushed, expired,
+        # and then when we hit the callable, we are detached.
+        u1.username = "jack"
+        s.commit()
+        assert "username" not in u1.__dict__
+
+        s.expunge(u1)
+
+        # InstanceState has a "last known values" feature we use
+        # to pick up on this
+        eq_(expr.left.callable(), "jack")
+
+        # doesn't unexpire the attribute
+        assert "username" not in u1.__dict__
+
+        # once we are persistent again, we check the DB
+        s.add(u1)
+        eq_(expr.left.callable(), "jack")
+        assert "username" in u1.__dict__
+
+        # scenario 6.  we are using del
+        u2 = User(username="jack", fullname="jack")
+        expr = Address.user == u2
+
+        eq_(expr.left.callable(), "jack")
+
+        del u2.username
+
+        assert_raises_message(
+            sa.exc.InvalidRequestError,
+            "Can't resolve value for column users.username",
+            expr.left.callable,
+        )
+
+        u2.username = "ed"
+        eq_(expr.left.callable(), "ed")
+
+        s.add(u2)
+        s.commit()
+
+        eq_(expr.left.callable(), "ed")
+
+        del u2.username
+
+        # object is persistent, so since we deleted, we get None
+        with expect_warnings("Got None for value of column "):
+            eq_(expr.left.callable(), None)
+
+        s.expunge(u2)
+
+        # however that None isn't in the dict, that's just the default
+        # attribute value, so after expunge it's gone
+        assert "username" not in u2.__dict__
+
+        # detached, we don't have it
+        assert_raises_message(
+            sa.exc.InvalidRequestError,
+            "Can't resolve value for column users.username",
+            expr.left.callable,
+        )
+
 
 class TransientExceptionTesst(_fixtures.FixtureTest):
     run_inserts = None
@@ -660,6 +812,9 @@ class ReversePKsTest(fixtures.MappedTest):
         session.add(a_editable)
         session.commit()
 
+        # see also much more recent issue #4890 where we add a warning
+        # for almost this same case
+
         # do the switch in both directions -
         # one or the other should raise the error
         # based on platform dictionary ordering
@@ -1125,7 +1280,15 @@ class CascadeToFKPKTest(fixtures.MappedTest, testing.AssertsCompiledSQL):
     def test_change_m2o_nonpassive(self):
         self._test_change_m2o(False)
 
-    def _test_change_m2o(self, passive_updates):
+    @testing.requires.on_update_cascade
+    def test_change_m2o_passive_uselist(self):
+        self._test_change_m2o(True, True)
+
+    @testing.requires.non_updating_cascade
+    def test_change_m2o_nonpassive_uselist(self):
+        self._test_change_m2o(False, True)
+
+    def _test_change_m2o(self, passive_updates, uselist=False):
         User, Address, users, addresses = (
             self.classes.User,
             self.classes.Address,
@@ -1138,13 +1301,18 @@ class CascadeToFKPKTest(fixtures.MappedTest, testing.AssertsCompiledSQL):
             Address,
             addresses,
             properties={
-                "user": relationship(User, passive_updates=passive_updates)
+                "user": relationship(
+                    User, uselist=uselist, passive_updates=passive_updates
+                )
             },
         )
 
         sess = create_session()
         u1 = User(username="jack")
-        a1 = Address(user=u1, email="foo@bar")
+        if uselist:
+            a1 = Address(user=[u1], email="foo@bar")
+        else:
+            a1 = Address(user=u1, email="foo@bar")
         sess.add_all([u1, a1])
         sess.flush()
 
@@ -1343,6 +1511,19 @@ class JoinedInheritanceTest(fixtures.MappedTest):
             test_needs_fk=True,
         )
 
+        Table(
+            "owner",
+            metadata,
+            Column(
+                "name",
+                String(50),
+                ForeignKey("manager.name", **fk_args),
+                primary_key=True,
+            ),
+            Column("owner_name", String(50)),
+            test_needs_fk=True,
+        )
+
     @classmethod
     def setup_classes(cls):
         class Person(cls.Comparable):
@@ -1354,6 +1535,48 @@ class JoinedInheritanceTest(fixtures.MappedTest):
         class Manager(Person):
             pass
 
+        class Owner(Manager):
+            pass
+
+    def _mapping_fixture(self, threelevel, passive_updates):
+        Person, Manager, Engineer, Owner = self.classes(
+            "Person", "Manager", "Engineer", "Owner"
+        )
+        person, manager, engineer, owner = self.tables(
+            "person", "manager", "engineer", "owner"
+        )
+
+        mapper(
+            Person,
+            person,
+            polymorphic_on=person.c.type,
+            polymorphic_identity="person",
+            passive_updates=passive_updates,
+        )
+
+        mapper(
+            Engineer,
+            engineer,
+            inherits=Person,
+            polymorphic_identity="engineer",
+            properties={
+                "boss": relationship(
+                    Manager,
+                    primaryjoin=manager.c.name == engineer.c.boss_name,
+                    passive_updates=passive_updates,
+                )
+            },
+        )
+
+        mapper(
+            Manager, manager, inherits=Person, polymorphic_identity="manager"
+        )
+
+        if threelevel:
+            mapper(
+                Owner, owner, inherits=Manager, polymorphic_identity="owner"
+            )
+
     @testing.requires.on_update_cascade
     def test_pk_passive(self):
         self._test_pk(True)
@@ -1371,40 +1594,26 @@ class JoinedInheritanceTest(fixtures.MappedTest):
     def test_fk_nonpassive(self):
         self._test_fk(False)
 
+    @testing.requires.on_update_cascade
+    def test_pk_threelevel_passive(self):
+        self._test_pk_threelevel(True)
+
+    @testing.requires.non_updating_cascade
+    def test_pk_threelevel_nonpassive(self):
+        self._test_pk_threelevel(False)
+
+    @testing.requires.on_update_cascade
+    def test_fk_threelevel_passive(self):
+        self._test_fk_threelevel(True)
+
+    # PG etc. need passive=True to allow PK->PK cascade
+    @testing.requires.non_updating_cascade
+    def test_fk_threelevel_nonpassive(self):
+        self._test_fk_threelevel(False)
+
     def _test_pk(self, passive_updates):
-        Person, Manager, person, manager, Engineer, engineer = (
-            self.classes.Person,
-            self.classes.Manager,
-            self.tables.person,
-            self.tables.manager,
-            self.classes.Engineer,
-            self.tables.engineer,
-        )
-
-        mapper(
-            Person,
-            person,
-            polymorphic_on=person.c.type,
-            polymorphic_identity="person",
-            passive_updates=passive_updates,
-        )
-        mapper(
-            Engineer,
-            engineer,
-            inherits=Person,
-            polymorphic_identity="engineer",
-            properties={
-                "boss": relationship(
-                    Manager,
-                    primaryjoin=manager.c.name == engineer.c.boss_name,
-                    passive_updates=passive_updates,
-                )
-            },
-        )
-        mapper(
-            Manager, manager, inherits=Person, polymorphic_identity="manager"
-        )
-
+        Engineer, = self.classes("Engineer")
+        self._mapping_fixture(False, passive_updates)
         sess = sa.orm.sessionmaker()()
 
         e1 = Engineer(name="dilbert", primary_language="java")
@@ -1412,41 +1621,28 @@ class JoinedInheritanceTest(fixtures.MappedTest):
         sess.commit()
         e1.name = "wally"
         e1.primary_language = "c++"
+
         sess.commit()
+        eq_(
+            sess.execute(self.tables.engineer.select()).fetchall(),
+            [("wally", "c++", None)],
+        )
+
+        eq_(e1.name, "wally")
+
+        e1.name = "dogbert"
+        sess.commit()
+        eq_(e1.name, "dogbert")
+
+        eq_(
+            sess.execute(self.tables.engineer.select()).fetchall(),
+            [("dogbert", "c++", None)],
+        )
 
     def _test_fk(self, passive_updates):
-        Person, Manager, person, manager, Engineer, engineer = (
-            self.classes.Person,
-            self.classes.Manager,
-            self.tables.person,
-            self.tables.manager,
-            self.classes.Engineer,
-            self.tables.engineer,
-        )
+        Manager, Engineer = self.classes("Manager", "Engineer")
 
-        mapper(
-            Person,
-            person,
-            polymorphic_on=person.c.type,
-            polymorphic_identity="person",
-            passive_updates=passive_updates,
-        )
-        mapper(
-            Engineer,
-            engineer,
-            inherits=Person,
-            polymorphic_identity="engineer",
-            properties={
-                "boss": relationship(
-                    Manager,
-                    primaryjoin=manager.c.name == engineer.c.boss_name,
-                    passive_updates=passive_updates,
-                )
-            },
-        )
-        mapper(
-            Manager, manager, inherits=Person, polymorphic_identity="manager"
-        )
+        self._mapping_fixture(False, passive_updates)
 
         sess = sa.orm.sessionmaker()()
 
@@ -1460,6 +1656,14 @@ class JoinedInheritanceTest(fixtures.MappedTest):
 
         eq_(e1.boss_name, "dogbert")
         eq_(e2.boss_name, "dogbert")
+
+        eq_(
+            sess.execute(
+                self.tables.engineer.select().order_by(Engineer.name)
+            ).fetchall(),
+            [("dilbert", "java", "dogbert"), ("wally", "c++", "dogbert")],
+        )
+
         sess.expire_all()
 
         m1.name = "pointy haired"
@@ -1470,6 +1674,159 @@ class JoinedInheritanceTest(fixtures.MappedTest):
         eq_(e1.boss_name, "pointy haired")
         eq_(e2.boss_name, "pointy haired")
 
+        eq_(
+            sess.execute(
+                self.tables.engineer.select().order_by(Engineer.name)
+            ).fetchall(),
+            [
+                ("dilbert", "scala", "pointy haired"),
+                ("wally", "cobol", "pointy haired"),
+            ],
+        )
+
+    def _test_pk_threelevel(self, passive_updates):
+        Owner, = self.classes("Owner")
+
+        self._mapping_fixture(True, passive_updates)
+
+        sess = sa.orm.sessionmaker()()
+
+        o1 = Owner(name="dogbert", owner_name="dog")
+        sess.add(o1)
+        sess.commit()
+        o1.name = "pointy haired"
+        o1.owner_name = "pointy"
+        sess.commit()
+
+        eq_(
+            sess.execute(self.tables.manager.select()).fetchall(),
+            [("pointy haired", None)],
+        )
+        eq_(
+            sess.execute(self.tables.owner.select()).fetchall(),
+            [("pointy haired", "pointy")],
+        )
+
+        eq_(o1.name, "pointy haired")
+
+        o1.name = "catbert"
+        sess.commit()
+
+        eq_(o1.name, "catbert")
+
+        eq_(
+            sess.execute(self.tables.manager.select()).fetchall(),
+            [("catbert", None)],
+        )
+        eq_(
+            sess.execute(self.tables.owner.select()).fetchall(),
+            [("catbert", "pointy")],
+        )
+
+    def _test_fk_threelevel(self, passive_updates):
+        Owner, Engineer = self.classes("Owner", "Engineer")
+        self._mapping_fixture(True, passive_updates)
+
+        sess = sa.orm.sessionmaker()()
+
+        m1 = Owner(name="dogbert", paperwork="lots", owner_name="dog")
+        e1, e2 = (
+            Engineer(name="dilbert", primary_language="java", boss=m1),
+            Engineer(name="wally", primary_language="c++", boss=m1),
+        )
+        sess.add_all([e1, e2, m1])
+        sess.commit()
+
+        eq_(e1.boss_name, "dogbert")
+        eq_(e2.boss_name, "dogbert")
+        sess.expire_all()
+
+        m1.name = "pointy haired"
+
+        e1.primary_language = "scala"
+        e2.primary_language = "cobol"
+        sess.commit()
+
+        eq_(e1.boss_name, "pointy haired")
+        eq_(e2.boss_name, "pointy haired")
+
+        eq_(
+            sess.execute(self.tables.manager.select()).fetchall(),
+            [("pointy haired", "lots")],
+        )
+        eq_(
+            sess.execute(self.tables.owner.select()).fetchall(),
+            [("pointy haired", "dog")],
+        )
+
+
+class UnsortablePKTest(fixtures.MappedTest):
+    """Test integration with TypeEngine.sort_key_function"""
+
+    class HashableDict(dict):
+        def __hash__(self):
+            return hash((self["x"], self["y"]))
+
+    @classmethod
+    def define_tables(cls, metadata):
+        class MyUnsortable(TypeDecorator):
+            impl = String(10)
+
+            def process_bind_param(self, value, dialect):
+                return "%s,%s" % (value["x"], value["y"])
+
+            def process_result_value(self, value, dialect):
+                rec = value.split(",")
+                return cls.HashableDict({"x": rec[0], "y": rec[1]})
+
+            def sort_key_function(self, value):
+                return (value["x"], value["y"])
+
+        Table(
+            "data",
+            metadata,
+            Column("info", MyUnsortable(), primary_key=True),
+            Column("int_value", Integer),
+        )
+
+    @classmethod
+    def setup_classes(cls):
+        class Data(cls.Comparable):
+            pass
+
+    @classmethod
+    def setup_mappers(cls):
+        mapper(cls.classes.Data, cls.tables.data)
+
+    def test_updates_sorted(self):
+        Data = self.classes.Data
+        s = Session()
+
+        s.add_all(
+            [
+                Data(info=self.HashableDict(x="a", y="b")),
+                Data(info=self.HashableDict(x="a", y="a")),
+                Data(info=self.HashableDict(x="b", y="b")),
+                Data(info=self.HashableDict(x="b", y="a")),
+            ]
+        )
+        s.commit()
+
+        aa, ab, ba, bb = s.query(Data).order_by(Data.info).all()
+
+        counter = itertools.count()
+        ab.int_value = bindparam(key=None, callable_=lambda: next(counter))
+        ba.int_value = bindparam(key=None, callable_=lambda: next(counter))
+        bb.int_value = bindparam(key=None, callable_=lambda: next(counter))
+        aa.int_value = bindparam(key=None, callable_=lambda: next(counter))
+
+        s.commit()
+
+        eq_(
+            s.query(Data.int_value).order_by(Data.info).all(),
+            [(0,), (1,), (2,), (3,)],
+        )
+
 
 class JoinedInheritancePKOnFKTest(fixtures.MappedTest):
     """Test cascades of pk->non-pk/fk on joined table inh."""
