@@ -8,11 +8,9 @@ from sqlalchemy.engine import default
 from sqlalchemy.orm import aliased
 from sqlalchemy.orm import contains_eager
 from sqlalchemy.orm import joinedload
-from sqlalchemy.orm import joinedload_all
 from sqlalchemy.orm import relationship
 from sqlalchemy.orm import Session
 from sqlalchemy.orm import subqueryload
-from sqlalchemy.orm import subqueryload_all
 from sqlalchemy.orm import with_polymorphic
 from sqlalchemy.testing import assert_raises_message
 from sqlalchemy.testing import eq_
@@ -464,8 +462,7 @@ class SubclassRelationshipTest(
         return [p.container for p in cls._fixture()]
 
     def test_contains_eager_wpoly(self):
-        ParentThing, DataContainer, Job, SubJob = (
-            self.classes.ParentThing,
+        DataContainer, Job, SubJob = (
             self.classes.DataContainer,
             self.classes.Job,
             self.classes.SubJob,
@@ -486,8 +483,7 @@ class SubclassRelationshipTest(
         self.assert_sql_count(testing.db, go, 5)
 
     def test_joinedload_wpoly(self):
-        ParentThing, DataContainer, Job, SubJob = (
-            self.classes.ParentThing,
+        DataContainer, Job, SubJob = (
             self.classes.DataContainer,
             self.classes.Job,
             self.classes.SubJob,
@@ -506,10 +502,8 @@ class SubclassRelationshipTest(
         self.assert_sql_count(testing.db, go, 5)
 
     def test_joinedload_wsubclass(self):
-        ParentThing, DataContainer, Job, SubJob = (
-            self.classes.ParentThing,
+        DataContainer, SubJob = (
             self.classes.DataContainer,
-            self.classes.Job,
             self.classes.SubJob,
         )
         s = Session(testing.db)
@@ -537,10 +531,8 @@ class SubclassRelationshipTest(
         self.assert_sql_count(testing.db, go, 11)
 
     def test_subquery_wsubclass(self):
-        ParentThing, DataContainer, Job, SubJob = (
-            self.classes.ParentThing,
+        DataContainer, SubJob = (
             self.classes.DataContainer,
-            self.classes.Job,
             self.classes.SubJob,
         )
         s = Session(testing.db)
@@ -554,16 +546,15 @@ class SubclassRelationshipTest(
         self.assert_sql_count(testing.db, go, 6)
 
     def test_twolevel_subqueryload_wsubclass(self):
-        ParentThing, DataContainer, Job, SubJob = (
+        ParentThing, DataContainer, SubJob = (
             self.classes.ParentThing,
             self.classes.DataContainer,
-            self.classes.Job,
             self.classes.SubJob,
         )
         s = Session(testing.db)
         q = s.query(ParentThing).options(
-            subqueryload_all(
-                ParentThing.container, DataContainer.jobs.of_type(SubJob)
+            subqueryload(ParentThing.container).subqueryload(
+                DataContainer.jobs.of_type(SubJob)
             )
         )
 
@@ -577,8 +568,8 @@ class SubclassRelationshipTest(
         s = Session(testing.db)
         sj_alias = aliased(SubJob)
         q = s.query(DataContainer).options(
-            subqueryload_all(
-                DataContainer.jobs.of_type(sj_alias), sj_alias.widget
+            subqueryload(DataContainer.jobs.of_type(sj_alias)).subqueryload(
+                sj_alias.widget
             )
         )
 
@@ -588,16 +579,15 @@ class SubclassRelationshipTest(
         self.assert_sql_count(testing.db, go, 3)
 
     def test_twolevel_joinedload_wsubclass(self):
-        ParentThing, DataContainer, Job, SubJob = (
+        ParentThing, DataContainer, SubJob = (
             self.classes.ParentThing,
             self.classes.DataContainer,
-            self.classes.Job,
             self.classes.SubJob,
         )
         s = Session(testing.db)
         q = s.query(ParentThing).options(
-            joinedload_all(
-                ParentThing.container, DataContainer.jobs.of_type(SubJob)
+            joinedload(ParentThing.container).joinedload(
+                DataContainer.jobs.of_type(SubJob)
             )
         )
 
@@ -607,8 +597,7 @@ class SubclassRelationshipTest(
         self.assert_sql_count(testing.db, go, 5)
 
     def test_any_wpoly(self):
-        ParentThing, DataContainer, Job, SubJob = (
-            self.classes.ParentThing,
+        DataContainer, Job, SubJob = (
             self.classes.DataContainer,
             self.classes.Job,
             self.classes.SubJob,
@@ -639,12 +628,7 @@ class SubclassRelationshipTest(
         )
 
     def test_any_walias(self):
-        ParentThing, DataContainer, Job, SubJob = (
-            self.classes.ParentThing,
-            self.classes.DataContainer,
-            self.classes.Job,
-            self.classes.SubJob,
-        )
+        DataContainer, Job, = (self.classes.DataContainer, self.classes.Job)
 
         Job_A = aliased(Job)
 
@@ -672,8 +656,7 @@ class SubclassRelationshipTest(
         )
 
     def test_join_wpoly(self):
-        ParentThing, DataContainer, Job, SubJob = (
-            self.classes.ParentThing,
+        DataContainer, Job, SubJob = (
             self.classes.DataContainer,
             self.classes.Job,
             self.classes.SubJob,
@@ -694,10 +677,8 @@ class SubclassRelationshipTest(
         )
 
     def test_join_wsubclass(self):
-        ParentThing, DataContainer, Job, SubJob = (
-            self.classes.ParentThing,
+        DataContainer, SubJob = (
             self.classes.DataContainer,
-            self.classes.Job,
             self.classes.SubJob,
         )
 
@@ -717,8 +698,7 @@ class SubclassRelationshipTest(
         )
 
     def test_join_wpoly_innerjoin(self):
-        ParentThing, DataContainer, Job, SubJob = (
-            self.classes.ParentThing,
+        DataContainer, Job, SubJob = (
             self.classes.DataContainer,
             self.classes.Job,
             self.classes.SubJob,
@@ -738,12 +718,7 @@ class SubclassRelationshipTest(
         )
 
     def test_join_walias(self):
-        ParentThing, DataContainer, Job, SubJob = (
-            self.classes.ParentThing,
-            self.classes.DataContainer,
-            self.classes.Job,
-            self.classes.SubJob,
-        )
+        DataContainer, Job, = (self.classes.DataContainer, self.classes.Job)
 
         Job_A = aliased(Job)
 
@@ -758,8 +733,7 @@ class SubclassRelationshipTest(
         )
 
     def test_join_explicit_wpoly_noalias(self):
-        ParentThing, DataContainer, Job, SubJob = (
-            self.classes.ParentThing,
+        DataContainer, Job, SubJob = (
             self.classes.DataContainer,
             self.classes.Job,
             self.classes.SubJob,
@@ -780,8 +754,7 @@ class SubclassRelationshipTest(
         )
 
     def test_join_explicit_wpoly_flat(self):
-        ParentThing, DataContainer, Job, SubJob = (
-            self.classes.ParentThing,
+        DataContainer, Job, SubJob = (
             self.classes.DataContainer,
             self.classes.Job,
             self.classes.SubJob,
@@ -802,8 +775,7 @@ class SubclassRelationshipTest(
         )
 
     def test_join_explicit_wpoly_full_alias(self):
-        ParentThing, DataContainer, Job, SubJob = (
-            self.classes.ParentThing,
+        DataContainer, Job, SubJob = (
             self.classes.DataContainer,
             self.classes.Job,
             self.classes.SubJob,
