@@ -10,20 +10,28 @@ from sqlalchemy.orm import class_mapper
 from sqlalchemy.orm import column_property
 from sqlalchemy.orm import create_session
 from sqlalchemy.orm import defaultload
+from sqlalchemy.orm import defer
+from sqlalchemy.orm import exc as orm_exc
 from sqlalchemy.orm import joinedload
-from sqlalchemy.orm import joinedload_all
 from sqlalchemy.orm import lazyload
 from sqlalchemy.orm import Load
+from sqlalchemy.orm import load_only
 from sqlalchemy.orm import mapper
 from sqlalchemy.orm import relationship
 from sqlalchemy.orm import Session
 from sqlalchemy.orm import strategy_options
 from sqlalchemy.orm import subqueryload
 from sqlalchemy.orm import util as orm_util
+from sqlalchemy.orm import with_polymorphic
 from sqlalchemy.testing import fixtures
 from sqlalchemy.testing.assertions import assert_raises_message
 from sqlalchemy.testing.assertions import eq_
 from test.orm import _fixtures
+from .inheritance._poly_fixtures import _Polymorphic
+from .inheritance._poly_fixtures import Company
+from .inheritance._poly_fixtures import Engineer
+from .inheritance._poly_fixtures import Manager
+from .inheritance._poly_fixtures import Person
 
 
 class QueryTest(_fixtures.FixtureTest):
@@ -108,7 +116,10 @@ class LoadTest(PathTest, QueryTest):
         result = Load(User)
         eq_(
             result._generate_path(
-                inspect(User)._path_registry, User.addresses, "relationship"
+                inspect(User)._path_registry,
+                User.addresses,
+                None,
+                "relationship",
             ),
             self._make_path_registry([User, "addresses", Address]),
         )
@@ -119,7 +130,7 @@ class LoadTest(PathTest, QueryTest):
         result = Load(User)
         eq_(
             result._generate_path(
-                inspect(User)._path_registry, User.name, "column"
+                inspect(User)._path_registry, User.name, None, "column"
             ),
             self._make_path_registry([User, "name"]),
         )
@@ -131,7 +142,7 @@ class LoadTest(PathTest, QueryTest):
         result = Load(User)
         eq_(
             result._generate_path(
-                inspect(User)._path_registry, "addresses", "relationship"
+                inspect(User)._path_registry, "addresses", None, "relationship"
             ),
             self._make_path_registry([User, "addresses", Address]),
         )
@@ -142,7 +153,7 @@ class LoadTest(PathTest, QueryTest):
         result = Load(User)
         eq_(
             result._generate_path(
-                inspect(User)._path_registry, "name", "column"
+                inspect(User)._path_registry, "name", None, "column"
             ),
             self._make_path_registry([User, "name"]),
         )
@@ -159,6 +170,7 @@ class LoadTest(PathTest, QueryTest):
             result._generate_path,
             result.path,
             User.addresses,
+            None,
             "relationship",
         )
 
@@ -175,6 +187,7 @@ class LoadTest(PathTest, QueryTest):
             result._generate_path,
             inspect(User)._path_registry,
             Order.items,
+            None,
             "relationship",
         )
 
@@ -188,6 +201,7 @@ class LoadTest(PathTest, QueryTest):
             result._generate_path(
                 inspect(User)._path_registry,
                 Order.items,
+                None,
                 "relationship",
                 False,
             ),
@@ -794,7 +808,6 @@ class OptionsTest(PathTest, QueryTest):
     def test_chained(self):
         User = self.classes.User
         Order = self.classes.Order
-        Item = self.classes.Item
         sess = Session()
         q = sess.query(User)
         opt = self._option_fixture(User.orders).joinedload("items")
@@ -931,8 +944,8 @@ class OptionsNoPropTest(_fixtures.FixtureTest):
         Item = self.classes.Item
 
         message = (
-            "Query has only expression-based entities - "
-            "can't find property named 'keywords'."
+            "Query has only expression-based entities - can't "
+            'find property named "keywords".'
         )
         self._assert_eager_with_just_column_exception(
             Item.id, "keywords", message
@@ -944,8 +957,8 @@ class OptionsNoPropTest(_fixtures.FixtureTest):
         self._assert_eager_with_just_column_exception(
             Item.id,
             Item.keywords,
-            "Query has only expression-based entities "
-            "- can't find property named 'keywords'.",
+            "Query has only expression-based entities, which do not apply "
+            'to relationship property "Item.keywords"',
         )
 
     def test_option_against_nonexistent_PropComparator(self):
@@ -954,10 +967,10 @@ class OptionsNoPropTest(_fixtures.FixtureTest):
         self._assert_eager_with_entity_exception(
             [Keyword],
             (joinedload(Item.keywords),),
-            r"Can't find property 'keywords' on any entity specified "
-            r"in this Query.  Note the full path from root "
-            r"\(Mapper\|Keyword\|keywords\) to target entity must be "
-            r"specified.",
+            'Mapped attribute "Item.keywords" does not apply to any of the '
+            "root entities in this query, e.g. mapped class "
+            "Keyword->keywords. Please specify the full path from one of "
+            "the root entities to the target attribute. ",
         )
 
     def test_option_against_nonexistent_basestring(self):
@@ -965,8 +978,8 @@ class OptionsNoPropTest(_fixtures.FixtureTest):
         self._assert_eager_with_entity_exception(
             [Item],
             (joinedload("foo"),),
-            r"Can't find property named 'foo' on the mapped "
-            r"entity Mapper\|Item\|items in this Query.",
+            'Can\'t find property named "foo" on mapped class '
+            "Item->items in this Query.",
         )
 
     def test_option_against_nonexistent_twolevel_basestring(self):
@@ -974,17 +987,17 @@ class OptionsNoPropTest(_fixtures.FixtureTest):
         self._assert_eager_with_entity_exception(
             [Item],
             (joinedload("keywords.foo"),),
-            r"Can't find property named 'foo' on the mapped entity "
-            r"Mapper\|Keyword\|keywords in this Query.",
+            'Can\'t find property named "foo" on mapped class '
+            "Keyword->keywords in this Query.",
         )
 
-    def test_option_against_nonexistent_twolevel_all(self):
+    def test_option_against_nonexistent_twolevel_chained(self):
         Item = self.classes.Item
         self._assert_eager_with_entity_exception(
             [Item],
-            (joinedload_all("keywords.foo"),),
-            r"Can't find property named 'foo' on the mapped entity "
-            r"Mapper\|Keyword\|keywords in this Query.",
+            (joinedload("keywords").joinedload("foo"),),
+            'Can\'t find property named "foo" on mapped class '
+            "Keyword->keywords in this Query.",
         )
 
     @testing.fails_if(
@@ -996,7 +1009,7 @@ class OptionsNoPropTest(_fixtures.FixtureTest):
         Keyword = self.classes.Keyword
         self._assert_eager_with_entity_exception(
             [Keyword, Item],
-            (joinedload_all("keywords"),),
+            (joinedload("keywords"),),
             r"Attribute 'keywords' of entity 'Mapper\|Keyword\|keywords' "
             "does not refer to a mapped entity",
         )
@@ -1010,28 +1023,64 @@ class OptionsNoPropTest(_fixtures.FixtureTest):
         Keyword = self.classes.Keyword
         self._assert_eager_with_entity_exception(
             [Keyword, Item],
-            (joinedload_all("keywords"),),
+            (joinedload("keywords"),),
             r"Attribute 'keywords' of entity 'Mapper\|Keyword\|keywords' "
             "does not refer to a mapped entity",
         )
 
     def test_option_against_wrong_entity_type_basestring(self):
         Item = self.classes.Item
-        self._assert_eager_with_entity_exception(
+        self._assert_loader_strategy_exception(
             [Item],
-            (joinedload_all("id", "keywords"),),
-            r"Attribute 'id' of entity 'Mapper\|Item\|items' does not "
-            r"refer to a mapped entity",
+            (joinedload("id").joinedload("keywords"),),
+            'Can\'t apply "joined loader" strategy to property "Item.id", '
+            'which is a "column property"; this loader strategy is '
+            'intended to be used with a "relationship property".',
+        )
+
+    def test_col_option_against_relationship_basestring(self):
+        Item = self.classes.Item
+        self._assert_loader_strategy_exception(
+            [Item],
+            (load_only("keywords"),),
+            'Can\'t apply "column loader" strategy to property '
+            '"Item.keywords", which is a "relationship property"; this '
+            'loader strategy is intended to be used with a "column property".',
+        )
+
+    def test_load_only_against_multi_entity_attr(self):
+        User = self.classes.User
+        Item = self.classes.Item
+        self._assert_eager_with_entity_exception(
+            [User, Item],
+            (load_only(User.id, Item.id),),
+            r"Can't apply wildcard \('\*'\) or load_only\(\) loader option "
+            "to multiple entities mapped class User->users, mapped class "
+            "Item->items. Specify loader options for each entity "
+            "individually, such as "
+            r"Load\(mapped class User->users\).some_option\('\*'\), "
+            r"Load\(mapped class Item->items\).some_option\('\*'\).",
+        )
+
+    def test_col_option_against_relationship_attr(self):
+        Item = self.classes.Item
+        self._assert_loader_strategy_exception(
+            [Item],
+            (load_only(Item.keywords),),
+            'Can\'t apply "column loader" strategy to property '
+            '"Item.keywords", which is a "relationship property"; this '
+            'loader strategy is intended to be used with a "column property".',
         )
 
     def test_option_against_multi_non_relation_twolevel_basestring(self):
         Item = self.classes.Item
         Keyword = self.classes.Keyword
-        self._assert_eager_with_entity_exception(
+        self._assert_loader_strategy_exception(
             [Keyword, Item],
-            (joinedload_all("id", "keywords"),),
-            r"Attribute 'id' of entity 'Mapper\|Keyword\|keywords' "
-            "does not refer to a mapped entity",
+            (joinedload("id").joinedload("keywords"),),
+            'Can\'t apply "joined loader" strategy to property "Keyword.id", '
+            'which is a "column property"; this loader strategy is intended '
+            'to be used with a "relationship property".',
         )
 
     def test_option_against_multi_nonexistent_basestring(self):
@@ -1039,9 +1088,9 @@ class OptionsNoPropTest(_fixtures.FixtureTest):
         Keyword = self.classes.Keyword
         self._assert_eager_with_entity_exception(
             [Keyword, Item],
-            (joinedload_all("description"),),
-            r"Can't find property named 'description' on the mapped "
-            r"entity Mapper\|Keyword\|keywords in this Query.",
+            (joinedload("description"),),
+            'Can\'t find property named "description" on mapped class '
+            "Keyword->keywords in this Query.",
         )
 
     def test_option_against_multi_no_entities_basestring(self):
@@ -1049,29 +1098,31 @@ class OptionsNoPropTest(_fixtures.FixtureTest):
         Keyword = self.classes.Keyword
         self._assert_eager_with_entity_exception(
             [Keyword.id, Item.id],
-            (joinedload_all("keywords"),),
+            (joinedload("keywords"),),
             r"Query has only expression-based entities - can't find property "
-            "named 'keywords'.",
+            'named "keywords".',
         )
 
     def test_option_against_wrong_multi_entity_type_attr_one(self):
         Item = self.classes.Item
         Keyword = self.classes.Keyword
-        self._assert_eager_with_entity_exception(
+        self._assert_loader_strategy_exception(
             [Keyword, Item],
-            (joinedload_all(Keyword.id, Item.keywords),),
-            r"Attribute 'id' of entity 'Mapper\|Keyword\|keywords' "
-            "does not refer to a mapped entity",
+            (joinedload(Keyword.id).joinedload(Item.keywords),),
+            'Can\'t apply "joined loader" strategy to property "Keyword.id", '
+            'which is a "column property"; this loader strategy is intended '
+            'to be used with a "relationship property".',
         )
 
     def test_option_against_wrong_multi_entity_type_attr_two(self):
         Item = self.classes.Item
         Keyword = self.classes.Keyword
-        self._assert_eager_with_entity_exception(
+        self._assert_loader_strategy_exception(
             [Keyword, Item],
-            (joinedload_all(Keyword.keywords, Item.keywords),),
-            r"Attribute 'keywords' of entity 'Mapper\|Keyword\|keywords' "
-            "does not refer to a mapped entity",
+            (joinedload(Keyword.keywords).joinedload(Item.keywords),),
+            'Can\'t apply "joined loader" strategy to property '
+            '"Keyword.keywords", which is a "column property"; this loader '
+            'strategy is intended to be used with a "relationship property".',
         )
 
     def test_option_against_wrong_multi_entity_type_attr_three(self):
@@ -1079,9 +1130,9 @@ class OptionsNoPropTest(_fixtures.FixtureTest):
         Keyword = self.classes.Keyword
         self._assert_eager_with_entity_exception(
             [Keyword.id, Item.id],
-            (joinedload_all(Keyword.keywords, Item.keywords),),
-            r"Query has only expression-based entities - "
-            "can't find property named 'keywords'.",
+            (joinedload(Keyword.keywords).joinedload(Item.keywords),),
+            "Query has only expression-based entities, which do not apply to "
+            'column property "Keyword.keywords"',
         )
 
     def test_wrong_type_in_option(self):
@@ -1089,7 +1140,7 @@ class OptionsNoPropTest(_fixtures.FixtureTest):
         Keyword = self.classes.Keyword
         self._assert_eager_with_entity_exception(
             [Item],
-            (joinedload_all(Keyword),),
+            (joinedload(Keyword),),
             r"mapper option expects string key or list of attributes",
         )
 
@@ -1097,7 +1148,7 @@ class OptionsNoPropTest(_fixtures.FixtureTest):
         User = self.classes.User
         self._assert_eager_with_entity_exception(
             [User],
-            (joinedload_all(User.addresses, User.orders),),
+            (joinedload(User.addresses).joinedload(User.orders),),
             r"Attribute 'User.orders' does not link "
             "from element 'Mapper|Address|addresses'",
         )
@@ -1107,7 +1158,11 @@ class OptionsNoPropTest(_fixtures.FixtureTest):
         Order = self.classes.Order
         self._assert_eager_with_entity_exception(
             [User],
-            (joinedload_all(User.addresses, User.orders.of_type(Order)),),
+            (
+                joinedload(User.addresses).joinedload(
+                    User.orders.of_type(Order)
+                ),
+            ),
             r"Attribute 'User.orders' does not link "
             "from element 'Mapper|Address|addresses'",
         )
@@ -1161,6 +1216,14 @@ class OptionsNoPropTest(_fixtures.FixtureTest):
         key = ("loader", (inspect(Item), inspect(Item).attrs.keywords))
         assert key in q._attributes
 
+    def _assert_loader_strategy_exception(self, entity_list, options, message):
+        assert_raises_message(
+            orm_exc.LoaderStrategyException,
+            message,
+            create_session().query(*entity_list).options,
+            *options
+        )
+
     def _assert_eager_with_entity_exception(
         self, entity_list, options, message
     ):
@@ -1182,6 +1245,102 @@ class OptionsNoPropTest(_fixtures.FixtureTest):
         )
 
 
+class OptionsNoPropTestInh(_Polymorphic):
+    def test_missing_attr_wpoly_subclasss(self):
+        s = Session()
+
+        wp = with_polymorphic(Person, [Manager], flat=True)
+
+        assert_raises_message(
+            sa.exc.ArgumentError,
+            r'Mapped attribute "Manager.status" does not apply to any of '
+            r"the root entities in this query, e.g. "
+            r"with_polymorphic\(Person, \[Manager\]\).",
+            s.query(wp).options,
+            load_only(Manager.status),
+        )
+
+    def test_missing_attr_of_type_subclass(self):
+        s = Session()
+
+        assert_raises_message(
+            sa.exc.ArgumentError,
+            r'Attribute "Manager.manager_name" does not link from element '
+            r'"with_polymorphic\(Person, \[Engineer\]\)".$',
+            s.query(Company).options,
+            joinedload(Company.employees.of_type(Engineer)).load_only(
+                Manager.manager_name
+            ),
+        )
+
+    def test_missing_attr_of_type_subclass_name_matches(self):
+        s = Session()
+
+        # the name "status" is present on Engineer also, make sure
+        # that doesn't get mixed up here
+        assert_raises_message(
+            sa.exc.ArgumentError,
+            r'Attribute "Manager.status" does not link from element '
+            r'"with_polymorphic\(Person, \[Engineer\]\)".$',
+            s.query(Company).options,
+            joinedload(Company.employees.of_type(Engineer)).load_only(
+                Manager.status
+            ),
+        )
+
+    def test_missing_str_attr_of_type_subclass(self):
+        s = Session()
+
+        assert_raises_message(
+            sa.exc.ArgumentError,
+            r'Can\'t find property named "manager_name" on '
+            r"mapped class Engineer->engineers in this Query.$",
+            s.query(Company).options,
+            joinedload(Company.employees.of_type(Engineer)).load_only(
+                "manager_name"
+            ),
+        )
+
+    def test_missing_attr_of_type_wpoly_subclass(self):
+        s = Session()
+
+        wp = with_polymorphic(Person, [Manager], flat=True)
+
+        assert_raises_message(
+            sa.exc.ArgumentError,
+            r'Attribute "Manager.manager_name" does not link from '
+            r'element "with_polymorphic\(Person, \[Manager\]\)".$',
+            s.query(Company).options,
+            joinedload(Company.employees.of_type(wp)).load_only(
+                Manager.manager_name
+            ),
+        )
+
+    def test_missing_attr_is_missing_of_type_for_alias(self):
+        s = Session()
+
+        pa = aliased(Person)
+
+        assert_raises_message(
+            sa.exc.ArgumentError,
+            r'Attribute "AliasedClass_Person.name" does not link from '
+            r'element "mapped class Person->people".  Did you mean to use '
+            r"Company.employees.of_type\(AliasedClass_Person\)\?",
+            s.query(Company).options,
+            joinedload(Company.employees).load_only(pa.name),
+        )
+
+        q = s.query(Company).options(
+            joinedload(Company.employees.of_type(pa)).load_only(pa.name)
+        )
+        orig_path = inspect(Company)._path_registry[
+            Company.employees.property
+        ][inspect(pa)][pa.name.property]
+        key = ("loader", orig_path.natural_path)
+        loader = q._attributes[key]
+        eq_(loader.path, orig_path)
+
+
 class PickleTest(PathTest, QueryTest):
     def _option_fixture(self, *arg):
         return strategy_options._UnboundLoad._from_keys(
@@ -1191,9 +1350,6 @@ class PickleTest(PathTest, QueryTest):
     def test_modern_opt_getstate(self):
         User = self.classes.User
 
-        sess = Session()
-        q = sess.query(User)
-
         opt = self._option_fixture(User.addresses)
         eq_(
             opt.__getstate__(),
@@ -1388,6 +1544,252 @@ class LocalOptsTest(PathTest, QueryTest):
         self._assert_attrs(opts, {"foo": "bar", "bat": "hoho"})
 
 
+class SubOptionsTest(PathTest, QueryTest):
+    run_create_tables = False
+    run_inserts = None
+    run_deletes = None
+
+    def _assert_opts(self, q, sub_opt, non_sub_opts):
+        existing_attributes = q._attributes
+        q._attributes = q._attributes.copy()
+        attr_a = {}
+
+        for val in sub_opt._to_bind:
+            val._bind_loader(
+                [ent.entity_zero for ent in q._mapper_entities],
+                q._current_path,
+                attr_a,
+                False,
+            )
+
+        q._attributes = existing_attributes.copy()
+
+        attr_b = {}
+
+        for opt in non_sub_opts:
+            for val in opt._to_bind:
+                val._bind_loader(
+                    [ent.entity_zero for ent in q._mapper_entities],
+                    q._current_path,
+                    attr_b,
+                    False,
+                )
+
+        for k, l in attr_b.items():
+            if not l.strategy:
+                del attr_b[k]
+
+        def strat_as_tuple(strat):
+            return (
+                strat.strategy,
+                strat.local_opts,
+                strat.propagate_to_loaders,
+                strat._of_type,
+                strat.is_class_strategy,
+                strat.is_opts_only,
+            )
+
+        eq_(
+            {path: strat_as_tuple(load) for path, load in attr_a.items()},
+            {path: strat_as_tuple(load) for path, load in attr_b.items()},
+        )
+
+    def test_one(self):
+        User, Address, Order, Item, SubItem = self.classes(
+            "User", "Address", "Order", "Item", "SubItem"
+        )
+        sub_opt = joinedload(User.orders).options(
+            joinedload(Order.items).options(defer(Item.description)),
+            defer(Order.description),
+        )
+        non_sub_opts = [
+            joinedload(User.orders),
+            defaultload(User.orders)
+            .joinedload(Order.items)
+            .defer(Item.description),
+            defaultload(User.orders).defer(Order.description),
+        ]
+
+        sess = Session()
+        self._assert_opts(sess.query(User), sub_opt, non_sub_opts)
+
+    def test_two(self):
+        User, Address, Order, Item, SubItem = self.classes(
+            "User", "Address", "Order", "Item", "SubItem"
+        )
+        sub_opt = defaultload(User.orders).options(
+            joinedload(Order.items),
+            defaultload(Order.items).options(subqueryload(Item.keywords)),
+            defer(Order.description),
+        )
+        non_sub_opts = [
+            defaultload(User.orders)
+            .joinedload(Order.items)
+            .subqueryload(Item.keywords),
+            defaultload(User.orders).defer(Order.description),
+        ]
+
+        sess = Session()
+        self._assert_opts(sess.query(User), sub_opt, non_sub_opts)
+
+    def test_three(self):
+        User, Address, Order, Item, SubItem = self.classes(
+            "User", "Address", "Order", "Item", "SubItem"
+        )
+        sub_opt = defaultload(User.orders).options(defer("*"))
+        non_sub_opts = [defaultload(User.orders).defer("*")]
+        sess = Session()
+        self._assert_opts(sess.query(User), sub_opt, non_sub_opts)
+
+    def test_four(self):
+        User, Address, Order, Item, SubItem, Keyword = self.classes(
+            "User", "Address", "Order", "Item", "SubItem", "Keyword"
+        )
+        sub_opt = joinedload(User.orders).options(
+            defer(Order.description),
+            joinedload(Order.items).options(
+                joinedload(Item.keywords).options(defer(Keyword.name)),
+                defer(Item.description),
+            ),
+        )
+        non_sub_opts = [
+            joinedload(User.orders),
+            defaultload(User.orders).defer(Order.description),
+            defaultload(User.orders).joinedload(Order.items),
+            defaultload(User.orders)
+            .defaultload(Order.items)
+            .joinedload(Item.keywords),
+            defaultload(User.orders)
+            .defaultload(Order.items)
+            .defer(Item.description),
+            defaultload(User.orders)
+            .defaultload(Order.items)
+            .defaultload(Item.keywords)
+            .defer(Keyword.name),
+        ]
+        sess = Session()
+        self._assert_opts(sess.query(User), sub_opt, non_sub_opts)
+
+    def test_four_strings(self):
+        User, Address, Order, Item, SubItem, Keyword = self.classes(
+            "User", "Address", "Order", "Item", "SubItem", "Keyword"
+        )
+        sub_opt = joinedload("orders").options(
+            defer("description"),
+            joinedload("items").options(
+                joinedload("keywords").options(defer("name")),
+                defer("description"),
+            ),
+        )
+        non_sub_opts = [
+            joinedload(User.orders),
+            defaultload(User.orders).defer(Order.description),
+            defaultload(User.orders).joinedload(Order.items),
+            defaultload(User.orders)
+            .defaultload(Order.items)
+            .joinedload(Item.keywords),
+            defaultload(User.orders)
+            .defaultload(Order.items)
+            .defer(Item.description),
+            defaultload(User.orders)
+            .defaultload(Order.items)
+            .defaultload(Item.keywords)
+            .defer(Keyword.name),
+        ]
+        sess = Session()
+        self._assert_opts(sess.query(User), sub_opt, non_sub_opts)
+
+    def test_five(self):
+        User, Address, Order, Item, SubItem, Keyword = self.classes(
+            "User", "Address", "Order", "Item", "SubItem", "Keyword"
+        )
+        sub_opt = joinedload(User.orders).options(load_only(Order.description))
+        non_sub_opts = [
+            joinedload(User.orders),
+            defaultload(User.orders).load_only(Order.description),
+        ]
+        sess = Session()
+        self._assert_opts(sess.query(User), sub_opt, non_sub_opts)
+
+    def test_five_strings(self):
+        User, Address, Order, Item, SubItem, Keyword = self.classes(
+            "User", "Address", "Order", "Item", "SubItem", "Keyword"
+        )
+        sub_opt = joinedload("orders").options(load_only("description"))
+        non_sub_opts = [
+            joinedload(User.orders),
+            defaultload(User.orders).load_only(Order.description),
+        ]
+        sess = Session()
+        self._assert_opts(sess.query(User), sub_opt, non_sub_opts)
+
+    def test_invalid_one(self):
+        User, Address, Order, Item, SubItem = self.classes(
+            "User", "Address", "Order", "Item", "SubItem"
+        )
+
+        # these options are "invalid", in that User.orders -> Item.keywords
+        # is not a path.  However, the "normal" option is not generating
+        # an error for now, which is bad, but we're testing here only that
+        # it works the same way, so there you go.   If and when we make this
+        # case raise, then both cases should raise in the same way.
+        sub_opt = joinedload(User.orders).options(
+            joinedload(Item.keywords), joinedload(Order.items)
+        )
+        non_sub_opts = [
+            joinedload(User.orders).joinedload(Item.keywords),
+            defaultload(User.orders).joinedload(Order.items),
+        ]
+        sess = Session()
+        self._assert_opts(sess.query(User), sub_opt, non_sub_opts)
+
+    def test_invalid_two(self):
+        User, Address, Order, Item, SubItem = self.classes(
+            "User", "Address", "Order", "Item", "SubItem"
+        )
+
+        # these options are "invalid", in that User.orders -> Item.keywords
+        # is not a path.  However, the "normal" option is not generating
+        # an error for now, which is bad, but we're testing here only that
+        # it works the same way, so there you go.   If and when we make this
+        # case raise, then both cases should raise in the same way.
+        sub_opt = joinedload("orders").options(
+            joinedload("keywords"), joinedload("items")
+        )
+        non_sub_opts = [
+            joinedload(User.orders).joinedload(Item.keywords),
+            defaultload(User.orders).joinedload(Order.items),
+        ]
+        sess = Session()
+        self._assert_opts(sess.query(User), sub_opt, non_sub_opts)
+
+    def test_not_implemented_fromload(self):
+        User, Address, Order, Item, SubItem = self.classes(
+            "User", "Address", "Order", "Item", "SubItem"
+        )
+
+        assert_raises_message(
+            NotImplementedError,
+            r"The options\(\) method is currently only supported "
+            "for 'unbound' loader options",
+            Load(User).joinedload(User.orders).options,
+            joinedload(Order.items),
+        )
+
+    def test_not_implemented_toload(self):
+        User, Address, Order, Item, SubItem = self.classes(
+            "User", "Address", "Order", "Item", "SubItem"
+        )
+
+        assert_raises_message(
+            NotImplementedError,
+            r"Only 'unbound' loader options may be used with the "
+            r"Load.options\(\) method",
+            joinedload(User.orders).options,
+            Load(Order).joinedload(Order.items),
+        )
+
+
 class CacheKeyTest(PathTest, QueryTest):
 
     run_create_tables = False
