@@ -3,10 +3,13 @@ from sqlalchemy import bindparam
 from sqlalchemy import ForeignKey
 from sqlalchemy import inspect
 from sqlalchemy import Integer
+from sqlalchemy import literal_column
+from sqlalchemy import select
 from sqlalchemy import String
 from sqlalchemy import testing
 from sqlalchemy.orm import aliased
 from sqlalchemy.orm import clear_mappers
+from sqlalchemy.orm import close_all_sessions
 from sqlalchemy.orm import create_session
 from sqlalchemy.orm import deferred
 from sqlalchemy.orm import joinedload
@@ -14,7 +17,6 @@ from sqlalchemy.orm import mapper
 from sqlalchemy.orm import relationship
 from sqlalchemy.orm import Session
 from sqlalchemy.orm import subqueryload
-from sqlalchemy.orm import subqueryload_all
 from sqlalchemy.orm import undefer
 from sqlalchemy.orm import with_polymorphic
 from sqlalchemy.testing import assert_raises
@@ -136,7 +138,7 @@ class EagerTest(_fixtures.FixtureTest, testing.AssertsCompiledSQL):
         self.assert_sql_count(testing.db, go, 2)
 
         q = sess.query(u).options(
-            subqueryload_all(u.addresses, Address.dingalings)
+            subqueryload(u.addresses).subqueryload(Address.dingalings)
         )
 
         def go():
@@ -791,32 +793,116 @@ class EagerTest(_fixtures.FixtureTest, testing.AssertsCompiledSQL):
         sess = create_session()
 
         self.assert_compile(
-            sess.query(User, "1"),
+            sess.query(User, literal_column("1")),
             "SELECT users.id AS users_id, users.name AS users_name, "
             "1 FROM users",
         )
 
-    def test_double(self):
-        """Eager loading with two relationships simultaneously,
-            from the same table, using aliases."""
+    def test_double_w_ac_against_subquery(self):
 
-        users, orders, User, Address, Order, addresses = (
+        (
+            users,
+            orders,
+            User,
+            Address,
+            Order,
+            addresses,
+            Item,
+            items,
+            order_items,
+        ) = (
             self.tables.users,
             self.tables.orders,
             self.classes.User,
             self.classes.Address,
             self.classes.Order,
             self.tables.addresses,
+            self.classes.Item,
+            self.tables.items,
+            self.tables.order_items,
         )
 
-        openorders = sa.alias(orders, "openorders")
-        closedorders = sa.alias(orders, "closedorders")
+        mapper(Address, addresses)
+        mapper(
+            Order,
+            orders,
+            properties={
+                "items": relationship(
+                    Item,
+                    secondary=order_items,
+                    lazy="subquery",
+                    order_by=items.c.id,
+                )
+            },
+        )
+        mapper(Item, items)
+
+        open_mapper = aliased(
+            Order, select([orders]).where(orders.c.isopen == 1).alias()
+        )
+        closed_mapper = aliased(
+            Order, select([orders]).where(orders.c.isopen == 0).alias()
+        )
+
+        mapper(
+            User,
+            users,
+            properties=dict(
+                addresses=relationship(
+                    Address, lazy="subquery", order_by=addresses.c.id
+                ),
+                open_orders=relationship(
+                    open_mapper, lazy="subquery", order_by=open_mapper.id
+                ),
+                closed_orders=relationship(
+                    closed_mapper, lazy="subquery", order_by=closed_mapper.id
+                ),
+            ),
+        )
+
+        self._run_double_test()
+
+    def test_double_w_ac(self):
+
+        (
+            users,
+            orders,
+            User,
+            Address,
+            Order,
+            addresses,
+            Item,
+            items,
+            order_items,
+        ) = (
+            self.tables.users,
+            self.tables.orders,
+            self.classes.User,
+            self.classes.Address,
+            self.classes.Order,
+            self.tables.addresses,
+            self.classes.Item,
+            self.tables.items,
+            self.tables.order_items,
+        )
 
         mapper(Address, addresses)
-        mapper(Order, orders)
+        mapper(
+            Order,
+            orders,
+            properties={
+                "items": relationship(
+                    Item,
+                    secondary=order_items,
+                    lazy="subquery",
+                    order_by=items.c.id,
+                )
+            },
+        )
+        mapper(Item, items)
 
-        open_mapper = mapper(Order, openorders, non_primary=True)
-        closed_mapper = mapper(Order, closedorders, non_primary=True)
+        open_mapper = aliased(Order, orders)
+        closed_mapper = aliased(Order, orders)
 
         mapper(
             User,
@@ -828,57 +914,25 @@ class EagerTest(_fixtures.FixtureTest, testing.AssertsCompiledSQL):
                 open_orders=relationship(
                     open_mapper,
                     primaryjoin=sa.and_(
-                        openorders.c.isopen == 1,
-                        users.c.id == openorders.c.user_id,
+                        open_mapper.isopen == 1,
+                        users.c.id == open_mapper.user_id,
                     ),
                     lazy="subquery",
-                    order_by=openorders.c.id,
+                    order_by=open_mapper.id,
                 ),
                 closed_orders=relationship(
                     closed_mapper,
                     primaryjoin=sa.and_(
-                        closedorders.c.isopen == 0,
-                        users.c.id == closedorders.c.user_id,
+                        closed_mapper.isopen == 0,
+                        users.c.id == closed_mapper.user_id,
                     ),
                     lazy="subquery",
-                    order_by=closedorders.c.id,
+                    order_by=closed_mapper.id,
                 ),
             ),
         )
 
-        q = create_session().query(User).order_by(User.id)
-
-        def go():
-            eq_(
-                [
-                    User(
-                        id=7,
-                        addresses=[Address(id=1)],
-                        open_orders=[Order(id=3)],
-                        closed_orders=[Order(id=1), Order(id=5)],
-                    ),
-                    User(
-                        id=8,
-                        addresses=[
-                            Address(id=2),
-                            Address(id=3),
-                            Address(id=4),
-                        ],
-                        open_orders=[],
-                        closed_orders=[],
-                    ),
-                    User(
-                        id=9,
-                        addresses=[Address(id=5)],
-                        open_orders=[Order(id=4)],
-                        closed_orders=[Order(id=2)],
-                    ),
-                    User(id=10),
-                ],
-                q.all(),
-            )
-
-        self.assert_sql_count(testing.db, go, 4)
+        self._run_double_test()
 
     def test_double_same_mappers(self):
         """Eager loading with two relationships simultaneously,
@@ -945,26 +999,30 @@ class EagerTest(_fixtures.FixtureTest, testing.AssertsCompiledSQL):
                 ),
             ),
         )
+        self._run_double_test()
+
+    def _run_double_test(self, no_items=False):
+        User, Address, Order, Item = self.classes(
+            "User", "Address", "Order", "Item"
+        )
         q = create_session().query(User).order_by(User.id)
 
+        def items(*ids):
+            if no_items:
+                return {}
+            else:
+                return {"items": [Item(id=id_) for id_ in ids]}
+
         def go():
             eq_(
                 [
                     User(
                         id=7,
                         addresses=[Address(id=1)],
-                        open_orders=[
-                            Order(
-                                id=3,
-                                items=[Item(id=3), Item(id=4), Item(id=5)],
-                            )
-                        ],
+                        open_orders=[Order(id=3, **items(3, 4, 5))],
                         closed_orders=[
-                            Order(
-                                id=1,
-                                items=[Item(id=1), Item(id=2), Item(id=3)],
-                            ),
-                            Order(id=5, items=[Item(id=5)]),
+                            Order(id=1, **items(1, 2, 3)),
+                            Order(id=5, **items(5)),
                         ],
                     ),
                     User(
@@ -980,22 +1038,18 @@ class EagerTest(_fixtures.FixtureTest, testing.AssertsCompiledSQL):
                     User(
                         id=9,
                         addresses=[Address(id=5)],
-                        open_orders=[
-                            Order(id=4, items=[Item(id=1), Item(id=5)])
-                        ],
-                        closed_orders=[
-                            Order(
-                                id=2,
-                                items=[Item(id=1), Item(id=2), Item(id=3)],
-                            )
-                        ],
+                        open_orders=[Order(id=4, **items(1, 5))],
+                        closed_orders=[Order(id=2, **items(1, 2, 3))],
                     ),
                     User(id=10),
                 ],
                 q.all(),
             )
 
-        self.assert_sql_count(testing.db, go, 6)
+        if no_items:
+            self.assert_sql_count(testing.db, go, 4)
+        else:
+            self.assert_sql_count(testing.db, go, 6)
 
     def test_limit(self):
         """Limit operations combined with lazy-load relationships."""
@@ -1059,33 +1113,6 @@ class EagerTest(_fixtures.FixtureTest, testing.AssertsCompiledSQL):
         result = q.order_by(sa.desc(User.id)).limit(2).offset(2).all()
         eq_(list(reversed(self.static.user_all_result[0:2])), result)
 
-    @testing.uses_deprecated("Mapper.order_by")
-    def test_mapper_order_by(self):
-        users, User, Address, addresses = (
-            self.tables.users,
-            self.classes.User,
-            self.classes.Address,
-            self.tables.addresses,
-        )
-
-        mapper(Address, addresses)
-        mapper(
-            User,
-            users,
-            properties={
-                "addresses": relationship(
-                    Address, lazy="subquery", order_by=addresses.c.id
-                )
-            },
-            order_by=users.c.id.desc(),
-        )
-
-        sess = create_session()
-        q = sess.query(User)
-
-        result = q.limit(2).all()
-        eq_(result, list(reversed(self.static.user_address_result[2:4])))
-
     def test_one_to_many_scalar(self):
         Address, addresses, users, User = (
             self.classes.Address,
@@ -1166,9 +1193,7 @@ class EagerTest(_fixtures.FixtureTest, testing.AssertsCompiledSQL):
                     order_by=orders.c.id,
                 ),
                 "max_order": relationship(
-                    mapper(Order, max_orders, non_primary=True),
-                    lazy="subquery",
-                    uselist=False,
+                    aliased(Order, max_orders), lazy="subquery", uselist=False
                 ),
             },
         )
@@ -1339,7 +1364,7 @@ class LoadOnExistingTest(_fixtures.FixtureTest):
         a2 = u1.addresses[0]
         a2.email_address = "foo"
         sess.query(User).options(
-            subqueryload_all("addresses.dingaling")
+            subqueryload("addresses").subqueryload("dingaling")
         ).filter_by(id=8).all()
         assert u1.addresses[-1] is a1
         for a in u1.addresses:
@@ -1357,9 +1382,9 @@ class LoadOnExistingTest(_fixtures.FixtureTest):
         u1.orders
         o1 = Order()
         u1.orders.append(o1)
-        sess.query(User).options(subqueryload_all("orders.items")).filter_by(
-            id=7
-        ).all()
+        sess.query(User).options(
+            subqueryload("orders").subqueryload("items")
+        ).filter_by(id=7).all()
         for o in u1.orders:
             if o is not o1:
                 assert "items" in o.__dict__
@@ -1376,7 +1401,7 @@ class LoadOnExistingTest(_fixtures.FixtureTest):
             .one()
         )
         sess.query(User).filter_by(id=8).options(
-            subqueryload_all("addresses.dingaling")
+            subqueryload("addresses").subqueryload("dingaling")
         ).first()
         assert "dingaling" in u1.addresses[0].__dict__
 
@@ -1390,7 +1415,7 @@ class LoadOnExistingTest(_fixtures.FixtureTest):
             .one()
         )
         sess.query(User).filter_by(id=7).options(
-            subqueryload_all("orders.items")
+            subqueryload("orders").subqueryload("items")
         ).first()
         assert "items" in u1.orders[0].__dict__
 
@@ -1481,7 +1506,7 @@ class OrderBySecondaryTest(fixtures.MappedTest):
 class BaseRelationFromJoinedSubclassTest(_Polymorphic):
     @classmethod
     def define_tables(cls, metadata):
-        people = Table(
+        Table(
             "people",
             metadata,
             Column(
@@ -1496,7 +1521,7 @@ class BaseRelationFromJoinedSubclassTest(_Polymorphic):
 
         # to test fully, PK of engineers table must be
         # named differently from that of people
-        engineers = Table(
+        Table(
             "engineers",
             metadata,
             Column(
@@ -1508,7 +1533,7 @@ class BaseRelationFromJoinedSubclassTest(_Polymorphic):
             Column("primary_language", String(50)),
         )
 
-        paperwork = Table(
+        Table(
             "paperwork",
             metadata,
             Column(
@@ -1521,7 +1546,7 @@ class BaseRelationFromJoinedSubclassTest(_Polymorphic):
             Column("person_id", Integer, ForeignKey("people.person_id")),
         )
 
-        pages = Table(
+        Table(
             "pages",
             metadata,
             Column(
@@ -2327,7 +2352,7 @@ class SelfReferentialTest(fixtures.MappedTest):
                 sess.query(Node)
                 .filter_by(data="n1")
                 .order_by(Node.id)
-                .options(subqueryload_all("children.children"))
+                .options(subqueryload("children").subqueryload("children"))
                 .first()
             )
             eq_(
@@ -2666,8 +2691,8 @@ class CyclicalInheritingEagerTestTwo(
         session.add_all([rscott, alien, brunner])
         session.commit()
 
-        session.close_all()
-        d = session.query(Director).options(subqueryload("*")).first()
+        close_all_sessions()
+        d = session.query(Director).options(subqueryload("*")).first()  # noqa
         assert len(list(session)) == 3
 
 
@@ -2828,7 +2853,7 @@ class SubqueryloadDistinctTest(
                 ),
             )
 
-        movies = q.all()
+        movies = q.all()  # noqa
 
         # check number of persistent objects in session
         eq_(len(list(s)), 5)
