@@ -5,20 +5,24 @@ import datetime
 
 import sqlalchemy as sa
 from sqlalchemy import Boolean
+from sqlalchemy import Enum
 from sqlalchemy import event
 from sqlalchemy import ForeignKey
 from sqlalchemy import func
 from sqlalchemy import Integer
+from sqlalchemy import literal
 from sqlalchemy import literal_column
 from sqlalchemy import select
 from sqlalchemy import String
 from sqlalchemy import testing
+from sqlalchemy.inspection import inspect
 from sqlalchemy.orm import column_property
 from sqlalchemy.orm import create_session
 from sqlalchemy.orm import exc as orm_exc
 from sqlalchemy.orm import mapper
 from sqlalchemy.orm import relationship
 from sqlalchemy.orm import Session
+from sqlalchemy.orm.persistence import _sort_states
 from sqlalchemy.testing import assert_raises
 from sqlalchemy.testing import assert_raises_message
 from sqlalchemy.testing import eq_
@@ -27,6 +31,7 @@ from sqlalchemy.testing.assertsql import AllOf
 from sqlalchemy.testing.assertsql import CompiledSQL
 from sqlalchemy.testing.schema import Column
 from sqlalchemy.testing.schema import Table
+from sqlalchemy.util import OrderedDict
 from sqlalchemy.util import u
 from sqlalchemy.util import ue
 from test.orm import _fixtures
@@ -442,7 +447,7 @@ class ForeignPKTest(fixtures.MappedTest):
             self.tables.people,
         )
 
-        m1 = mapper(PersonSite, peoplesites)
+        mapper(PersonSite, peoplesites)
         m2 = mapper(
             Person, people, properties={"sites": relationship(PersonSite)}
         )
@@ -499,6 +504,15 @@ class ClauseAttributesTest(fixtures.MappedTest):
             Column("value", Boolean),
         )
 
+        Table(
+            "pk_t",
+            metadata,
+            Column(
+                "p_id", Integer, key="id", autoincrement=True, primary_key=True
+            ),
+            Column("data", String(30)),
+        )
+
     @classmethod
     def setup_classes(cls):
         class User(cls.Comparable):
@@ -507,12 +521,17 @@ class ClauseAttributesTest(fixtures.MappedTest):
         class HasBoolean(cls.Comparable):
             pass
 
+        class PkDefault(cls.Comparable):
+            pass
+
     @classmethod
     def setup_mappers(cls):
         User, users_t = cls.classes.User, cls.tables.users_t
         HasBoolean, boolean_t = cls.classes.HasBoolean, cls.tables.boolean_t
+        PkDefault, pk_t = cls.classes.PkDefault, cls.tables.pk_t
         mapper(User, users_t)
         mapper(HasBoolean, boolean_t)
+        mapper(PkDefault, pk_t)
 
     def test_update(self):
         User = self.classes.User
@@ -568,6 +587,19 @@ class ClauseAttributesTest(fixtures.MappedTest):
 
         assert (u.counter == 5) is True
 
+    @testing.requires.sql_expressions_inserted_as_primary_key
+    def test_insert_pk_expression(self):
+        PkDefault = self.classes.PkDefault
+
+        pk = PkDefault(id=literal(5) + 10, data="some data")
+        session = Session()
+        session.add(pk)
+        session.flush()
+
+        eq_(pk.id, 15)
+        session.commit()
+        eq_(pk.id, 15)
+
     def test_update_special_comparator(self):
         HasBoolean = self.classes.HasBoolean
 
@@ -891,8 +923,7 @@ class ExtraPassiveDeletesTest(fixtures.MappedTest):
         mc.children[0].data = "some new data"
         assert_raises(sa.exc.DBAPIError, session.flush)
 
-    def test_extra_passive_obj_removed_o2m_still_nulls_out(self):
-        # see #3844, which we decided was not a bug
+    def test_extra_passive_obj_removed_o2m(self):
         myothertable, MyClass, MyOtherClass, mytable = (
             self.tables.myothertable,
             self.classes.MyClass,
@@ -911,15 +942,20 @@ class ExtraPassiveDeletesTest(fixtures.MappedTest):
 
         session = create_session()
         mc = MyClass()
-        moc = MyOtherClass()
-        mc.children.append(moc)
-        session.add_all([mc, moc])
+        moc1 = MyOtherClass()
+        moc2 = MyOtherClass()
+        mc.children.append(moc1)
+        mc.children.append(moc2)
+        session.add_all([mc, moc1, moc2])
         session.flush()
 
-        mc.children.remove(moc)
+        mc.children.remove(moc1)
+        mc.children.remove(moc2)
+        moc1.data = "foo"
         session.flush()
 
-        eq_(moc.parent_id, None)
+        eq_(moc1.parent_id, mc.id)
+        eq_(moc2.parent_id, mc.id)
 
     def test_dont_emit(self):
         myothertable, MyClass, MyOtherClass, mytable = (
@@ -1098,7 +1134,7 @@ class DefaultTest(fixtures.MappedTest):
         eq_(h3.hoho, althohoval)
 
         def go():
-            # test deferred load of attribues, one select per instance
+            # test deferred load of attributes, one select per instance
             self.assert_(h2.hoho == h4.hoho == h5.hoho == hohoval)
 
         self.sql_count_(3, go)
@@ -1447,7 +1483,7 @@ class OneToManyTest(_fixtures.FixtureTest):
             self.classes.User,
         )
 
-        m = mapper(
+        mapper(
             User,
             users,
             properties=dict(
@@ -1507,7 +1543,7 @@ class OneToManyTest(_fixtures.FixtureTest):
             self.classes.User,
         )
 
-        m = mapper(
+        mapper(
             User,
             users,
             properties=dict(
@@ -1576,7 +1612,7 @@ class OneToManyTest(_fixtures.FixtureTest):
             self.classes.User,
         )
 
-        m = mapper(
+        mapper(
             User,
             users,
             properties=dict(
@@ -1613,7 +1649,7 @@ class OneToManyTest(_fixtures.FixtureTest):
             self.classes.User,
         )
 
-        m = mapper(
+        mapper(
             User,
             users,
             properties=dict(
@@ -1649,7 +1685,7 @@ class OneToManyTest(_fixtures.FixtureTest):
             self.classes.User,
         )
 
-        m = mapper(
+        mapper(
             User,
             users,
             properties=dict(
@@ -1685,7 +1721,7 @@ class OneToManyTest(_fixtures.FixtureTest):
             self.classes.User,
         )
 
-        m = mapper(
+        mapper(
             User,
             users,
             properties=dict(
@@ -1717,7 +1753,7 @@ class OneToManyTest(_fixtures.FixtureTest):
         )
 
         m1 = mapper(User, users)
-        m2 = mapper(
+        mapper(
             Address,
             addresses,
             properties=dict(
@@ -1726,7 +1762,7 @@ class OneToManyTest(_fixtures.FixtureTest):
         )
 
         u = User(name="test")
-        a = Address(email_address="testaddress", user=u)
+        Address(email_address="testaddress", user=u)
 
         session = create_session()
         session.add(u)
@@ -1743,7 +1779,7 @@ class OneToManyTest(_fixtures.FixtureTest):
         )
 
         m2 = mapper(Address, addresses)
-        m = mapper(
+        mapper(
             User,
             users,
             properties={
@@ -2128,7 +2164,7 @@ class SaveTest(_fixtures.FixtureTest):
 
         sa.orm.clear_mappers()
 
-        m = mapper(User, users)
+        mapper(User, users)
         evt = Events()
         event.listen(User, "before_insert", evt.before_insert)
         event.listen(User, "after_insert", evt.after_insert)
@@ -2150,8 +2186,7 @@ class ManyToOneTest(_fixtures.FixtureTest):
             self.classes.User,
         )
 
-        # TODO: put assertion in here !!!
-        m = mapper(
+        mapper(
             Address,
             addresses,
             properties=dict(
@@ -2229,7 +2264,7 @@ class ManyToOneTest(_fixtures.FixtureTest):
             self.classes.User,
         )
 
-        m = mapper(
+        mapper(
             Address,
             addresses,
             properties=dict(
@@ -2265,7 +2300,7 @@ class ManyToOneTest(_fixtures.FixtureTest):
             self.classes.User,
         )
 
-        m = mapper(
+        mapper(
             Address,
             addresses,
             properties=dict(
@@ -2307,7 +2342,7 @@ class ManyToOneTest(_fixtures.FixtureTest):
             self.classes.User,
         )
 
-        m = mapper(
+        mapper(
             Address,
             addresses,
             properties=dict(
@@ -2390,7 +2425,7 @@ class ManyToManyTest(_fixtures.FixtureTest):
 
         mapper(Keyword, keywords)
 
-        m = mapper(
+        mapper(
             Item,
             items,
             properties=dict(
@@ -2656,17 +2691,13 @@ class ManyToManyTest(_fixtures.FixtureTest):
 
         mapper(Keyword, keywords)
 
-        # note that we are breaking a rule here, making a second
-        # mapper(Keyword, keywords) the reorganization of mapper construction
-        # affected this, but was fixed again
-
         mapper(
             IKAssociation,
             item_keywords,
             primary_key=[item_keywords.c.item_id, item_keywords.c.keyword_id],
             properties=dict(
                 keyword=relationship(
-                    mapper(Keyword, keywords, non_primary=True),
+                    Keyword,
                     lazy="joined",
                     uselist=False,
                     # note here is a valid place where
@@ -3197,9 +3228,9 @@ class TransactionTest(fixtures.MappedTest):
 
     @classmethod
     def define_tables(cls, metadata):
-        t1 = Table("t1", metadata, Column("id", Integer, primary_key=True))
+        Table("t1", metadata, Column("id", Integer, primary_key=True))
 
-        t2 = Table(
+        Table(
             "t2",
             metadata,
             Column("id", Integer, primary_key=True),
@@ -3340,3 +3371,144 @@ class PartialNullPKTest(fixtures.MappedTest):
 
         t.col1 = "1"
         s.commit()
+
+
+class EnsurePKSortableTest(fixtures.MappedTest):
+    class SomeEnum(object):
+        # Implements PEP 435 in the minimal fashion needed by SQLAlchemy
+        __members__ = OrderedDict()
+
+        def __init__(self, name, value, alias=None):
+            self.name = name
+            self.value = value
+            self.__members__[name] = self
+            setattr(self.__class__, name, self)
+            if alias:
+                self.__members__[alias] = self
+                setattr(self.__class__, alias, self)
+
+    class MySortableEnum(SomeEnum):
+        __members__ = OrderedDict()
+
+        def __lt__(self, other):
+            return self.value < other.value
+
+    class MyNotSortableEnum(SomeEnum):
+        __members__ = OrderedDict()
+
+    one = MySortableEnum("one", 1)
+    two = MySortableEnum("two", 2)
+    three = MyNotSortableEnum("three", 3)
+    four = MyNotSortableEnum("four", 4)
+    five = MyNotSortableEnum("five", 5)
+
+    @classmethod
+    def define_tables(cls, metadata):
+        Table(
+            "t1",
+            metadata,
+            Column(
+                "id",
+                Enum(cls.MySortableEnum, create_constraint=False),
+                primary_key=True,
+            ),
+            Column("data", String(10)),
+        )
+
+        Table(
+            "t2",
+            metadata,
+            Column(
+                "id",
+                Enum(
+                    cls.MyNotSortableEnum,
+                    sort_key_function=None,
+                    create_constraint=False,
+                ),
+                primary_key=True,
+            ),
+            Column("data", String(10)),
+        )
+
+        Table(
+            "t3",
+            metadata,
+            Column(
+                "id",
+                Enum(cls.MyNotSortableEnum, create_constraint=False),
+                primary_key=True,
+            ),
+            Column("value", Integer),
+        )
+
+    @staticmethod
+    def sort_enum_key_value(value):
+        return value.value
+
+    @classmethod
+    def setup_classes(cls):
+        class T1(cls.Basic):
+            pass
+
+        class T2(cls.Basic):
+            pass
+
+        class T3(cls.Basic):
+            def __str__(self):
+                return "T3(id={})".format(self.id)
+
+    @classmethod
+    def setup_mappers(cls):
+        mapper(cls.classes.T1, cls.tables.t1)
+        mapper(cls.classes.T2, cls.tables.t2)
+        mapper(cls.classes.T3, cls.tables.t3)
+
+    def test_exception_persistent_flush_py3k(self):
+        s = Session()
+
+        a, b = self.classes.T2(id=self.three), self.classes.T2(id=self.four)
+        s.add_all([a, b])
+        s.commit()
+
+        a.data = "bar"
+        b.data = "foo"
+        if sa.util.py3k:
+            assert_raises_message(
+                sa.exc.InvalidRequestError,
+                r"Could not sort objects by primary key; primary key values "
+                r"must be sortable in Python \(was: '<' not supported between "
+                r"instances of 'MyNotSortableEnum' and 'MyNotSortableEnum'\)",
+                s.flush,
+            )
+        else:
+            s.flush()
+        s.close()
+
+    def test_persistent_flush_sortable(self):
+        s = Session()
+
+        a, b = self.classes.T1(id=self.one), self.classes.T1(id=self.two)
+        s.add_all([a, b])
+        s.commit()
+
+        a.data = "bar"
+        b.data = "foo"
+        s.commit()
+
+    def test_pep435_custom_sort_key(self):
+        s = Session()
+
+        a = self.classes.T3(id=self.three, value=1)
+        b = self.classes.T3(id=self.four, value=2)
+        s.add_all([a, b])
+        s.commit()
+
+        c = self.classes.T3(id=self.five, value=0)
+        s.add(c)
+
+        states = [o._sa_instance_state for o in [b, a, c]]
+        eq_(
+            _sort_states(inspect(self.classes.T3), states),
+            # pending come first, then "four" < "three"
+            [o._sa_instance_state for o in [c, b, a]],
+        )
