@@ -37,6 +37,7 @@ from sqlalchemy.testing import fixtures
 from sqlalchemy.testing import is_
 from sqlalchemy.testing import is_not_
 
+
 A = B = t1 = t2 = t3 = table1 = table2 = table3 = table4 = None
 
 
@@ -365,7 +366,7 @@ class ClauseTest(fixtures.TestBase, AssertsCompiledSQL):
         assert str(clause2) == str(t1.join(t2, t1.c.col2 == t2.c.col3))
 
     def test_aliased_column_adapt(self):
-        clause = t1.select()
+        t1.select()
 
         aliased = t1.select().alias()
         aliased2 = t1.alias()
@@ -499,8 +500,8 @@ class ClauseTest(fixtures.TestBase, AssertsCompiledSQL):
         )
 
     def test_text(self):
-        clause = text(
-            "select * from table where foo=:bar", bindparams=[bindparam("bar")]
+        clause = text("select * from table where foo=:bar").bindparams(
+            bindparam("bar")
         )
         c1 = str(clause)
 
@@ -1087,6 +1088,90 @@ class ClauseAdapterTest(fixtures.TestBase, AssertsCompiledSQL):
             "AS anon_1 FROM table1 AS t1alias",
         )
 
+    def test_correlate_except_on_clone(self):
+        # test [ticket:4537]'s issue
+
+        t1alias = t1.alias("t1alias")
+        j = t1.join(t1alias, t1.c.col1 == t1alias.c.col2)
+
+        vis = sql_util.ClauseAdapter(j)
+
+        # "control" subquery - uses correlate which has worked w/ adaption
+        # for a long time
+        control_s = (
+            select([t2.c.col1])
+            .where(t2.c.col1 == t1.c.col1)
+            .correlate(t2)
+            .as_scalar()
+        )
+
+        # test subquery - given only t1 and t2 in the enclosing selectable,
+        # will do the same thing as the "control" query since the correlation
+        # works out the same
+        s = (
+            select([t2.c.col1])
+            .where(t2.c.col1 == t1.c.col1)
+            .correlate_except(t1)
+            .as_scalar()
+        )
+
+        # use both subqueries in statements
+        control_stmt = select([control_s, t1.c.col1, t2.c.col1]).select_from(
+            t1.join(t2, t1.c.col1 == t2.c.col1)
+        )
+
+        stmt = select([s, t1.c.col1, t2.c.col1]).select_from(
+            t1.join(t2, t1.c.col1 == t2.c.col1)
+        )
+        # they are the same
+        self.assert_compile(
+            control_stmt,
+            "SELECT "
+            "(SELECT table2.col1 FROM table1 "
+            "WHERE table2.col1 = table1.col1) AS anon_1, "
+            "table1.col1, table2.col1 "
+            "FROM table1 "
+            "JOIN table2 ON table1.col1 = table2.col1",
+        )
+        self.assert_compile(
+            stmt,
+            "SELECT "
+            "(SELECT table2.col1 FROM table1 "
+            "WHERE table2.col1 = table1.col1) AS anon_1, "
+            "table1.col1, table2.col1 "
+            "FROM table1 "
+            "JOIN table2 ON table1.col1 = table2.col1",
+        )
+
+        # now test against the adaption of "t1" into "t1 JOIN t1alias".
+        # note in the control case, we aren't actually testing that
+        # Select is processing the "correlate" list during the adaption
+        # since we aren't adapting the "correlate"
+        self.assert_compile(
+            vis.traverse(control_stmt),
+            "SELECT "
+            "(SELECT table2.col1 FROM "
+            "table1 JOIN table1 AS t1alias ON table1.col1 = t1alias.col2 "
+            "WHERE table2.col1 = table1.col1) AS anon_1, "
+            "table1.col1, table2.col1 "
+            "FROM table1 JOIN table1 AS t1alias ON table1.col1 = t1alias.col2 "
+            "JOIN table2 ON table1.col1 = table2.col1",
+        )
+
+        # but here, correlate_except() does have the thing we're adapting
+        # so whatever is in there has to be expanded out to include
+        # the adaptation target, in this case "t1 JOIN t1alias".
+        self.assert_compile(
+            vis.traverse(stmt),
+            "SELECT "
+            "(SELECT table2.col1 FROM "
+            "table1 JOIN table1 AS t1alias ON table1.col1 = t1alias.col2 "
+            "WHERE table2.col1 = table1.col1) AS anon_1, "
+            "table1.col1, table2.col1 "
+            "FROM table1 JOIN table1 AS t1alias ON table1.col1 = t1alias.col2 "
+            "JOIN table2 ON table1.col1 = table2.col1",
+        )
+
     @testing.fails_on_everything_except()
     def test_joins_dont_adapt(self):
         # adapting to a join, i.e. ClauseAdapter(t1.join(t2)), doesn't
@@ -1128,14 +1213,6 @@ class ClauseAdapterTest(fixtures.TestBase, AssertsCompiledSQL):
         )
 
     def test_table_to_alias_3(self):
-        t1alias = t1.alias("t1alias")
-        vis = sql_util.ClauseAdapter(t1alias)
-        self.assert_compile(
-            select([literal_column("*")], t1.c.col1 == t2.c.col2),
-            "SELECT * FROM table1, table2 WHERE table1.col1 = table2.col2",
-        )
-
-    def test_table_to_alias_4(self):
         t1alias = t1.alias("t1alias")
         vis = sql_util.ClauseAdapter(t1alias)
         self.assert_compile(
@@ -1146,7 +1223,7 @@ class ClauseAdapterTest(fixtures.TestBase, AssertsCompiledSQL):
             "WHERE t1alias.col1 = table2.col2",
         )
 
-    def test_table_to_alias_5(self):
+    def test_table_to_alias_4(self):
         t1alias = t1.alias("t1alias")
         vis = sql_util.ClauseAdapter(t1alias)
         self.assert_compile(
@@ -1161,7 +1238,7 @@ class ClauseAdapterTest(fixtures.TestBase, AssertsCompiledSQL):
             "WHERE t1alias.col1 = table2.col2",
         )
 
-    def test_table_to_alias_6(self):
+    def test_table_to_alias_5(self):
         t1alias = t1.alias("t1alias")
         vis = sql_util.ClauseAdapter(t1alias)
         self.assert_compile(
@@ -1181,7 +1258,7 @@ class ClauseAdapterTest(fixtures.TestBase, AssertsCompiledSQL):
             "(SELECT * FROM table2 WHERE t1alias.col1 = table2.col2)",
         )
 
-    def test_table_to_alias_7(self):
+    def test_table_to_alias_6(self):
         t1alias = t1.alias("t1alias")
         vis = sql_util.ClauseAdapter(t1alias)
         self.assert_compile(
@@ -1203,7 +1280,7 @@ class ClauseAdapterTest(fixtures.TestBase, AssertsCompiledSQL):
             "WHERE t1alias.col1 = table2.col2)",
         )
 
-    def test_table_to_alias_8(self):
+    def test_table_to_alias_7(self):
         t1alias = t1.alias("t1alias")
         vis = sql_util.ClauseAdapter(t1alias)
         self.assert_compile(
@@ -1212,7 +1289,7 @@ class ClauseAdapterTest(fixtures.TestBase, AssertsCompiledSQL):
             "t1alias.col2 ELSE t1alias.col1 END",
         )
 
-    def test_table_to_alias_9(self):
+    def test_table_to_alias_8(self):
         t1alias = t1.alias("t1alias")
         vis = sql_util.ClauseAdapter(t1alias)
         self.assert_compile(
@@ -1223,13 +1300,13 @@ class ClauseAdapterTest(fixtures.TestBase, AssertsCompiledSQL):
             "t1alias.col2 ELSE t1alias.col1 END",
         )
 
-    def test_table_to_alias_10(self):
+    def test_table_to_alias_9(self):
         s = select([literal_column("*")], from_obj=[t1]).alias("foo")
         self.assert_compile(
             s.select(), "SELECT foo.* FROM (SELECT * FROM table1) " "AS foo"
         )
 
-    def test_table_to_alias_11(self):
+    def test_table_to_alias_10(self):
         s = select([literal_column("*")], from_obj=[t1]).alias("foo")
         t1alias = t1.alias("t1alias")
         vis = sql_util.ClauseAdapter(t1alias)
@@ -1238,13 +1315,13 @@ class ClauseAdapterTest(fixtures.TestBase, AssertsCompiledSQL):
             "SELECT foo.* FROM (SELECT * FROM table1 " "AS t1alias) AS foo",
         )
 
-    def test_table_to_alias_12(self):
+    def test_table_to_alias_11(self):
         s = select([literal_column("*")], from_obj=[t1]).alias("foo")
         self.assert_compile(
             s.select(), "SELECT foo.* FROM (SELECT * FROM table1) " "AS foo"
         )
 
-    def test_table_to_alias_13(self):
+    def test_table_to_alias_12(self):
         t1alias = t1.alias("t1alias")
         vis = sql_util.ClauseAdapter(t1alias)
         ff = vis.traverse(func.count(t1.c.col1).label("foo"))
@@ -1259,7 +1336,7 @@ class ClauseAdapterTest(fixtures.TestBase, AssertsCompiledSQL):
     # .col1).l abel('foo')]), clone=True), "SELECT
     # count(t1alias.col1) AS foo FROM table1 AS t1alias")
 
-    def test_table_to_alias_14(self):
+    def test_table_to_alias_13(self):
         t1alias = t1.alias("t1alias")
         vis = sql_util.ClauseAdapter(t1alias)
         t2alias = t2.alias("t2alias")
@@ -1273,7 +1350,7 @@ class ClauseAdapterTest(fixtures.TestBase, AssertsCompiledSQL):
             "t2alias.col2",
         )
 
-    def test_table_to_alias_15(self):
+    def test_table_to_alias_14(self):
         t1alias = t1.alias("t1alias")
         vis = sql_util.ClauseAdapter(t1alias)
         t2alias = t2.alias("t2alias")
@@ -1287,7 +1364,7 @@ class ClauseAdapterTest(fixtures.TestBase, AssertsCompiledSQL):
             "t2alias.col2",
         )
 
-    def test_table_to_alias_16(self):
+    def test_table_to_alias_15(self):
         t1alias = t1.alias("t1alias")
         vis = sql_util.ClauseAdapter(t1alias)
         t2alias = t2.alias("t2alias")
@@ -1309,7 +1386,7 @@ class ClauseAdapterTest(fixtures.TestBase, AssertsCompiledSQL):
             "WHERE t1alias.col1 = t2alias.col2)",
         )
 
-    def test_table_to_alias_17(self):
+    def test_table_to_alias_16(self):
         t1alias = t1.alias("t1alias")
         vis = sql_util.ClauseAdapter(t1alias)
         t2alias = t2.alias("t2alias")
@@ -1558,9 +1635,6 @@ class ClauseAdapterTest(fixtures.TestBase, AssertsCompiledSQL):
             "WHERE c.bid = anon_1.b_aid",
         )
 
-        t1 = table("table1", column("col1"), column("col2"), column("col3"))
-        t2 = table("table2", column("col1"), column("col2"), column("col3"))
-
     def test_label_anonymize_one(self):
         t1a = t1.alias()
         adapter = sql_util.ClauseAdapter(t1a, anonymize_labels=True)
@@ -1768,10 +1842,10 @@ class SelectTest(fixtures.TestBase, AssertsCompiledSQL):
         s2 = s.execution_options(bar="baz")
         s3 = s.execution_options(foo="not bar")
         # The original select should not be modified.
-        assert s._execution_options == dict(foo="bar")
+        eq_(s.get_execution_options(), dict(foo="bar"))
         # s2 should have its execution_options based on s, though.
-        assert s2._execution_options == dict(foo="bar", bar="baz")
-        assert s3._execution_options == dict(foo="not bar")
+        eq_(s2.get_execution_options(), dict(foo="bar", bar="baz"))
+        eq_(s3.get_execution_options(), dict(foo="not bar"))
 
     def test_invalid_options(self):
         assert_raises(
