@@ -34,6 +34,7 @@ from sqlalchemy.testing import is_
 from sqlalchemy.testing.schema import Column
 from sqlalchemy.testing.schema import Table
 
+
 # ongoing - these are old tests.  those which are of general use
 # to test a dialect are being slowly migrated to
 # sqlalhcemy.testing.suite
@@ -591,13 +592,7 @@ class QueryTest(fixtures.TestBase):
                 [(7, "jack"), (8, "fred")],
             )
 
-            assert_raises_message(
-                exc.StatementError,
-                "'expanding' parameters with an empty list not supported",
-                conn.execute,
-                stmt,
-                {"uname": []},
-            )
+            eq_(conn.execute(stmt, {"uname": []}).fetchall(), [])
 
             assert_raises_message(
                 exc.StatementError,
