@@ -28,6 +28,7 @@ from sqlalchemy import type_coerce
 from sqlalchemy import TypeDecorator
 from sqlalchemy import union
 from sqlalchemy import util
+from sqlalchemy.sql import Alias
 from sqlalchemy.sql import column
 from sqlalchemy.sql import elements
 from sqlalchemy.sql import expression
@@ -235,7 +236,6 @@ class SelectableTest(
         s1c1 = s1._clone()
         s1c2 = s1._clone()
         s2c1 = s2._clone()
-        s2c2 = s2._clone()
         s3c1 = s3._clone()
 
         eq_(
@@ -274,6 +274,28 @@ class SelectableTest(
         cloned.append_column(func.foo())
         eq_(list(cloned.c.keys()), ["a", "b", "foo()"])
 
+    def test_clone_col_list_changes_then_proxy(self):
+        t = table("t", column("q"), column("p"))
+        stmt = select([t.c.q]).alias()
+
+        def add_column(stmt):
+            stmt.append_column(t.c.p)
+
+        stmt2 = visitors.cloned_traverse(stmt, {}, {"select": add_column})
+        eq_(list(stmt.c.keys()), ["q"])
+        eq_(list(stmt2.c.keys()), ["q", "p"])
+
+    def test_clone_col_list_changes_then_schema_proxy(self):
+        t = Table("t", MetaData(), Column("q", Integer), Column("p", Integer))
+        stmt = select([t.c.q]).alias()
+
+        def add_column(stmt):
+            stmt.append_column(t.c.p)
+
+        stmt2 = visitors.cloned_traverse(stmt, {}, {"select": add_column})
+        eq_(list(stmt.c.keys()), ["q"])
+        eq_(list(stmt2.c.keys()), ["q", "p"])
+
     def test_append_column_after_replace_selectable(self):
         basesel = select([literal_column("1").label("a")])
         tojoin = select(
@@ -428,6 +450,18 @@ class SelectableTest(
         assert u1.corresponding_column(table1.c.colx) is u1.c.col2
         assert u1.corresponding_column(table1.c.col3) is u1.c.col1
 
+    def test_proxy_set_pollution(self):
+        s1 = select([table1.c.col1, table1.c.col2])
+        s2 = select([table1.c.col2, table1.c.col1])
+
+        for c in s1.c:
+            c.proxy_set
+        for c in s2.c:
+            c.proxy_set
+
+        u1 = union(s1, s2)
+        assert u1.corresponding_column(table1.c.col2) is u1.c.col2
+
     def test_singular_union(self):
         u = union(
             select([table1.c.col1, table1.c.col2, table1.c.col3]),
@@ -652,7 +686,7 @@ class SelectableTest(
 
     def test_scalar_cloned_comparator(self):
         sel = select([table1.c.col1]).as_scalar()
-        expr = sel == table1.c.col1
+        sel == table1.c.col1
 
         sel2 = visitors.ReplacingCloningVisitor().traverse(sel)
 
@@ -767,7 +801,7 @@ class SelectableTest(
         c1 = Column("c1", Integer)
         c2 = Column("c2", Integer)
 
-        s = select([c1])
+        select([c1])
 
         t = Table("t", MetaData(), c1, c2)
 
@@ -781,7 +815,7 @@ class SelectableTest(
         c1 = Column("c1", Integer)
         c2 = Column("c2", Integer)
 
-        s = select([c1]).where(c1 == 5)
+        select([c1]).where(c1 == 5)
 
         t = Table("t", MetaData(), c1, c2)
 
@@ -796,7 +830,7 @@ class SelectableTest(
         t1 = Table("t1", m, Column("x", Integer))
 
         c1 = Column("c1", Integer)
-        s = select([c1]).where(c1 == 5).select_from(t1)
+        select([c1]).where(c1 == 5).select_from(t1)
 
         t2 = Table("t2", MetaData(), c1)
 
@@ -871,6 +905,18 @@ class SelectableTest(
         Table("t1", MetaData(), c1)
         eq_(c1._label, "t1_c1")
 
+    def test_no_alias_construct(self):
+        a = table("a", column("x"))
+
+        assert_raises_message(
+            NotImplementedError,
+            "The Alias class is not intended to be constructed directly.  "
+            r"Please use the alias\(\) standalone function",
+            Alias,
+            a,
+            "foo",
+        )
+
 
 class RefreshForNewColTest(fixtures.TestBase):
     def test_join_uninit(self):
@@ -1060,7 +1106,7 @@ class AnonLabelTest(fixtures.TestBase):
         assert c1.label(None) is not c1
 
         eq_(str(select([c1])), "SELECT count(:count_2) AS count_1")
-        c2 = select([c1]).compile()
+        select([c1]).compile()
 
         eq_(str(select([c1.label(None)])), "SELECT count(:count_2) AS count_1")
 
@@ -1903,6 +1949,43 @@ class AnnotationsTest(fixtures.TestBase):
         assert x_p.compare(x_p_a)
         assert not x_p_a.compare(x_a)
 
+    def test_proxy_set_iteration_includes_annotated(self):
+        from sqlalchemy.schema import Column
+
+        c1 = Column("foo", Integer)
+
+        stmt = select([c1]).alias()
+        proxy = stmt.c.foo
+
+        proxy.proxy_set
+
+        # create an annotated of the column
+        p2 = proxy._annotate({"weight": 10})
+
+        # now see if our annotated version is in that column's
+        # proxy_set, as corresponding_column iterates through proxy_set
+        # in this way
+        d = {}
+        for col in p2._uncached_proxy_set():
+            d.update(col._annotations)
+        eq_(d, {"weight": 10})
+
+    def test_proxy_set_iteration_includes_annotated_two(self):
+        from sqlalchemy.schema import Column
+
+        c1 = Column("foo", Integer)
+
+        stmt = select([c1]).alias()
+        proxy = stmt.c.foo
+        c1.proxy_set
+
+        proxy._proxies = [c1._annotate({"weight": 10})]
+
+        d = {}
+        for col in proxy._uncached_proxy_set():
+            d.update(col._annotations)
+        eq_(d, {"weight": 10})
+
     def test_late_name_add(self):
         from sqlalchemy.schema import Column
 
@@ -1974,7 +2057,7 @@ class AnnotationsTest(fixtures.TestBase):
             pass
 
         col = MyColumn("x", Integer)
-        binary_1 = col == 5
+        col == 5
         col_anno = MyColumn("x", Integer)._annotate({"foo": "bar"})
         binary_2 = col_anno == 5
         eq_(binary_2.left._annotations, {"foo": "bar"})
@@ -2241,10 +2324,10 @@ class AnnotationsTest(fixtures.TestBase):
         c = column("a")
 
         c1 = c._annotate({"foo": "bar"})
-        comp1 = c1.comparator
+        c1.comparator
 
         c2 = c1._annotate({"bat": "hoho"})
-        comp2 = c2.comparator
+        c2.comparator
 
         assert (c2 == 5).left._annotations == {"foo": "bar", "bat": "hoho"}
 
@@ -2457,7 +2540,7 @@ class ResultMapTest(fixtures.TestBase):
 
     def test_select_table_alias_column(self):
         t = self._fixture()
-        x, y = t.c.x, t.c.y
+        x = t.c.x
 
         ta = t.alias()
         s = select([ta.c.x, ta.c.y])
@@ -2466,7 +2549,7 @@ class ResultMapTest(fixtures.TestBase):
 
     def test_select_label_alt_name_table_alias_column(self):
         t = self._fixture()
-        x, y = t.c.x, t.c.y
+        x = t.c.x
 
         ta = t.alias()
         l1, l2 = ta.c.x.label("a"), ta.c.y.label("b")
@@ -2544,39 +2627,6 @@ class ResultMapTest(fixtures.TestBase):
 class ForUpdateTest(fixtures.TestBase, AssertsCompiledSQL):
     __dialect__ = "default"
 
-    def _assert_legacy(self, leg, read=False, nowait=False):
-        t = table("t", column("c"))
-        s1 = select([t], for_update=leg)
-
-        if leg is False:
-            assert s1._for_update_arg is None
-            assert s1.for_update is None
-        else:
-            eq_(s1._for_update_arg.read, read)
-            eq_(s1._for_update_arg.nowait, nowait)
-            eq_(s1.for_update, leg)
-
-    def test_false_legacy(self):
-        self._assert_legacy(False)
-
-    def test_plain_true_legacy(self):
-        self._assert_legacy(True)
-
-    def test_read_legacy(self):
-        self._assert_legacy("read", read=True)
-
-    def test_nowait_legacy(self):
-        self._assert_legacy("nowait", nowait=True)
-
-    def test_read_nowait_legacy(self):
-        self._assert_legacy("read_nowait", read=True, nowait=True)
-
-    def test_legacy_setter(self):
-        t = table("t", column("c"))
-        s = select([t])
-        s.for_update = "nowait"
-        eq_(s._for_update_arg.nowait, True)
-
     def test_basic_clone(self):
         t = table("t", column("c"))
         s = select([t]).with_for_update(read=True, of=t.c.c)
