@@ -19,13 +19,12 @@ from sqlalchemy import text
 from sqlalchemy import union
 from sqlalchemy import util
 from sqlalchemy.sql import column
+from sqlalchemy.sql import quoted_name
 from sqlalchemy.sql import table
 from sqlalchemy.sql import util as sql_util
 from sqlalchemy.testing import assert_raises_message
-from sqlalchemy.testing import assert_warnings
 from sqlalchemy.testing import AssertsCompiledSQL
 from sqlalchemy.testing import eq_
-from sqlalchemy.testing import expect_warnings
 from sqlalchemy.testing import fixtures
 from sqlalchemy.types import NullType
 
@@ -137,7 +136,9 @@ class SelectCompositionTest(fixtures.TestBase, AssertsCompiledSQL):
     def test_select_composition_six(self):
         # test that "auto-labeling of subquery columns"
         # doesn't interfere with literal columns,
-        # exported columns don't get quoted
+        # exported columns don't get quoted.
+        # [ticket:4730] refines this but for the moment the behavior with
+        # no columns is being maintained.
         self.assert_compile(
             select(
                 [
@@ -197,18 +198,6 @@ class SelectCompositionTest(fixtures.TestBase, AssertsCompiledSQL):
 class BindParamTest(fixtures.TestBase, AssertsCompiledSQL):
     __dialect__ = "default"
 
-    def test_legacy(self):
-        t = text(
-            "select * from foo where lala=:bar and hoho=:whee",
-            bindparams=[bindparam("bar", 4), bindparam("whee", 7)],
-        )
-
-        self.assert_compile(
-            t,
-            "select * from foo where lala=:bar and hoho=:whee",
-            checkparams={"bar": 4, "whee": 7},
-        )
-
     def test_positional(self):
         t = text("select * from foo where lala=:bar and hoho=:whee")
         t = t.bindparams(bindparam("bar", 4), bindparam("whee", 7))
@@ -290,6 +279,19 @@ class BindParamTest(fixtures.TestBase, AssertsCompiledSQL):
             dialect="postgresql",
         )
 
+    def test_unique_binds(self):
+        # unique binds can be used in text() however they uniquify across
+        # multiple text() constructs only, not within a single text
+
+        t1 = text("select :foo").bindparams(bindparam("foo", 5, unique=True))
+        t2 = text("select :foo").bindparams(bindparam("foo", 10, unique=True))
+        stmt = select([t1, t2])
+        self.assert_compile(
+            stmt,
+            "SELECT select :foo_1, select :foo_2",
+            checkparams={"foo_1": 5, "foo_2": 10},
+        )
+
     def test_binds_compiled_positional(self):
         self.assert_compile(
             text(
@@ -344,6 +346,19 @@ class BindParamTest(fixtures.TestBase, AssertsCompiledSQL):
             dialect="postgresql",
         )
 
+    def test_double_colons_dont_actually_need_escaping(self):
+        # this is news to me. bound param won't work but you can put the
+        # double colons in
+        self.assert_compile(
+            text(
+                r"SELECT * FROM pg_attribute WHERE "
+                r"attrelid = foo::regclass"
+            ),
+            "SELECT * FROM pg_attribute WHERE " "attrelid = foo::regclass",
+            params={},
+            dialect="postgresql",
+        )
+
     def test_text_in_select_nonfrom(self):
 
         generate_series = text(
@@ -512,7 +527,7 @@ class AsFromTest(fixtures.TestBase, AssertsCompiledSQL):
 
     def test_select_table_alias_column(self):
         t = self._xy_table_fixture()
-        x, y = t.c.x, t.c.y
+        x = t.c.x
 
         ta = t.alias()
         s = text("select ta.x, ta.y FROM t AS ta").columns(ta.c.x, ta.c.y)
@@ -521,7 +536,7 @@ class AsFromTest(fixtures.TestBase, AssertsCompiledSQL):
 
     def test_select_label_alt_name_table_alias_column(self):
         t = self._xy_table_fixture()
-        x, y = t.c.x, t.c.y
+        x = t.c.x
 
         ta = t.alias()
         l1, l2 = ta.c.x.label("a"), ta.c.y.label("b")
@@ -585,16 +600,12 @@ class AsFromTest(fixtures.TestBase, AssertsCompiledSQL):
         eq_(set(t.element._bindparams), set(["bat", "foo", "bar"]))
 
 
-class TextWarningsTest(fixtures.TestBase, AssertsCompiledSQL):
+class TextErrorsTest(fixtures.TestBase, AssertsCompiledSQL):
     __dialect__ = "default"
 
-    def _test(self, fn, arg, offending_clause, expected):
-        with expect_warnings("Textual "):
-            stmt = fn(arg)
-            self.assert_compile(stmt, expected)
-
+    def _test(self, fn, arg, offending_clause):
         assert_raises_message(
-            exc.SAWarning,
+            exc.ArgumentError,
             r"Textual (?:SQL|column|SQL FROM) expression %(stmt)r should be "
             r"explicitly declared (?:with|as) text\(%(stmt)r\)"
             % {"stmt": util.ellipses_string(offending_clause)},
@@ -603,46 +614,31 @@ class TextWarningsTest(fixtures.TestBase, AssertsCompiledSQL):
         )
 
     def test_where(self):
-        self._test(
-            select([table1.c.myid]).where,
-            "myid == 5",
-            "myid == 5",
-            "SELECT mytable.myid FROM mytable WHERE myid == 5",
-        )
+        self._test(select([table1.c.myid]).where, "myid == 5", "myid == 5")
 
     def test_column(self):
-        self._test(select, ["myid"], "myid", "SELECT myid")
+        self._test(select, ["myid"], "myid")
 
     def test_having(self):
-        self._test(
-            select([table1.c.myid]).having,
-            "myid == 5",
-            "myid == 5",
-            "SELECT mytable.myid FROM mytable HAVING myid == 5",
-        )
+        self._test(select([table1.c.myid]).having, "myid == 5", "myid == 5")
 
     def test_from(self):
-        self._test(
-            select([table1.c.myid]).select_from,
-            "mytable",
-            "mytable",
-            "SELECT mytable.myid FROM mytable, mytable",  # two FROMs
-        )
+        self._test(select([table1.c.myid]).select_from, "mytable", "mytable")
 
 
 class OrderByLabelResolutionTest(fixtures.TestBase, AssertsCompiledSQL):
     __dialect__ = "default"
 
-    def _test_warning(self, stmt, offending_clause, expected):
-        with expect_warnings(
-            "Can't resolve label reference %r;" % offending_clause
-        ):
-            self.assert_compile(stmt, expected)
+    def _test_exception(self, stmt, offending_clause, dialect=None):
         assert_raises_message(
-            exc.SAWarning,
-            "Can't resolve label reference %r; converting to text"
-            % offending_clause,
+            exc.CompileError,
+            r"Can't resolve label reference for ORDER BY / GROUP BY / "
+            "DISTINCT etc. "
+            "Textual SQL "
+            "expression %r should be explicitly "
+            r"declared as text\(%r\)" % (offending_clause, offending_clause),
             stmt.compile,
+            dialect=dialect,
         )
 
     def test_order_by_label(self):
@@ -691,10 +687,32 @@ class OrderByLabelResolutionTest(fixtures.TestBase, AssertsCompiledSQL):
 
     def test_unresolvable_warning_order_by(self):
         stmt = select([table1.c.myid]).order_by("foobar")
-        self._test_warning(
-            stmt, "foobar", "SELECT mytable.myid FROM mytable ORDER BY foobar"
+        self._test_exception(stmt, "foobar")
+
+    def test_distinct_label(self):
+
+        stmt = select([table1.c.myid.label("foo")]).distinct("foo")
+        self.assert_compile(
+            stmt,
+            "SELECT DISTINCT ON (foo) mytable.myid AS foo FROM mytable",
+            dialect="postgresql",
         )
 
+    def test_distinct_label_keyword(self):
+
+        stmt = select([table1.c.myid.label("foo")], distinct="foo")
+        self.assert_compile(
+            stmt,
+            "SELECT DISTINCT ON (foo) mytable.myid AS foo FROM mytable",
+            dialect="postgresql",
+        )
+
+    def test_unresolvable_distinct_label(self):
+        from sqlalchemy.dialects import postgresql
+
+        stmt = select([table1.c.myid.label("foo")]).distinct("not a label")
+        self._test_exception(stmt, "not a label", dialect=postgresql.dialect())
+
     def test_group_by_label(self):
         stmt = select([table1.c.myid.label("foo")]).group_by("foo")
         self.assert_compile(
@@ -709,9 +727,7 @@ class OrderByLabelResolutionTest(fixtures.TestBase, AssertsCompiledSQL):
 
     def test_unresolvable_warning_group_by(self):
         stmt = select([table1.c.myid]).group_by("foobar")
-        self._test_warning(
-            stmt, "foobar", "SELECT mytable.myid FROM mytable GROUP BY foobar"
-        )
+        self._test_exception(stmt, "foobar")
 
     def test_asc(self):
         stmt = select([table1.c.myid]).order_by(asc("name"), "description")
@@ -733,6 +749,44 @@ class OrderByLabelResolutionTest(fixtures.TestBase, AssertsCompiledSQL):
             "GROUP BY anon_1.myid",
         )
 
+    def test_order_by_literal_col_quoting_one(self):
+        col = literal_column("SUM(ABC)").label("SUM(ABC)")
+        tbl = table("my_table")
+        query = select([col]).select_from(tbl).order_by(col)
+        self.assert_compile(
+            query,
+            'SELECT SUM(ABC) AS "SUM(ABC)" FROM my_table ORDER BY "SUM(ABC)"',
+        )
+
+    def test_order_by_literal_col_quoting_two(self):
+        col = literal_column("SUM(ABC)").label("SUM(ABC)_")
+        tbl = table("my_table")
+        query = select([col]).select_from(tbl).order_by(col)
+        self.assert_compile(
+            query,
+            'SELECT SUM(ABC) AS "SUM(ABC)_" FROM my_table ORDER BY '
+            '"SUM(ABC)_"',
+        )
+
+    def test_order_by_literal_col_quoting_one_explict_quote(self):
+        col = literal_column("SUM(ABC)").label(quoted_name("SUM(ABC)", True))
+        tbl = table("my_table")
+        query = select([col]).select_from(tbl).order_by(col)
+        self.assert_compile(
+            query,
+            'SELECT SUM(ABC) AS "SUM(ABC)" FROM my_table ORDER BY "SUM(ABC)"',
+        )
+
+    def test_order_by_literal_col_quoting_two_explicit_quote(self):
+        col = literal_column("SUM(ABC)").label(quoted_name("SUM(ABC)_", True))
+        tbl = table("my_table")
+        query = select([col]).select_from(tbl).order_by(col)
+        self.assert_compile(
+            query,
+            'SELECT SUM(ABC) AS "SUM(ABC)_" FROM my_table ORDER BY '
+            '"SUM(ABC)_"',
+        )
+
     def test_order_by_func_label_desc(self):
         stmt = select([func.foo("bar").label("fb"), table1]).order_by(
             desc("fb")
@@ -810,23 +864,14 @@ class OrderByLabelResolutionTest(fixtures.TestBase, AssertsCompiledSQL):
             .order_by("myid", "t1name", "x")
         )
 
-        def go():
-            # the labels here are anonymized, so label naming
-            # can't catch these.
-            self.assert_compile(
-                s1,
-                "SELECT mytable_1.myid AS mytable_1_myid, "
-                "mytable_1.name AS name_1, foo(:foo_2) AS foo_1 "
-                "FROM mytable AS mytable_1 ORDER BY mytable_1.myid, t1name, x",
-            )
-
-        assert_warnings(
-            go,
-            [
-                "Can't resolve label reference 't1name'",
-                "Can't resolve label reference 'x'",
-            ],
-            regex=True,
+        assert_raises_message(
+            exc.CompileError,
+            r"Can't resolve label reference for ORDER BY / GROUP BY / "
+            "DISTINCT etc. "
+            "Textual SQL "
+            "expression 't1name' should be explicitly "
+            r"declared as text\('t1name'\)",
+            s1.compile,
         )
 
     def test_columnadapter_non_anonymized(self):
